/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xkra.h"

FD_xkra *fdui;
extern FD_xkra *fd_main_form;

struct fund
	{
	char *name;
	float strike_price;
	int month;
	int year;
	int type;
	float current_option_price;
	int quantity;
	float days_to_expiration;
	float current_stock_price;
	float current_call_value;
	float current_put_value;
	float percentage;
	int browser_line;
		
	struct fund *nxtentr;
	struct fund *prventr;
	};
struct fund *fundtab[2];


struct fund *lookup_fund(char *name)
{
struct fund *pa;

if(debug_flag) 
	{
	fprintf(stdout, "lookup_fund(): arg name=%s\n", name);
	}

for(pa = fundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return(0); /* not found */
}


struct fund *install_fund_at_end_of_list(char *name)
{
struct fund *plast, *pnew;
struct fund *lookup_fund();

if(debug_flag)
	{
	fprintf(stdout, "install_fund_at_end_of_list(): arg name=%s\n", name);
	}

pnew = lookup_fund(name);
if(pnew)
	{
	/* already there */
	if(debug_flag)
		{
		fprintf(stdout,\
		"install_fund_at_end_of_list(): already there ignored\n");
		}
	return(pnew);/* allow double entries in file, return old address */
	}

/* create new structure */
pnew = (struct fund *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = fundtab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !fundtab[0] ) fundtab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
fundtab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_fund_at_end_of_list */


int delete_fund(char *name)/* delete entry from double linked list */
{
struct fund *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stdout, "delete_fund(): arg name=%s\n", name);
	}

/* parameter check */
if(! name) return(0);

pa = fundtab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return(0);

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify fundtab[0] */
	if(pprev == 0) fundtab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify fundtab[1] */
	if(pnext == 0) fundtab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	/* delete structure data */	
	free(pdel -> name);/* free name */
	free(pdel); /* free structure */

	/* return OK deleted */
	return(1);
	}/* end for all structures */
}/* end function delete_fund */


int delete_all_funds()/* delete all entries from table */
{
struct fund *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_funds(): arg none\n");
	}

while(1)
	{
	pa = fundtab[0];
	if(! pa) break;
	fundtab[0] = pa -> nxtentr;/* fundtab entry points to next one,
												            this could be 0 */
	free(pa -> name);/* free name */

	free(pa);/* free structure */
	}/* end for all structures */
fundtab[1] = 0;

return 1;
}/* end function delete_all_funds */


int add_option_to_database(\
	char *option_name, float strike_price, int type, int month, int year,\
	float current_option_price, int quantity, float days_to_expiration,\
	float current_stock_price,\
	float current_call_value, float current_put_value, float percentage)
{
struct fund *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_option_to_database(): arg\n\
	option_name=%s\n\
	strike_price=%.2f\n\
	type=%d\n\
	month=%d\n\
	year=%d\n\
	current_option_price=%.2f\n\
	quantity=%d\n\
	days_to_expiration=%.2f\n\
	current_stock_price=%.2f\n\
	current_call_value=%.2f\n\
	current_put_value=%.2f\n\
	percentage=%.2f\n",\
	option_name, strike_price, type, month, year, current_option_price,\
	quantity, days_to_expiration, current_stock_price,\
	current_call_value, current_put_value, percentage);
	}

if(! option_name) return(0);

pa = install_fund_at_end_of_list(option_name);
if(! pa)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"add_option_to_database(): cannot install fund %s\n", option_name);	
		
		return(0);
		}
	
	fl_show_alert("cannot install fund", option_name, "", 1);
	}/* end if ! pa */

/*
pa -> name = strsave(option_name);
if(! pa -> name)
	{
	fprintf(stdout, "add_option_to_database(): malloc failed\n");
	return(0);
	}
*/
pa -> strike_price = strike_price;
pa -> month = month;
pa -> year = year;
pa -> type = type;
pa -> current_option_price = current_option_price;
pa -> quantity = quantity;
pa -> days_to_expiration = days_to_expiration;
pa -> current_stock_price = current_stock_price;
pa -> current_call_value = current_call_value;
pa -> current_put_value = current_put_value;
pa -> percentage = percentage;
pa -> browser_line =0;

return(1);
}/* end function add_option_to_database */


int get_option_from_database(\
 char *option_name, float *strike_price, int *type, int *month, int *year,\
 float *current_option_price, int *quantity, float *days_to_expiration,\
 float *current_call_value, float *current_put_value, float *percentage)
{
int c, i;
struct fund *pa;
char option_name_uppercase[READSIZE];

i = 0;
while(1)
	{
	c = option_name[i];
	option_name_uppercase[i] = toupper(c);
	if(c == 0) break;
	i++;
	}

pa = lookup_fund(option_name_uppercase);
if(! pa) return(0);

*strike_price = pa -> strike_price;
*type = pa -> type;
*month = pa -> month;
*year = pa -> year;
*current_option_price = pa -> current_option_price;
*quantity = pa -> quantity;
*days_to_expiration = pa -> days_to_expiration;
*current_call_value = pa -> current_call_value;
*current_put_value = pa -> current_put_value;
*percentage = pa -> percentage;

return(1);
}/* end function get_option_from_database */


int swap_position(struct fund *ptop, struct fund *pbottom)
{
struct fund *punder;
struct fund *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): swapping top=%lu bottom=%lu\n", ptop, pbottom);
	}

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): punder=%lu\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): pabove=%lu\n", pabove);
	}

/* the next pointer in punder (or fundtab[0]) must now point to ptop */
if(! punder)
	{
	fundtab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in fundtab */
if(! pabove)
	{
	fundtab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function  option_swap */


int sort_options()
/*
sorts the double linked list with as criterium that the cheapest goes
on top.
The cheapest is the one that has the lowest
current_option_price / current_option_value (for call or put).
doing some sort of bubble sort.
*/
{
struct fund *pa;
struct fund *pb;
float percentage_this_one;
float percentage_previous_one;
int swap_flag;

while(1)/* go through list again and again */
	{
	if(debug_flag)
		{
		fprintf(stdout, "CHECKING LIST\n");
		for(pa = fundtab[0]; pa != 0; pa = pa -> nxtentr)
			{
			fprintf(stdout, "PA=%lu\n\
			pa->prventr=%lu\n\
			pa->nxtentr=%lu\n\
			pa->percentage=%.2f\n",\
			pa,\
			pa -> prventr,\
			pa -> nxtentr,\
			pa -> percentage);
			}
	
		fprintf(stdout, "ENTERING FOR\n");
		fprintf(stdout, "fundtab[0]=%lu fundtab[1]=%lu\n", fundtab[0], fundtab[1]);
		}/* end if debug_flag */
		
	swap_flag = 0;
	for(pa = fundtab[0]; pa != 0; pa = pa -> nxtentr)
		{
		if(debug_flag)
			{
			fprintf(stdout, "sorting %s pa=%lu\n",\
			pa -> name, pa);		
			}	
	
		/* get percentage this one */
		percentage_this_one = pa -> percentage;
		if(debug_flag)
			{
			fprintf(stdout,\
			"percentage_this_one=%.2f\n", percentage_this_one);	
			}
			
		pb = pa -> prventr;
		if(debug_flag)
			{
			fprintf(stdout, "pb=pa->prventr=%lu\n", pb);
			}
		
		if(pb)
			{
			/* get percentage previous one */
			percentage_previous_one = pb -> percentage;

			if(debug_flag)
				{
				fprintf(stdout,\
				"percentage_previous_one=%.2f\n", percentage_previous_one);	
				}
			}
		else
			{
			if(debug_flag)
				{
				fprintf(stdout, "no previous one \n");			
				}
			}
	
		if(pb)
			{
			if(percentage_this_one < percentage_previous_one)
				{
				if(debug_flag)
					{
					fprintf(stdout,\
					"BEFORE SWAP pa->prventr=%lu pa->nxtentr=%lu\n\
					pb->prventr=%lu pb-nxtentrr=%lu\n",\
					pa -> prventr, pa -> nxtentr,\
					pb -> prventr, pb -> nxtentr);
					}

				swap_flag = swap_position(pa , pb);
				/* indicate position was swapped */
				if(debug_flag)
					{
					fprintf(stdout, "swap_flag=%d\n", swap_flag);
					fprintf(stdout,\
					"AFTER SWAP pa->prventr=%lu pa->nxtentr=%lu\n\
					pb->prventr=%lu pb-nxtentrr=%lu\n",\
					pa -> prventr, pa -> nxtentr,\
					pb -> prventr, pb -> nxtentr);		
					}										
				}/* end if percentage < */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_options */


int print_sorted_option_list()
{
int i;

struct fund *pa;
char fundstr[20];
char *monthptr;

for(pa = fundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/*
	aab call oct 1999 25.00  price=1.50 value=1.70 88.2% days=1.22 stock=22.10
	*/ 

	/* extract fund name */
	i = 0;
	while(1)
		{
		fundstr[i] = pa -> name[i];
		if(pa -> name[i] == '/')
			{
			fundstr[i] = 0;
			break;
			}
		if(pa -> name[i] == 0) break;
		i++;
		}/* end while all chars */

	monthptr = digit_to_month(pa -> month);	
	if(! monthptr)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"print_sorted_option_list(): digit_to_month() failed\n");
			}
		/* this should really never happen */
		return 0;
		}

	if(pa -> type == TYPE_CALL)
		{
		fprintf(stdout,\
		"%s call %s %d %.2f  \
price=%.2f value=%.2f %.1f%s days=%.2f stock=%.2f quant=%d\n",\
		fundstr, monthptr, pa -> year, pa -> strike_price,\
		pa -> current_option_price, pa -> current_call_value,\
		pa -> percentage, "%", pa -> days_to_expiration,\
		pa -> current_stock_price, pa -> quantity);

		free(monthptr);
		}/* end if call */
	if(pa -> type == TYPE_PUT)
		{
		fprintf(stdout,\
		"%s put %s %d %.2f  \
price=%.2f value=%.2f %.1f%s days=%.2f stock=%.2f quant=%d\n",\
		fundstr, monthptr, pa -> year, pa -> strike_price,\
		pa -> current_option_price, pa -> current_put_value,\
		pa -> percentage, "%", pa -> days_to_expiration,\
		pa -> current_stock_price, pa -> quantity);

		free(monthptr);
		}/* end if put */
	}/* end for all entries */

return 1;
}/* end function print_sorted_option_list */


int show_options(int position)
{
int i;
struct fund *pa;
char fundstr[20];
char *monthptr;
char temp[1024];
int browser_line;
int browser_topline;
int maxline;
int first_new_option_line;
char *ptra;
char *ptrb;
char *ptrc;
int match_failed_flag;
int last_string_flag;
char *filter_keywords;
int double_quote_flag;

if(debug_flag)
	{
	fprintf(stdout, "show_options(): arg none\n");
	}

filter_keywords =\
	(char*)fl_get_input(fdui -> option_evaluation_form_filter_input_field);

/* remember vertical slider position */
browser_topline =\
	fl_get_browser_topline(fdui -> option_evaluation_form_option_browser);

fl_set_object_label(fdui -> option_evaluation_form_status_display,\
	"building display, this may take a while");
XSync(fl_get_display(), 0);

fl_freeze_form(fd_main_form -> option_evaluation_form);

fl_clear_browser(fdui -> option_evaluation_form_option_browser);

first_new_option_line = 0;
browser_line = 1;
for(pa = fundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/*
	aab call oct 1999 25.00  price=1.50 value=1.70 88.2% days=1.22 stock=22.10
	*/ 

	/* extract fund name */
	i = 0;
	while(1)
		{
		fundstr[i] = pa -> name[i];
		if(pa -> name[i] == '/')
			{
			fundstr[i] = 0;
			break;
			}
		if(pa -> name[i] == 0) break;
		i++;
		}/* end while all chars */

	monthptr = digit_to_month(pa -> month);	
	if(! monthptr)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"print_sorted_option_list(): digit_to_month() failed\n");
			}
		/* this should really never happen */
		return 0;
		}

	if(pa -> type == TYPE_CALL)
		{
		sprintf(temp,\
		"%s call %s %d %.2f  \
price=%.2f value=%.2f %.1f%s days=%.2f stock=%.2f quant=%d",\
		fundstr, monthptr, pa -> year, pa -> strike_price,\
		pa -> current_option_price, pa -> current_call_value,\
		pa -> percentage, "%", pa -> days_to_expiration,\
		pa -> current_stock_price, pa -> quantity);

		free(monthptr);
		}/* end if call */

	else if(pa -> type == TYPE_PUT)
		{
		sprintf(temp,\
		"%s put %s %d %.2f  \
price=%.2f value=%.2f %.1f%s days=%.2f stock=%.2f quant=%d",\
		fundstr, monthptr, pa -> year, pa -> strike_price,\
		pa -> current_option_price, pa -> current_put_value,\
		pa -> percentage, "%", pa -> days_to_expiration,\
		pa -> current_stock_price, pa -> quantity);
		free(monthptr);
		}/* end if put */
	else continue;
	
	/* test if any filter keywords */
	if(filter_keywords[0])/* some filter */
		{
		/* test if filter keywords present in item name */
		/* for all entries in filter */
		double_quote_flag = 0;
		ptra = filter_keywords;
		ptrb = strsave(filter_keywords);/* create space */
		match_failed_flag = 0;
		last_string_flag = 0;
		while(1)
			{
			ptrc = ptrb;/* point to start availeble space */
			while(1)/* string in filter to ptrb */
				{
				*ptrc = *ptra;
				if(! *ptra) break;/* end filter */
				
				if(*ptra == '"')/* double quote */
					{
					double_quote_flag = 1 - double_quote_flag;
					ptra++;
					continue;
					}
				if(! double_quote_flag)
					{
					if(*ptra == ' ')/* end of string in filter */
						{
						*ptrc = 0;
						ptra++;/* scip the space */
						break;
						}
					}
				ptra++;
				ptrc++;
				}/* end while all characters in string */
			/* test if string present in name */
			/* test for case sensitive */
			if(option_evaluation_form_filter_case_sensitive_flag)
				{
				if(! strstr(temp, ptrb) )
					{
					match_failed_flag = 1;
					break;
					}
				}
			else /* not case sensitive */
				{
				if(! strcasestr(temp, ptrb) )
					{
					match_failed_flag = 1;
					break;
					}
				}
			if(! *ptra) break;/* end filter */
			}/* end while all strings in filter */	
		free(ptrb);/* release space */
		if(match_failed_flag)
			{
			/* to prevent false finds in double click mark not existing */
			pa -> browser_line = 0;
			continue;
			}
		}/* end if filter */

	fl_add_browser_line(fdui -> option_evaluation_form_option_browser, temp);

	pa -> browser_line = browser_line;
	browser_line++;
	}/* end for all entries */

/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(\
		fdui -> option_evaluation_form_option_browser, first_new_option_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last option visible */
	maxline = fl_get_browser_maxline(fdui -> option_evaluation_form_option_browser);
	fl_set_browser_topline(fdui -> option_evaluation_form_option_browser, maxline);
	}
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> option_evaluation_form_option_browser, 1);
	}
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(\
		fdui -> option_evaluation_form_option_browser, browser_topline);
	}	

fl_unfreeze_form(fd_main_form -> option_evaluation_form);

/* clear status display */
fl_set_object_label(fdui -> option_evaluation_form_status_display, "");
XSync(fl_get_display(), 0);

return 1;
}/* end function show_options */


int save_options()
{
FILE *options_dat_file;
char pathfilename[TEMP_SIZE];
char pathfilename2[TEMP_SIZE];
struct fund *pa;
FILE *fptr;

if(debug_flag)
	{
	fprintf(stdout, "save_options() arg none\n");
	}

sprintf(pathfilename, "%s/options.tmp", file_path);
options_dat_file = fopen(pathfilename, "w");
if(! options_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"could not open file %s for write\n", pathfilename);
		}
	fl_show_alert("Cannot open file", pathfilename, "for write", 1);	

	return(0);
	} 

/* status line not used here */
fprintf(options_dat_file, "\n");

for(pa = fundtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(options_dat_file,\
	"%s %d %.2f %d %d %d %.2f %d %.2f %.2f %.2f %.2f %.2f\n",\
	pa -> name,\
	pa -> browser_line,\
	pa -> strike_price,\
	pa -> month,\
	pa -> year,\
	pa -> type,\
	pa -> current_option_price,\
	pa -> quantity,\
	pa -> days_to_expiration,\
	pa -> current_stock_price,\
	pa -> current_call_value,\
	pa -> current_put_value,\
	pa -> percentage\
	);

	}/* end while all elements in list */

fclose(options_dat_file);

if(debug_flag)
	{
	fprintf(stdout, "save_options(): backing up options.dat\n");
	}

/* set some path file names */
sprintf(pathfilename, "%s/options.dat", file_path);
sprintf(pathfilename2, "%s/options.dat~", file_path);

/* unlink the old .dat~ */
unlink(pathfilename2);

/* test if there is a /.dat file (not first time) */
fptr = fopen(pathfilename, "r");
if(fptr)
	{
	fclose(fptr);
	/* rename .dat to .dat~ */
	if( rename(pathfilename, pathfilename2) == -1)
		{
		if(debug_flag)
			{
			fprintf(stdout, "save_options(); rename %s into %s failed\n",\
			pathfilename, pathfilename2);
			}
		return(0);
		}
	}/* end if .dat file existed */
else
	{
	fl_show_alert("Warning", pathfilename, "does not exist, creating it", 1);
	}		

/* rename .tmp to .dat */
sprintf(pathfilename2, "%s/options.tmp", file_path);
if( rename(pathfilename2, pathfilename) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_options(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return(0);
	}

return(1);/* ok */
}/* end function save_options */


int load_options()
{
int a;
FILE *options_dat_file;
char temp[READSIZE];
char pathfilename[TEMP_SIZE];
struct fund *pa;
int first_option_flag;
char name[512];
int browser_line;
float strike_price;
int month;
int year;
int type;
float current_option_price;
int quantity;
float days_to_expiration;
float current_stock_price;
float current_call_value;
float current_put_value;
float percentage;
int status_line;

if(debug_flag)
	{
	fprintf(stdout, "load_options() arg none\n");
	}
	
if(! delete_all_funds() )
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not delete all funds\n");
		}
	return 0;
	}

sprintf(pathfilename, "%s/options.dat", file_path);
options_dat_file = fopen(pathfilename, "r");
if(! options_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}
	fl_show_alert("Cannot open file", pathfilename, "for read", 1);

	return(0);
	} 

status_line = 1;
first_option_flag = 1;
while(1)
	{
	if(status_line)
		{
		/* read header line or first line of entry */
		a = readline(options_dat_file, temp);
		if(debug_flag)
			{
			fprintf(stdout, "readline 1 temp=%s a=%d\n", temp, a);
			}

		/* status line not used at present */
		status_line = 0;
		if(a == EOF) return 1;/* only this line, no entries */
		continue;
		}/* end if status line */

	/* read line 2 and following */
	a = readline(options_dat_file, temp);

	if(debug_flag)
		{
		fprintf(stdout,\
		"load_options(): readline 2 temp=%s a=%d\n",\
		temp, a);
		}

	if(a == EOF) return 1;/* no more lines */

	/* parse the line */
	sscanf(temp,\
	"%s %d %f %d %d %d %f %d %f %f %f %f %f\n",\
	name,\
	&browser_line,\
	&strike_price,\
	&month,\
	&year,\
	&type,\
	&current_option_price,\
	&quantity,\
	&days_to_expiration,\
	&current_stock_price,\
	&current_call_value,\
	&current_put_value,\
	&percentage\
	);
	
	/* create a structure entry */
	pa = install_fund_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_options: cannot install option %s\n", temp);
			}
		return(0);
		}

	/* data to structure */
	pa -> browser_line = browser_line;
	pa -> strike_price = strike_price;
	pa -> month = month;
	pa -> year = year;
	pa -> type = type;
	pa -> current_option_price = current_option_price;
	pa -> quantity = quantity;
	pa -> days_to_expiration = days_to_expiration;
	pa -> current_stock_price = current_stock_price;
	pa -> current_call_value = current_call_value;
	pa -> current_put_value = current_put_value;
	pa -> percentage = percentage;

	}/* end while all lines in options.dat */

/* never here */
fl_show_alert("file format error in", pathfilename, "", 1);
return 0;/* file format error */
}/* end function load options */


