/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

int parse_davieff_options_file(char *filename)
{
FILE *fileptr;
char temp[READSIZE];
char *ptr;
int a, i;
char fundname[512];
char *fundname_lower_case;
float strike_price;
long volume;
int month;
int monthday;
int year;
int type;
char field1[512];
char field2[512];
char field3[512];
char field4[512];
char field5[512];
char field6[512];
char field7[512];
char field8[512];
int traded_quantity;
char option_name[512];
char call_putstr[20];
char expiration_month[20];
char expiration_year[20];
char strike_price_str[40];
float current_price;
float percentage;
struct notering *pa;
char *month_name;

/* 
IMPORTANT
There are no 2 the same entries in this file, but same entries may already be
present, in the portfolio.
So no need to use lookup before install here (speed).
Because the portfolio list may already hold some of the same entries,
now this data is stored in a hashed list for speedy lookup later.
Then later entries already present in the portfolio are marked as such in the
hash list.
Then the unmarked entries in the hash list are copied to the portfolio
list without lookup.
All this for speed, else the whole operation would take hours.
*/

if(debug_flag)
	{
	fprintf(stdout,\
	"parse_davieff_options_file(): arg filename=%s\n",\
	filename);
	}

if(! filename) return 0;

fileptr = fopen(filename, "r");
if(! fileptr)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_options_file(): cannot open file %s for read\n",\
		filename);
		}
	fl_show_alert("Cannot open file", filename, "for read", 1);

	return 0;
	}

while(1)
	{
	/*
	This is what it looks like, only use first entry.
	AAB C 07/1999 12.47,AAB C 07/1999 12.47,29/06/1999,0.00,0.00,0.00,8.90,0
	8 fields.
	*/

	a = readline(fileptr, temp);/* closes file on EOF */
	if(a == EOF) break;

	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_options_file(): line read=\n\
%s\n", temp);
		}

	ptr = strtok(temp, ",");/* field 1  AAB C 07/1999 12.47 */
	if(! ptr) continue;
	strcpy(field1, ptr);
	
	ptr = strtok(NULL, ",");/* field 2 AAB C 07/1999 12.47 same as field 1 */
	if(! ptr) continue;
	strcpy(field2, ptr);

	ptr = strtok(NULL, ",");/* 29/06/1999 */ /* date */
	if(! ptr) continue;
	strcpy(field3, ptr);
		
	ptr = strtok(NULL, ",");/* 0.00 */ /* do not know */
	if(! ptr) continue;
	strcpy(field4, ptr);
	
	ptr = strtok(NULL, ",");/* 0.00 */ /* do not know */
	if(! ptr) continue;
	strcpy(field5, ptr);
	
	ptr = strtok(NULL, ",");/* 0.00 */ /* do not know */
	if(! ptr) continue;
	strcpy(field6, ptr);
	
	ptr = strtok(NULL, ",");/* 8.90 */ /* current price */
	if(! ptr) continue;
	strcpy(field7, ptr);
	
	ptr = strtok(NULL, ",");/* 0 */ /* traded_quantity */
	if(! ptr) continue;
	strcpy(field8, ptr);
	
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_options_file():\n\
		field1=%s\n\
		field2=%s\n\
		field3=%s\n\
		field4=%s\n\
		field5=%s\n\
		field6=%s\n\
		field7=%s\n\
		field8=%s\n",\
		field1, field2, field3, field4, field5, field6, field7, field8);
		}

	/*
	field1 looks like AAB C 07/1999 12.47
	*/
	sscanf(field1,\
	"%s %s %d/%d %f",\
	fundname, call_putstr, &month, &year, &strike_price);
		
	/* test if fund in list to be processed (~/.option_scanrc */
/*	if(! test_stock(fundname_lower_case) ) continue;*/
	
	/* field3 looks like 06/11/1998 */

	/* field7 looks like 20.10 */
	sscanf(field7, "%f", &current_price);

	/* field8 looks like 435 */
	sscanf(field8, "%d", &traded_quantity);
	
	if(call_putstr[0] == 'C') type = TYPE_CALL;
	else if(call_putstr[0] == 'P') type = TYPE_PUT;
	else type = 0;

	if( (type == TYPE_CALL) || (type == TYPE_PUT) || (type == TYPE_BOND) )
		{
		month_name = digit_to_month(month);
		if(! month_name) return 0;
		}

	/* fund to lower case */
	fundname_lower_case = strsave(fundname);/* should be same length  */
	if(! fundname_lower_case) return 0;
	i = 0;
	while(1)
		{
		fundname_lower_case[i] = tolower(fundname[i]);
		if(fundname[i] == 0) break;
		i++;
		}

	/* create an unique identifier */
	if(type == TYPE_STOCK)
		{
		/* aab */
		sprintf(temp,\
		"%s",\
		fundname_lower_case);
		free(fundname_lower_case);
		}
	else if(type == TYPE_CALL)
		{
		/* aab call oct 1999 22.00 */
		sprintf(temp,\
		"%s call %s %d %.2f",\
		fundname_lower_case, month_name, year, strike_price);
		free(fundname_lower_case);
		free(month_name);
		}
	else if(type == TYPE_PUT)
		{
		/* aab put oct 1999 22.00 */
		sprintf(temp,\
		"%s put %s %d %.2f",\
		fundname_lower_case, month_name, year, strike_price);
		free(fundname_lower_case);
		free(month_name);
		}
	else if(type == TYPE_BOND)
		{
		/* somebond */
		sprintf(temp,\
		"%s bond %s %d %.2f",\
		fundname_lower_case, month_name, year, strike_price);
		free(fundname_lower_case);
		free(month_name);
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_options_file(): installing %s in hash table\n",\
		temp);
		}

	pa = install_notering(temp);
	if(! pa)
		{
		fl_show_alert(\
			"Cannot install Davieff option", temp, "aborting", 1);
		return 0;
		}

	pa -> type = type;
	pa -> expiration_month = month;
	pa -> expiration_year = year;
	pa -> strike_price = strike_price;
	pa -> present_flag = 0;
	pa -> current_price = current_price;
	}/* end while all lines in .csv file */

return(1);
}/* end function parse_davieff_options_file */


int parse_davieff_shares_file(char *filename)
{
int i;
FILE *fileptr;
char temp[READSIZE];
char *ptr;
int a;
char fundname[512];
char datestr[80];
float close_price;
long volume;
int monthday, year, month;
int type;
char *fundname_lower_case;
struct notering *pa;
float current_price;

if(debug_flag)
	{
	fprintf(stdout,\
	"parse_davieff_shares_file(): arg filename=%s\n",\
	filename);
	}

/* argument check */
if(! filename) return(0);

fileptr = fopen(filename, "r");
if(! fileptr)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_shares_file(): cannot open file %s for read\n",\
		filename);
		}

	fl_show_alert("Cannot open file", filename, "for read", 1);
	return(0);
	}

/* this is a shares file */
type = TYPE_STOCK;
while(1)
	{
	/* ABN AMRO HOLDING,AAB,06/11/1998,40.1,40.3,38.8,38.8,8774554
 */
	a = readline(fileptr, temp);/* closes file on EOF */
	if(a == EOF) break;

	ptr = strtok(temp, ",");/* description  ABN AMRO HOLDING */
	if(! ptr) continue;

	ptr = strtok(NULL, ",");/* abbreviation AAB */
	if(! ptr) continue;
	strcpy(fundname, ptr);

	ptr = strtok(NULL, ",");/* date 06/11/1998 */
	if(! ptr) continue;
	strcpy(datestr, ptr);	

	ptr = strtok(NULL, ",");/* ?? 40.1 */
	if(! ptr) continue;
		
	ptr = strtok(NULL, ",");/* ?? ,40.3 */
	if(! ptr) continue;

	ptr = strtok(NULL, ",");/* ?? 38.8 */
	if(! ptr) continue;

	ptr = strtok(NULL, ",");/* close price 38.8 */
	if(! ptr) continue;
	close_price = atof(ptr);
		
	ptr = strtok(NULL, ",");/* volume 8774554 */
	if(! ptr) continue;
	volume = atof(ptr);

	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_shares_file():\n\
		fund=%s date=%s close=%.2f volume=%ld\n",\
		fundname, datestr, close_price, volume);
		}

	/* datestr now looks like 06/11/1998 */
	sscanf(datestr, "%d/%d/%d", &monthday, &month, &year);

	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_shares_file():\n\
		monthday=%d year=%d month=%d\n",\
		monthday, year, month);
		}

	/* fund to lower case */
	fundname_lower_case = strsave(fundname);/* should be same length  */
	if(! fundname_lower_case) return 0;
	i = 0;
	while(1)
		{
		fundname_lower_case[i] = tolower(fundname[i]);
		if(fundname[i] == 0) break;
		i++;
		}

	/* create an unique identifier */
	if(type == TYPE_STOCK)
		{
		/* aab */
		sprintf(temp,\
		"%s",\
		fundname_lower_case);
		free(fundname_lower_case);
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_shares_file(): installing %s in hash table\n",\
		temp);
		}

	pa = install_notering(temp);
	if(! pa)
		{
		fl_show_alert(\
			"Cannot install Davieff share", temp, "aborting", 1);
		return 0;
		}

	pa -> type = type;
	pa -> expiration_month = 0;
	pa -> expiration_year = 0;
	pa -> strike_price = 0;
	pa -> present_flag = 0;
	pa -> current_price = close_price;
	}/* end while all lines in .csv file */

return 1;
}/* end function parse_davieff_shares_file */


int import_davieff_file(int file_type)
{
int i;
char *ptr;
struct notering *pa;
char fund[512];
int test_exists_flag;
char temp[512];


if(debug_flag)
	{
	fprintf(stdout,\
	"import_davieff_file(): arg file_type=%d\n",\
	file_type);
	}

/* argument_check */
if( (file_type != OPTIONS_FILE) &&\
(file_type != SHARES_FILE) )
	{
	return 0;
	}
if(file_type == OPTIONS_FILE)
	{
	ptr = (char *) fl_show_fselector(\
		"Davieff options file?", home_dir, "*", davieff_options_filename);
	if(! ptr) return;

	/*
	modify global for latest entry, so if wrong, user does not have to
	retype everything
	*/
	if(davieff_options_filename) free(davieff_options_filename);
	davieff_options_filename = strsave(ptr);
	if(! davieff_options_filename) return 0;
	}
else if(file_type == SHARES_FILE)
	{
	ptr = (char *) fl_show_fselector(\
		"Davieff shares file?", home_dir, "*", davieff_shares_filename);
	if(! ptr) return;

	/*
	modify global for latest entry, so if wrong, user does not have to
	retype everything
	*/
	if(davieff_shares_filename) free(davieff_shares_filename);
	davieff_shares_filename = strsave(ptr);
	if(! davieff_shares_filename) return 0;
	}

sprintf(temp, "parsing %s", ptr);
fl_set_object_label(fdui -> portfolio_form_status_display,\
	temp);
XSync(fl_get_display(), 0);

/* clear the hashed list */
if(! delete_all_noteringen() )
	{
	fl_set_object_label(fdui -> portfolio_form_status_display,\
		"could not clear list, aborted");

	XSync(fl_get_display(), 0);
	return 0;
	}

if(file_type == OPTIONS_FILE)
	{
	/* load the davieff options file in the hashed list */
	if(! parse_davieff_options_file(ptr) )
		{
		return 0;
		}
	}
else if(file_type == SHARES_FILE)
	{
	/* load the davieff shares file in the hashed list */
	if(! parse_davieff_shares_file(ptr) )
		{
		return 0;
		}
	}

/*
test if any of the portfolio entries are present in the hashed list,
if present mark entry in hashed list 
*/
fl_set_object_label(fdui -> portfolio_form_status_display,\
	"testing for double entries");
XSync(fl_get_display(), 0);

if(! mark_noteringen_already_present() )
	{
	fl_set_object_label(fdui -> portfolio_form_status_display,\
		"could not mark already present, aborted");
	XSync(fl_get_display(), 0);
	
	fl_show_alert("Cannot mark noteringen already present", "", "", 1);
	return 0;	
	}

/* 
copy not marked entries in hashed list to portfolio list without using
lookup.
*/
fl_set_object_label(fdui -> portfolio_form_status_display,\
	"adding new entries to list");
XSync(fl_get_display(), 0);

test_exists_flag = 0;
for(i = 0; i < NOTERINGEN_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = noteringtab[i]; pa != 0; pa = pa -> nxtentr)	
		{
		if(pa -> type == 0) continue;/* unkown type */

		/* extract fund from description */
		sscanf(pa -> name, "%s", fund);

		/* scip if item already present */
		if (pa -> present_flag)
			{
			/* need to modify the old price, we have a new one now */
			if(! modify_portfolio_item_price(\
			fund, pa -> type, pa -> expiration_month, pa -> expiration_year, pa -> strike_price,\
			pa -> current_price\
			) )
				{
				sprintf(temp, "could not modify price for item %s", fund);
				fl_set_object_label(fdui -> portfolio_form_status_display,\
				temp);
				XSync(fl_get_display(), 0);

				fl_show_alert(\
					"Cannot modify price for item",\
					fund,\
					"scipping",\
					1);
				continue;
				}			
			continue;
			}

				
		if(! add_item_to_portfolio(\
		fund, pa -> type, pa -> expiration_month, pa -> expiration_year, pa -> strike_price,\
		test_exists_flag, pa -> current_price\
		) )
			{
			sprintf(temp, "could not add item %s", fund);
			fl_set_object_label(fdui -> portfolio_form_status_display, temp);
			XSync(fl_get_display(), 0);

			fl_show_alert("Cannot add", pa -> name, "to portfolio, scipping", 1);
			continue;
			}	
		}/* end for all structures */
	}/* end for all entries in noteringtab */

/* for speed */
fl_freeze_form(fd_main_form -> portfolio_form);

if(! show_items(SHOW_TOP) )
	{
	fl_unfreeze_form(fd_main_form -> portfolio_form);
	return 0;
	}

fl_unfreeze_form(fd_main_form -> portfolio_form);

fl_set_object_label(fdui -> portfolio_form_status_display, "");
XSync(fl_get_display(), 0);

return 1;
}/* end function import_davieff_file */
 

int parse_davieff_option_file_for_evaluation(char *filename)
{
FILE *fileptr;
char temp[READSIZE];
char *ptr;
int a, c, i;
char fundname[512];
char fundname_lower_case[512];
float strike_price;
long volume;
int month;
int year;
int type;
char field1[512];
char field2[512];
char field3[512];
char field4[512];
char field5[512];
char field6[512];
char field7[512];
char field8[512];
int quantity;
char option_name[512];
char call_putstr[20];
char expiration_month[20];
char expiration_year[20];
char strike_price_str[40];
float days_to_expiration;
float current_riskfree_interest_rate;
float current_stock_price;
float daily_standard_deviation;
double yearly_standard_deviation;
float current_call_value;
float current_put_value;
float current_option_price;
float percentage;
int lines;
int done;

if(debug_flag)
	{
	fprintf(stdout,\
	"parse_davieff_option_file_for_evaluation(): arg filename=%s\n",\
	filename);
	}

if(! filename) return(0);

fileptr = fopen(filename, "r");
if(! fileptr)
	{
	fl_show_alert("Cannot open file for read", filename, "", 1);
	return(0);
	}

/* calculate number of lines for progress percentage display */
lines = 0;
while(1)
	{
	c = getc(fileptr);
	if(c == EOF) break;
	if(c == '\n') lines++;
	}
fclose(fileptr);

fileptr = fopen(filename, "r");
if(! fileptr)
	{
	fl_show_alert("Cannot open file for read", filename, "", 1);
	return(0);
	}

/* get the current interest rate */
sprintf(temp, "%s%s.krs", file_path, "r10j");
/* arg 2 already a pointer */
if(! get_current_value(temp, &current_riskfree_interest_rate) )
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_option_file_for_evaluation():\n\
		cannot get current value for %s\n",\
		temp);
		}

	return 0;
	}
/* convert from percentages to real values */
current_riskfree_interest_rate /= 100.0;
if(debug_flag)
	{
	fprintf(stdout,\
	"parse_davieff_option_file_for_evaluation(): current risk free interest rate=%.6f\n",\
	current_riskfree_interest_rate);
	}

done = 0;
while(1)
	{
	/*
	This is what it looks like, only use first entry.
	AAB C 07/1999 12.47,AAB C 07/1999 12.47,29/06/1999,0.00,0.00,0.00,8.90,0
	8 fields.
	*/

	a = readline(fileptr, temp);/* closes file on EOF */
	if(a == EOF) break;

	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_option_file_for_evaluation(): line read=\n\
%s\n", temp);
		}

	ptr = strtok(temp, ",");/* field 1  AAB C 07/1999 12.47 */
	if(! ptr) continue;
	strcpy(field1, ptr);
	
	ptr = strtok(NULL, ",");/* field 2 AAB C 07/1999 12.47 same as field 1 */
	if(! ptr) continue;
	strcpy(field2, ptr);

	ptr = strtok(NULL, ",");/* 29/06/1999 */ /* date */
	if(! ptr) continue;
	strcpy(field3, ptr);
		
	ptr = strtok(NULL, ",");/* 0.00 */ /* do not know */
	if(! ptr) continue;
	strcpy(field4, ptr);
	
	ptr = strtok(NULL, ",");/* 0.00 */ /* do not know */
	if(! ptr) continue;
	strcpy(field5, ptr);
	
	ptr = strtok(NULL, ",");/* 0.00 */ /* do not know */
	if(! ptr) continue;
	strcpy(field6, ptr);
	
	ptr = strtok(NULL, ",");/* 8.90 */ /* current price */
	if(! ptr) continue;
	strcpy(field7, ptr);
	
	ptr = strtok(NULL, ",");/* 0 */ /* quantity */
	if(! ptr) continue;
	strcpy(field8, ptr);
	
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_option_file_for_evaluation():\n\
		field1=%s\n\
		field2=%s\n\
		field3=%s\n\
		field4=%s\n\
		field5=%s\n\
		field6=%s\n\
		field7=%s\n\
		field8=%s\n",\
		field1, field2, field3, field4, field5, field6, field7, field8);
		}

	/*
	field1 looks like AAB C 07/1999 12.47
	*/
	sscanf(field1,\
	"%s %s %d/%d %f",\
	fundname, call_putstr, &month, &year, &strike_price);
		
	/* convert fundname to lower case */
	i = 0;
	while(1)
		{
		fundname_lower_case[i] = tolower(fundname[i]);
		if(fundname[i] == 0) break;
		i++;
		}

	/* test if fund in list to be processed */
	if(! test_stock(fundname_lower_case) ) continue;
	
	/* field3 looks like 06/11/1998 */
	/*sscanf(field3, "%d/%d/%d", &monthday, &month, &year);*/

	/* field7 looks like 20.10 */
	sscanf(field7, "%f", &current_option_price);

	/* field8 looks like 435 */
	sscanf(field8, "%d", &quantity);
	
	if(call_putstr[0] == 'C') type = TYPE_CALL;
	else if(call_putstr[0] == 'P') type = TYPE_PUT;
	else type = 0;

	percentage = ((float)done / (float)lines) * 100.0;
/*
	if( (percentage - old_percentage) > 1.0)
		old_percentage = percentage;
		}
*/

	sprintf(temp, "calculating price for %s (total %2.0f%c done)",\
	field1, percentage, '%');
	fl_set_object_label(fdui -> option_evaluation_form_status_display,\
	temp);
	XSync(fl_get_display(), 0);

	/* calculate current_call_value and current_put_value */
	sprintf(strike_price_str, "%.2f", strike_price);

	ptr = digit_to_month(month);
	if(ptr)
		{
		sprintf(expiration_month, "%s", ptr); 
		free(ptr);
		}

	sprintf(expiration_year, "%d", year);

	if(! calculate_option_price_fast(\
	 fundname_lower_case,\
	 expiration_month,\
	 expiration_year,\
	 strike_price_str,\
	 &days_to_expiration,\
	 &current_riskfree_interest_rate,\
	 &current_stock_price,\
	 &daily_standard_deviation,\
	 &yearly_standard_deviation,\
	 &current_call_value,\
	 &current_put_value\
	 ) )
	 	{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_davieff_option_file_for_evaluation():\n\
			passing on %s\n",\
			fundname_lower_case);
			}

	 	continue;/* no such krs file */
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_davieff_option_file_for_evaluation():\n\tfundname=%s days_to_expiration=%.2f\n",\
		fundname, days_to_expiration);
		if(type == TYPE_CALL)
			{
			fprintf(stdout, "\tcurrent_call_value=%.2f\n", current_call_value);
			}
		if(type == TYPE_PUT)
			{
			fprintf(stdout, "\tcurrent_put_value=%.2f\n", current_put_value);
			}
		fprintf(stdout,\
		"\tdays_to_expiration=%.2f\n",\
		days_to_expiration);
		fprintf(stdout,\
		"\tcurrent_option_price=%.2f\n",\
		current_option_price);
		}

	/* scip if already expired */
	if(days_to_expiration <= 0) continue;
	
	/* scip if no real price ? */
	if(current_option_price <= 0)
		{
		continue;
		}

	/* scip if no quantity (not enough trade, dangerous) */
	if(quantity < 1)
		{
		continue;
		}
	
	/* calculate percentage */	
	if(type == TYPE_CALL)
		{
		/* avoid divide by zero */ 
		if(current_call_value == 0)
			{
			continue;
			}
		percentage = (current_option_price / current_call_value) * 100.0;	
		}
	else if(type == TYPE_PUT)
		{
		/* avoid divide by zero */
		if(current_put_value == 0)
			{
			continue;
			}
		percentage = (current_option_price / current_put_value) * 100.0;
		}
	else
		{
		/* no call or put */
		continue;
		}

	/* aab/C/9/1999/20.50 */
	sprintf(option_name, "%s/%s/%d/%d/%.2f",\
	 fundname_lower_case, call_putstr, month, year, strike_price);

	if(! add_option_to_database(\
	option_name, strike_price, type, month, year, current_option_price,\
	quantity, days_to_expiration, current_stock_price,\
	current_call_value, current_put_value,\
	percentage) )
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_davieff_option_file_for_evaluation():\n\
			cannot add option %s to database\n",\
			option_name);
			}

		return(0);
		}

	done++;
	}/* end while all lines in .csv file */

/* clear display */
fl_set_object_label(fdui -> option_evaluation_form_status_display, "");
XSync(fl_get_display(), 0);

return(1);
}/* end function parse_davieff_option_file_for_evaluation */


