/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define START_COLUMN_CLOSE_PRICE 16


#define FSFUNDS_HASH_SIZE 100

struct fsfund /* used for translation fastservice fundname to real ticker */
	{
	char *name;
	char *ticker;
			
	struct fsfund *nxtentr;
	struct fsfund *prventr;
	};
struct fsfund *fsfundtab[FSFUNDS_HASH_SIZE];


int fshash(s)/* form hash value for string s */
char *s;
{
int hashval;

for(hashval = 0; *s != '\0';) hashval += *s++;
/* sum of ascii value of characters divided by tablesize */
return(hashval % FSFUNDS_HASH_SIZE);
}/* end function fshash */


struct fsfund *lookup_fsfund(char *name)
{
struct fsfund *pa;

if(debug_flag)
	{
	fprintf(stdout, "lookup_fsfund(): arg name=%s\n", name);
	}
	
for(pa = fsfundtab[fshash(name)]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcasecmp(pa -> name, name) == 0) return(pa);/* found sequence entry */
	}
return(0); /* not found */
}/* end function lookup_fsfund */


struct fsfund *install_fsfund(char *name)
{
struct fsfund *pnew, *pnext;
int hashval;

if(debug_flag)
	{
	fprintf(stdout, "install_fsfund(): arg name=%s\n", name);
	}
	
/* create new structure */
pnew = (struct fsfund *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get next structure */
hashval = fshash(name);
pnext = fsfundtab[hashval];/* may be zero, if there was nothing */

/* insert before next structure (if any, else at start) */
fsfundtab[hashval] = pnew;

/* set pointers for next structure */
if(pnext) pnext -> prventr = pnew;
	
/* set pointers for new structure */
pnew -> nxtentr = pnext;
pnew -> prventr = 0;/* always inserting at start of chain of structures */
	
return(pnew);/* pointer to new structure */
}/* end function install_fsfund */


int delete_all_fsfunds()
{
struct fsfund *pa;
int i;

for(i = 0; i < FSFUNDS_HASH_SIZE; i++)
	{
	while(1)
		{
		pa = fsfundtab[i];
		if(! pa) break;

		fsfundtab[i] = pa -> nxtentr;

		free(pa -> name);/* free name */
		if(pa -> ticker) free(pa -> ticker);
		free(pa);/* free structure */
		}/* end while all structures hashing to this value */ 
	}/* end for all entries in fsfundtab */

return 1;
}/* end function delete_all_fsfunds */


int load_fsfunds()
{
int a;
FILE *fsfund_fptr;
char temp[1024];
char pathfilename[TEMP_SIZE];
char name[1024];
char ticker[1024];
char *ptr;
struct fsfund *pa;

if(debug_flag)
	{
	fprintf(stdout, "load_fsfunds(): arg none\n");
	}

delete_all_fsfunds();

sprintf(pathfilename, "%s/fastservice.dat", file_path);
fsfund_fptr = fopen(pathfilename, "r");
if(! fsfund_fptr)
	{
	fl_show_alert("Could not open file ", pathfilename, "for read", 0);
	return(0);
	} 

sprintf(temp, "loading %s this may take a while", pathfilename);
fl_set_object_label(fdui -> portfolio_form_status_display, temp);
XSync(fl_get_display(), 0);

while(1)
	{
	a = readline(fsfund_fptr, temp);/* closes file if EOF */
	if(a == EOF) return 1;

	/* scip comments */
	if(temp[0] == '#') continue;
	
	/* scip empty lines */
	if(strlen(temp) == 0) continue;
	
	ptr = strtok(temp, ",");
	if(! ptr) continue;
	strcpy(name, ptr);
	
	ptr = strtok(NULL, ",");
	if(! ptr) continue;
	strcpy(ticker, ptr);/* empty fields cause continue */

	pa = install_fsfund(name);
	if(! pa)
		{
		fl_show_alert("load_fsfunds():", "cannot install", name, 0);
		fclose(fsfund_fptr);
		return(0);
		}
	
	pa -> ticker = strsave(ticker);
	if(! pa -> ticker)
		{
		fl_show_alert("load_fsfunds():", "cannot install", ticker, 0);
		fclose(fsfund_fptr);
		return 0;
		}
	}/* end while all lines in file */

fl_set_object_label(fdui -> portfolio_form_status_display, "");
XSync(fl_get_display(), 0);

return 1;
}/* end function load_fsfund */


/* not used */
int save_fsfunds()
{
int i;
FILE *fsfund_fptr;
char pathfilename[TEMP_SIZE];
struct fsfund *pa;
extern char *strsave();

if(debug_flag) 
	{
	fprintf(stdout, "save_fsfunds(): arg none\n");
	}

sprintf(pathfilename, "%s/fastservice.dat", file_path);
fsfund_fptr = fopen(pathfilename, "w");
if(! fsfund_fptr)
	{
	fl_show_alert("Cannot open file", pathfilename, "for write", 0);
	return(0);
	} 

for(i = 0; i < FSFUNDS_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = fsfundtab[i]; pa != 0; pa = pa -> nxtentr)
		{
		fprintf(fsfund_fptr, "%s %s\n", pa -> name, pa -> ticker);
		}
	}/* end for all entries in fsfundtab */

fclose(fsfund_fptr);

return(1);
}/* end function save_fsfunds */


int import_fastservice_files()
{
int a, i;
int messages;
char temp[512];
char fund[512];
int test_exists_flag;
struct notering *pa;

if(debug_flag)
	{
	fprintf(stdout, "import_fastservice_files(): arg none\n");
	}

/* read fund descriptions and tickers into double linked hashed list */
if(! load_fsfunds() )
	{
	return 0;
	}

/* check for mail */
/* This will test for mail, and if any load it into a linked list */
messages = 0;
a = load_incoming_mail(&messages);
if(! a)
	{
	fl_show_alert("Cannot read incoming mail", "command aborted", "", 1);
	return 0;
	}

sprintf(temp, "%d incoming messages found", messages);
fl_set_object_label(fdui -> portfolio_form_status_display, temp);
XSync(fl_get_display(), 0);
if(debug_flag)
	{
	fprintf(stdout, "import_fastservice_files(): %s\n", temp);
	}

if(messages == 0)
	{
	fl_show_alert("No email messages found", "command aborted", "", 1);
	return 0;
	}

/* clear the notering list */
/*
done every email 
if(! delete_all_noteringen() )
	{
	fl_set_object_label(fdui -> portfolio_form_status_display,\
		"could not clear notering list, aborted");

	XSync(fl_get_display(), 0);
	return 0;
	}
*/

/*
test for any mail from fastservice (mails should be sorted on date) or
do a fastservice date sort?
parse the fastservice mail and add to notering linked list.
*/
if(! add_fastservice_shares_to_notering_list() )
	{
	fl_show_alert("Cannot add fastservice mail", "command aborted", "", 1);
	return 0;
	}

/*
test if any of the portfolio entries are present in the notering list,
if present mark entry in hashed list 
*/
fl_set_object_label(fdui -> portfolio_form_status_display,\
	"testing for double entries");
XSync(fl_get_display(), 0);

/*
done every email
if(! mark_noteringen_already_present() )
	{
	fl_set_object_label(fdui -> portfolio_form_status_display,\
		"could not mark already present, aborted");
	XSync(fl_get_display(), 0);
	
	fl_show_alert("Cannot mark noteringen already present", "", "", 1);
	return 0;	
	}
*/

/* 
copy not marked entries in hashed list to portfolio list without using
lookup.
*/
fl_set_object_label(fdui -> portfolio_form_status_display,\
	"adding new entries to list");
XSync(fl_get_display(), 0);

test_exists_flag = 0;
for(i = 0; i < NOTERINGEN_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = noteringtab[i]; pa != 0; pa = pa -> nxtentr)	
		{
		if(pa -> type == 0) continue;/* unkown type */

		if(debug_flag)
			{
			fprintf(stdout,\
			"import_fastservice_files() notering to items\n\
			pa->present_flag=%d pa->name=%s len=%d\n",\
			pa -> present_flag, pa -> name, strlen(pa -> name));
			}

		/* extract fund from description */
		sscanf(pa -> name, "%s", fund);

		/* scip if item already present */
		if (pa -> present_flag)
			{
			/* need to modify the old price, we have a new one now */
			if(! modify_portfolio_item_price(\
			fund, pa -> type, pa -> expiration_month, pa -> expiration_year, pa -> strike_price,\
			pa -> current_price\
			) )
				{
				sprintf(temp, "could not modify price for item %s", fund);
				fl_set_object_label(fdui -> portfolio_form_status_display,\
				temp);
				XSync(fl_get_display(), 0);

				fl_show_alert(\
					"Cannot modify price for item",\
					fund,\
					"scipping",\
					1);
				continue;
				}			
			continue;
			}
				
		if(! add_item_to_portfolio(\
		fund, pa -> type, pa -> expiration_month, pa -> expiration_year, pa -> strike_price,\
		test_exists_flag, pa -> current_price\
		) )
			{
			sprintf(temp, "could not add item %s", fund);
			fl_set_object_label(fdui -> portfolio_form_status_display, temp);
			XSync(fl_get_display(), 0);

			fl_show_alert("Cannot add", pa -> name, "to portfolio, scipping", 1);
			continue;
			}	
		}/* end for all structures */
	}/* end for all entries in noteringtab */

/* for speed */
fl_freeze_form(fd_main_form -> portfolio_form);

if(! show_items(SHOW_TOP) )
	{
	fl_unfreeze_form(fd_main_form -> portfolio_form);
	return 0;
	}

fl_unfreeze_form(fd_main_form -> portfolio_form);

fl_set_object_label(fdui -> portfolio_form_status_display, "");
XSync(fl_get_display(), 0);

return 1;
}/* end function import_fastservice_files */


int parse_fastservice_shares_email(char *text)
{
int i;
FILE *fileptr;
char temp[READSIZE];
char *ptr;
int a;
char fundname[512];
float close_price;
long volume;
int monthday, year, month;
int type;
char fundname_lower_case[512];
struct notering *pa;
struct fsfund *pb;
float current_price;
int data_flag;
char *textptr;
int eot_flag;
int space_cnt;
char *close_price_ptr;
int line_format_error_flag;
int email_format_error_flag;

if(debug_flag)
	{
	fprintf(stdout,\
	"parse_fastservice_shares_email(): arg text=%s\n",\
	text);
	}

/* argument check */
if(! text) return(0);

if(! delete_all_noteringen() )
	{
	fl_set_object_label(fdui -> portfolio_form_status_display,\
		"could not clear notering list, aborted");

	XSync(fl_get_display(), 0);
	return 0;
	}


email_format_error_flag = 0;
eot_flag =0;
textptr = text;
data_flag = 0;
type = TYPE_STOCK;
while(1)
	{
/*
Fonds		Huidig	Delta	Tijd	Hoog	Laag	Open	Bied	Laat	Volume

AEX-index      	560.52	-9.58	16:30	570.60	559.81	569.29	-	-	-
wolt-kluw c    	34.25	0.05	16:29	34.70	33.80	34.00	34.30	34.35	1453666

Legenda:
*/
	/* this could happen if empty line */
	if(eot_flag) break;
	
	/* copy textptr up to LF to temp */
	i = 0;
	while(1)
		{
		temp[i] = textptr[i];
		if(textptr[i] == 0)
			{
			eot_flag = 1;
			break;
			}/* end if EOT */
		if(textptr[i] == '\n')
			{
			temp[i] = 0;
			textptr++;/* scip the LF */
			textptr += i;
			break;
			}/* end if LF */
		i++;
		}/* end for all chars in a line */
	
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_fastservice_shares_email(): eot_flag=%d temp=%s\n",\
		eot_flag, temp);
		}

	/* test for first line */
	if(strncasecmp(temp, "Fonds", 5) == 0) continue;
	
	/* test for empty line */
	if(strlen(temp) == 0) continue;
	
	/* test for end of data */
	if(strncasecmp(temp, "Legenda", 7) == 0) break;
	
	/*
	extract fund name (according to fastservice), it ends with 2 spaces,
	and convert to lower case
	*/
	line_format_error_flag = 0;
	space_cnt = 0;
	i = 0;
	while(1)
		{
		fundname_lower_case[i] = tolower(temp[i]);
		if(temp[i] == 0)
			{
			line_format_error_flag = 1;/* nothing after fund name */
			email_format_error_flag = 1;
			break;
			}
		/*
		Some long fund names are followed only by a single space and then
		the close price.
		*/
		if(i == START_COLUMN_CLOSE_PRICE)
			{
			fundname_lower_case[i - 1] = 0;
			close_price_ptr = temp + i;
			break;
			}
		
		if(temp[i] == ' ')
			{
			if(space_cnt == 1)
				{
				fundname_lower_case[i - 1] = 0;
				close_price_ptr = temp + i + 1;
				break;
				}
			space_cnt++;
			}
		else space_cnt = 0;
		i++;
		}/* end while  name to lower case */
	if(line_format_error_flag) continue;  /* scip incorrect entry */
	
	/* scanf will scip any leading spaces */
	sscanf(close_price_ptr, "%f", &close_price);
	
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_fastservice_shares_email():\n\
		fundname_lower_case=%s\n\
		close_price_ptr=%s\n\
		close_price=%.2f\n",\
		fundname_lower_case,
		close_price_ptr,\
		close_price);
		}
	
	/*
	test if fundname in fastservice.dat, if not continue, else replace
	with standard name
	*/
	pb = lookup_fsfund(fundname_lower_case);
	if(! pb)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_fastservice_shares_email(): %s not found\n",\
			fundname_lower_case);
			}
		continue;
		}
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_fastservice_shares_email():\n\
		fundname_lower_case=%s pb->ticker=%s\n",\
		fundname_lower_case, pb -> ticker);
		}

	pa = install_notering(pb -> ticker);
	if(! pa)
		{
		fl_show_alert(\
			"Cannot install fastservice share", temp, "aborting", 1);
		return 0;
		}

	pa -> type = type;
	pa -> expiration_month = 0;
	pa -> expiration_year = 0;
	pa -> strike_price = 0;
	pa -> present_flag = 0;
	pa -> current_price = close_price;
	}/* end while all lines in text */

if(email_format_error_flag)
	{
	fl_show_alert(\
	"There were format errors in the fastservice email", "ignored", "", 1);
	}

return 1;
}/* end function parse_fastservice_shares_email */


int update_krs_files_from_fastservice_email()
{
int a;
struct item *pa;
char temp[1024];
char datestr[40];
char weekday[20];
char monthstr[20];
int year, month, monthday, hour, minute, second;
char filename[1024];
int messages;

if(debug_flag)
	{
	fprintf(stdout,\
	"update_krs_files_from_fastservice_email(): arg none\n");
	}

/* read fund descriptions and tickers into double linked hashed list */
if(! load_fsfunds() )
	{
	fl_set_object_label(fdui -> portfolio_form_status_display, "");
	XSync(fl_get_display(), 0);

	return 0;
	}

/* check for mail */
/* This will test for mail, and if any load it into a linked list */
messages = 0;
a = load_incoming_mail(&messages);
if(! a)
	{
	fl_show_alert("Cannot read incoming mail", "command aborted", "", 1);

	fl_set_object_label(fdui -> portfolio_form_status_display, "");
	XSync(fl_get_display(), 0);

	return 0;
	}

sprintf(temp, "%d incoming messages found", messages);
fl_set_object_label(fdui -> portfolio_form_status_display, temp);
XSync(fl_get_display(), 0);
if(debug_flag)
	{
	fprintf(stdout, "import_fastservice_files(): %s\n", temp);
	}

if(messages == 0)
	{
	fl_show_alert("No email messages found", "command aborted", "", 1);

	fl_set_object_label(fdui -> portfolio_form_status_display, "");
	XSync(fl_get_display(), 0);

	return 0;
	}

if(! parse_all_fastservice_shares_emails_and_update_krs_files() )
	{
	fl_show_alert(\
		"Cannot parse all fastservice emails",\
		"and update .krs files",\
		"command aborted", 1);

	fl_set_object_label(fdui -> portfolio_form_status_display, "");
	XSync(fl_get_display(), 0);

	return 0;
	}

fl_set_object_label(fdui -> portfolio_form_status_display, "");
XSync(fl_get_display(), 0);

return 1;
}/* end function update_krs_files_from_fastservice_email */


int parse_fastservice_shares_email_and_update_krs_files(\
	char *text, int monthday, int month, int year)
{
int i;
FILE *fileptr;
char temp[READSIZE];
char *ptr;
int a;
char fundname[512];
float close_price;
long volume;
int type;
char fundname_lower_case[512];
struct notering *pa;
struct fsfund *pb;
float current_price;
int data_flag;
char *textptr;
int eot_flag;
int space_cnt;
char *close_price_ptr;
int line_format_error_flag;
int email_format_error_flag;
char datestr[40];
char weekday[20];
char monthstr[20];

if(debug_flag)
	{
	fprintf(stdout,\
	"parse_all_fastservice_shares_emails_and_update_krs_files():\n\
	arg text=%s monthday=%d month=%d year=%d\n",\
	"text dummy", monthday, month, year);
	}

/* argument check */
if(! text) return(0);

if(! delete_all_noteringen() )
	{
	fl_set_object_label(fdui -> portfolio_form_status_display,\
		"could not clear notering list, aborted");

	XSync(fl_get_display(), 0);
	return 0;
	}

email_format_error_flag = 0;
eot_flag =0;
textptr = text;
data_flag = 0;
type = TYPE_STOCK;
while(1)
	{
/*
Fonds		Huidig	Delta	Tijd	Hoog	Laag	Open	Bied	Laat	Volume

AEX-index      	560.52	-9.58	16:30	570.60	559.81	569.29	-	-	-
wolt-kluw c    	34.25	0.05	16:29	34.70	33.80	34.00	34.30	34.35	1453666

Legenda:
*/
	/* this could happen if empty line */
	if(eot_flag) break;
	
	/* copy textptr up to LF to temp */
	i = 0;
	while(1)
		{
		temp[i] = textptr[i];
		if(textptr[i] == 0)
			{
			eot_flag = 1;
			break;
			}/* end if EOT */
		if(textptr[i] == '\n')
			{
			temp[i] = 0;
			textptr++;/* scip the LF */
			textptr += i;
			break;
			}/* end if LF */
		i++;
		}/* end for all chars in a line */
	
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_fastservice_shares_email_and_update_krs_files(): eot_flag=%d temp=%s\n",\
		eot_flag, temp);
		}

	/* test for first line */
	if(strncasecmp(temp, "Fonds", 5) == 0) continue;
	
	/* test for empty line */
	if(strlen(temp) == 0) continue;
	
	/* test for end of data */
	if(strncasecmp(temp, "Legenda", 7) == 0) break;
	
	/*
	extract fund name (according to fastservice), it ends with 2 spaces,
	and convert to lower case
	*/
	line_format_error_flag = 0;
	space_cnt = 0;
	i = 0;
	while(1)
		{
		fundname_lower_case[i] = tolower(temp[i]);
		if(temp[i] == 0)
			{
			line_format_error_flag = 1;/* nothing after fund name */
			email_format_error_flag = 1;
			break;
			}
		/*
		Some long fund names are followed only by a single space and then
		the close price.
		*/
		if(i == START_COLUMN_CLOSE_PRICE)
			{
			fundname_lower_case[i - 1] = 0;
			close_price_ptr = temp + i;
			break;
			}
		
		if(temp[i] == ' ')
			{
			if(space_cnt == 1)
				{
				fundname_lower_case[i - 1] = 0;
				close_price_ptr = temp + i + 1;
				break;
				}
			space_cnt++;
			}
		else space_cnt = 0;
		i++;
		}/* end while  name to lower case */
	if(line_format_error_flag) continue;  /* scip incorrect entry */
	
	/* scanf will scip any leading spaces */
	sscanf(close_price_ptr, "%f", &close_price);
	
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_fastservice_shares_email_and_update_krs_files():\n\
		fundname_lower_case=%s\n\
		close_price_ptr=%s\n\
		close_price=%.2f\n",\
		fundname_lower_case,
		close_price_ptr,\
		close_price);
		}
	
	/*
	test if fundname in fastservice.dat, if not continue, else replace
	with standard name
	*/
	pb = lookup_fsfund(fundname_lower_case);
	if(! pb)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_fastservice_shares_email_and_update_krs_files(): %s not found\n",\
			fundname_lower_case);
			}
		continue;
		}
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_fastservice_shares_email_and_update_krs_files():\n\
		fundname_lower_case=%s pb->ticker=%s\n",\
		fundname_lower_case, pb -> ticker);
		}

	/* use beurs time form subject line */
	/* update fund files (.krs and .csv) */
	/* volume = 0 */
	if(! update_fund_file(\
	pb -> ticker, close_price, 0, monthday, month, year)
	)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_fastservice_email_and_update_krs_files():\n\
			could not update stock %s scipping\n",\
			pb -> ticker);
			}
		continue;
		}/* end if update_fund_file failed */

	}/* end while all lines in text */

if(email_format_error_flag)
	{
	fl_show_alert(\
	"There were format errors in the fastservice email", "ignored", "", 1);
	}

return 1;
} /* end function parse_fastservice_email_and_update_krs_files */


