/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xkra.h"

FD_xkra *fdui;

extern FD_xkra *fd_main_form;

#define NOTERINGEN_HASH_SIZE 100

struct notering
	{
	char *name;
	int type;
	int expiration_month;
	int expiration_year;
	float strike_price;
	int present_flag;
	float current_price;
		
	struct notering *nxtentr;
	struct notering *prventr;
	};
struct notering *noteringtab[NOTERINGEN_HASH_SIZE];


int hash(s)/* form hash value for string s */
char *s;
{
int hashval;

for(hashval = 0; *s != '\0';) hashval += *s++;
/* sum of ascii value of characters divided by tablesize */
return(hashval % NOTERINGEN_HASH_SIZE);
}


struct notering *lookup_notering(char *name)
{
struct notering *pa;

if(debug_flag)
	{
	fprintf(stdout, "lookup_notering(): arg name=%s\n", name);
	}
	
for(pa = noteringtab[hash(name)]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);/* found sequence entry */
	}
return(0); /* not found */
}/* end function lookup_notering */


struct notering *install_notering(char *name)
{
struct notering *pnew, *pnext;
int hashval;

if(debug_flag)
	{
	fprintf(stdout, "install_notering(): arg name=%s\n", name);
	}
	
/* create new structure */
pnew = (struct notering *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get next structure */
hashval = hash(name);
pnext = noteringtab[hashval];/* may be zero, if there was nothing */

/* insert before next structure (if any, else at start) */
noteringtab[hashval] = pnew;

/* set pointers for next structure */
if(pnext) pnext -> prventr = pnew;
	
/* set pointers for new structure */
pnew -> nxtentr = pnext;
pnew -> prventr = 0;/* always inserting at start of chain of structures */
	
return(pnew);/* pointer to new structure */
}/* end function install_notering */


int delete_all_noteringen()
{
struct notering *pa;
int i;

for(i = 0; i < NOTERINGEN_HASH_SIZE; i++)
	{
	while(1)
		{
		pa = noteringtab[i];
		if(! pa) break;

		noteringtab[i] = pa -> nxtentr;

		free(pa -> name);/* free name */
		free(pa);/* free structure */
		}/* end while all structures hashing to this value */ 
	}/* end for all entries in noteringtab */

return 1;
}/* end function delete_all_noteringen */


int mark_if_notering_present_in_list(char *name, int *present_flag)
{
struct notering *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"mark_if_notering_present_in_list(): arg name=%s\n",\
	name);
	}

pa = lookup_notering(name);
if(! pa)
	{
	*present_flag = 0;
	return 1;
	}
if(debug_flag)
	{
	fprintf(stdout,\
	"mark_if_notering_present_in_list(): marking pa -> name=%s\n",\
	pa -> name);
	}

pa -> present_flag = 1;
*present_flag = 1;

return 1;
}/* end function mark_if_notering_present_in_list */


/*
here the files for the different formats that use the above structure
notering
*/
#include "davieff.c"
#include "fastservice.c"
