/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xkra.h"

FD_xkra *fdui;

extern FD_xkra *fd_main_form;


/*
Note:
I have written this without having the format specification of
/var/spool/mail/user
*/

#define IN_START	0
#define IN_HEADER	1
#define IN_BODY		2

struct mail
	{
	char *name;/* this is the mail id long decimal */
	char *_from;
	char *return_path;
	char *received;
	char *date;
	char *from;
	char *message_id;
	char *to;
	char *subject;
	char *status;
	char *x_status;
	char *cc;
	char *reply_to;
	char *mime_version;
	char *content_type;
	char *summary;
	char *text;
	long lines;
	int browser_line;
	struct mail *nxtentr;
	struct mail *prventr;
	}; 
struct mail *mailtab[2]; /* first element points to first entry,
									second element to last entry */


struct mail *lookup_mail(char *name)
{
struct mail *pa;

for(pa = mailtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}

return(0);/*not found*/
}/* end function lookup_mail */


struct mail *install_mail_at_end_of_list(char *name)
{
struct mail *plast, *pnew;
struct mail *lookup_mail();

if(debug_flag)
	{
	fprintf(stdout,\
	"install_mail_at_end_of_list(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return(0);

pnew = lookup_mail(name);
if(pnew)
	{
	/* free previous definition */
	free(pnew -> _from);
	free(pnew -> return_path);
	free(pnew -> received);
	free(pnew -> date);
	free(pnew -> from);
	free(pnew -> message_id);
	free(pnew -> to);
	free(pnew -> subject);
	free(pnew -> status);
	free(pnew -> x_status);
	free(pnew -> cc);
	free(pnew -> reply_to);
	free(pnew -> mime_version);
	free(pnew -> content_type);
	free(pnew -> summary);
	free(pnew -> text);
	return(pnew);/* already there */
	}

/* create new structure */
pnew = (struct mail *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = mailtab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previous structure pointers */
if( !mailtab[0] ) mailtab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
mailtab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_mail_at_end_of_list */


int delete_all_mails()/* delete all entries from table */
{
struct mail *pa;

if(debug_flag) fprintf(stdout, "delete_all_mails() arg none\n");

while(1)
	{	
	pa = mailtab[0];
	if(! pa) break;
	mailtab[0] = pa -> nxtentr;
	free(pa -> _from);
	free(pa -> return_path);
	free(pa -> received);
	free(pa -> date);
	free(pa -> from);
	free(pa -> message_id);
	free(pa -> to);
	free(pa -> subject);
	free(pa -> status);
	free(pa -> x_status);
	free(pa -> cc);
	free(pa -> reply_to);
	free(pa -> mime_version);
	free(pa -> content_type);
	free(pa -> summary);
	free(pa -> text);
	free(pa);/* free structure */
	}/* end while all structures */

mailtab[1] = 0;
if(debug_flag) fprintf(stdout, "returning from delete_all_mails\n");
return(1);
}/* end function delete_all_mail */


int load_incoming_mail(int *messages)
{
/* load from /var/spool/mail/user_name int structure mail*/
struct mail *pa;
int a;
char temp[TEMP_SIZE];
char temp2[TEMP_SIZE];
FILE *user_mail_fptr;
char *ptr;
char *_from;
char *return_path;
char *received;
char *date;
char *from;
char *message_id;
char *to;
char *subject;
char *status;
char *x_status;
char *cc;
char *reply_to;
char *mime_version;
char *content_type;
char *summary;
char *text;
long lines;
long entry;
int first_line_flag;
int where_flag;
char *mail_name;

if(debug_flag)
	{
	fprintf(stdout,\
	"load_incoming_mail(): arg none, using user_name=%s\n",\
	user_name);
	}

(*messages) = 0;

mail_name = (char *) load_setting("fastservice_mail_name");
if(! mail_name)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"load_incoming_mail()\n\
		could not load fastservice_mail_name\n");
		}
	fl_show_alert(\
	"Could not read mail name from",\
	"~/.xkra/setup/fastservice_mail_name",\
	"command cancelled", 1);

	return 0;
	}
	
/* load the structure from the (most recent) /var/spool/mail/user */
sprintf(temp, "/var/spool/mail/%s", mail_name);
if(debug_flag)
	{
	fprintf(stdout, "load_incoming_mail(): opening %s\n", temp);
	}

free(mail_name);

user_mail_fptr = fopen(temp, "r");
if(! user_mail_fptr)
	{
	/* no mail, no file, no error report */
	return(1);
	}

/* clear structure */
delete_all_mails();

/* prevent null pointers */
_from = strsave("");
return_path = strsave("");
received = strsave("");
date = strsave("");
from = strsave("");
message_id = strsave("");
to = strsave("");
subject = strsave("");
status = strsave("");
x_status = strsave("");
cc = strsave("");
reply_to = strsave("");
mime_version = strsave("");
content_type = strsave("");
summary = strsave("");
text = strsave("");
lines = 0;
entry = 0;
first_line_flag = 1;
where_flag = IN_HEADER;
/*ptr = 0;*/ /* keep gcc -Wall happy */

/* read from /var/spool/mail/user_name */
while(1)
	{
	a = readline(user_mail_fptr, temp);/* closes file if EOF */
	if(a == EOF)
		{
		if(where_flag == IN_HEADER) break;/* some format error */
		if(first_line_flag) break;/* no entries in file */

		sprintf(temp2, "%ld", entry);
		pa = (struct mail *) install_mail_at_end_of_list(temp2);
		if(! pa) return(0);

		pa -> _from = strsave(_from);
		pa -> return_path = strsave(return_path);
		pa -> received = strsave(received);
		pa -> date = strsave(date);
		pa -> from = strsave(from);
		pa -> message_id = strsave(message_id);
		pa -> to = strsave(to);
		pa -> subject = strsave(subject);
		pa -> status = strsave(status);
		pa -> x_status = strsave(x_status);
		pa -> cc = strsave(cc);
		pa -> reply_to = strsave(reply_to);
		pa -> mime_version = strsave(mime_version);
		pa -> content_type = strsave(content_type);
		pa -> summary = strsave(summary);
		pa -> text = strsave(text);
		pa -> lines = lines - 1;
		pa -> browser_line = 0;
		
		(*messages)++;

		break;
		}/* end if a == EOF */

if(debug_flag) fprintf(stdout, "%s\n", temp);	

/*
From root@panteltje.pi.net Fri Feb  6 13:50:25 1998
Return-Path: root
Received: (from root@localhost) by panteltje.pi.net (8.6.12/8.6.9) id NAA076
Date: Fri, 6 Feb 1998 13:50:24 +0100
From: root <root@panteltje.pi.net>
Message-Id: <199802061250.NAA07604@panteltje.pi.net>
To: root@panteltje.pi.net
Subject: flap
Status: RO
X-Status:

text
text
*/

	if(where_flag == IN_BODY)
		{
		/*
		It seems that 'From ' in the body is prepended by '>',
		so it is guaranteed never to occur.
		So if it occusrs, it is the start of the next header.
		*/
		ptr = strstr(temp, "From ");
		if(ptr == temp)
			{
			where_flag = IN_HEADER;
			/* store previous header and body in structure */
			sprintf(temp2, "%ld", entry);
			pa = (struct mail *) install_mail_at_end_of_list(temp2);
			if(! pa) return(0);

			pa -> _from = strsave(_from);
			pa -> return_path = strsave(return_path);
			pa -> received = strsave(received);
			pa -> date = strsave(date);
			pa -> from = strsave(from);
			pa -> message_id = strsave(message_id);
			pa -> to = strsave(to);
			pa -> subject = strsave(subject);
			pa -> status = strsave(status);
			pa -> x_status = strsave(x_status);
			pa -> cc = strsave(cc);
			pa -> reply_to = strsave(reply_to);
			pa -> mime_version = strsave(mime_version);
			pa -> content_type = strsave(content_type);
			pa -> summary = strsave(summary);
			pa -> text = strsave(text);
			pa -> lines = lines - 1;
			pa -> browser_line = 0;

			free(_from);
			_from = strsave("");
			free(return_path);
			return_path = strsave("");
			free(received);
			received = strsave("");
			free(date);
			date = strsave("");
			free(from);
			from = strsave("");
			free(message_id);
			message_id = strsave("");
			free(to);
			to = strsave("");
			free(subject);
			subject = strsave("");
			free(status);
			status = strsave("");
			free(x_status);
			x_status = strsave("");
			free(cc);
			cc = strsave("");
			free(reply_to);
			reply_to = strsave("");
			free(mime_version);
			mime_version = strsave("");
			free(content_type);
			content_type = strsave("");
			free(summary);
			summary = strsave("");
			free(text);
			text = strsave("");
			
			(*messages)++;

			entry++;
			}/* end if new header detected, end of body */
		else /* body text */
			{
			/* make space */
			ptr = malloc(strlen(text) + strlen(temp) + strlen("\n") + 1);
			sprintf(ptr, "%s%s\n", text, temp);
			free(text);
			text = ptr;
			lines++;
			}/* end if body text */
		}/* end if whereflag == IN_BODY */

	if(where_flag == IN_HEADER)
		{
		/* End header detection, empty line in header. */
		if(strlen(temp) == 0)
			{
			lines = 0;
			where_flag = IN_BODY;
			/* clear any previous text */
			free(text);
			text = strsave("");
			}/* end if 'From ' */
		else /* header */
			{
			ptr = strstr(temp, "From ");
			if(ptr == temp)
				{
				free(_from);
				_from = strsave(ptr + 5);
				}
			ptr = strstr(temp, "From: ");
			if(ptr == temp)
				{
				free(from);
				from = strsave(ptr + 6);
				}
			ptr = strstr(temp, "To: ");
			if(ptr == temp)
				{
				free(to);
				to = strsave(ptr + 4);
				}
			ptr = strstr(temp, "Subject: ");
			if(ptr == temp)
				{
				free(subject);
				subject = strsave(ptr + 9);
				}
			ptr = strstr(temp, "Status: ");
			if(ptr == temp)
				{
				free(status);
				status = strsave(ptr + 8);
				}
			ptr = strstr(temp, "Reply-To: ");
			if(ptr == temp)
				{
				free(reply_to);
				reply_to = strsave(ptr + 10);
				}
			ptr = strstr(temp, "Summary: ");
			if(ptr == temp)
				{
				free(summary);
				summary = strsave(ptr + 9);
				}
			}/* end if not end header */
		}/* end if where_flag == IN_HEADER */

	first_line_flag = 0;
	}/* end while all lines in /var/spool/mail/user_name */ 

return(1);
}/* end function load_incoming_mail */


int check_mail_local()
{
int messages;
int a;
struct stat *statptr;
static time_t last_modification_time;
char temp[TEMP_SIZE];

if(debug_flag)
	{
	fprintf(stdout, "check_mail_local(): arg none\n");
	}

/*
Only want to access the mail file (and read it) if it has been modified,
else, if it is very long, the system will slow down.
*/
/* get stat /var/spool/mail */
sprintf(temp, "/var/spool/mail/%s", user_name);
if(debug_flag)
	{
	fprintf(stdout, "opening %s\n", temp);
	}

statptr = (struct stat*) malloc(sizeof(struct stat) );
a = stat(temp, statptr);
if(a == 0)/* file exists */
	{
	if(statptr -> st_mtime == last_modification_time)
		{
		free(statptr);
		if(debug_flag)
			{
			fprintf(stdout,\
			"check_local_mail(): %s was not modified returning\n", temp);
			}
		return(0);
		}
	last_modification_time = statptr -> st_mtime;
	
	if(debug_flag)
		{
		fprintf(stdout, "check_local_mail(): %s was modified\n", temp);
		}
	}/* end if mail file exists */
free(statptr);

messages = 0;
a = load_incoming_mail(&messages);
if(! a) return(0);

return(1);
}/* end function check_mail_local */


int add_fastservice_shares_to_notering_list()
{
int a;
struct mail *pa;
char temp[512];
int found_fastservice_email_flag;
time_t beurs_time;
time_t last_update_time;
char *data_read;
char *ptr;
int year, month, monthday;
struct tm *petm;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_fastservice_shares_to_notering_list(): arg none\n");
	}

/* allocate space for a structure time */
petm = (struct tm *) malloc(sizeof(struct tm) );
if(! petm)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"add_fastservice_shares_to_notering_list():\n\
		malloc struct tm failed, aborting command\n");
		}
	return 0;
	}

found_fastservice_email_flag = 0;
for(pa = mailtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* test if from fastservice */
	if(! strcasestr(pa -> from, "<stocks@fastservice.com>") ) continue;

	/* test if subject 'beurs' (Subject: Beurs 99/08/19 17:27, AEX 560.52) */
	if(strncasecmp(pa -> subject, "Beurs", 5) != 0) continue;
	
	/* compute beurs_time */
	a = sscanf(pa -> subject, "Beurs %d/%d/%d",\
	&year, &month, &monthday);	
	if(a < 3)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"add_fastservice_shares_to_notering_list():\n\
			could not get date from %s\n scipping", pa -> subject); 
			}
		continue;
		}		
	petm -> tm_sec = 0;
	petm -> tm_min = 0;
	petm -> tm_hour = 0;
	petm -> tm_mday = monthday;
	petm -> tm_mon = month - 1;
	
	/* fix year 2000 problem */
	if(year == 99) petm -> tm_year = year;
	else petm -> tm_year = year + 100;
	
	petm -> tm_isdst = -1;/* daylight savings time n.a. */
	/* the following are not implemented (see /usr/include/time.h) */
	/* petm -> tm_gmtoff = 0;*/
	/* strcp(petm -> tm_zone, ""); */ /* time zone 3 letters */
	beurs_time = mktime(( struct tm *) petm);
	if(debug_flag)
		{
		fprintf(stdout,\
		"add_fastservice_shares_to_notering_list():\n\
		beursTime=%lu\n", (long)beurs_time);
		}
	if(beurs_time == -1)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"add_fastservice_shares_to_notering_list():\n\
			mktime() failed scipping\n");
			}
		continue;
		}

	/* get last_update_time */
	data_read = (char *) load_setting("fastservice_time");
	if(! data_read)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"add_fastservice_shares_to_notering_list():\n\
			could not load fastservice_time\n");
			}
		fl_show_alert(\
		"Could not read fastservice time file", "creating it", "", 1);

		last_update_time = 0;
		}
	else
		{
		last_update_time = atol(data_read);
		free(data_read);
		}

	/* scip if mail beurs time <= last update time */
	if(beurs_time <= last_update_time) continue;
	
	found_fastservice_email_flag = 1;

	/* ******NOTE: only using the most recent email here**** */

	/* clear the notering list */
	if(! delete_all_noteringen() )
		{
		fl_set_object_label(fdui -> portfolio_form_status_display,\
			"could not clear notering list, aborted");

		XSync(fl_get_display(), 0);
		return 0;
		}

	/* ad the data to the notering list */
	if(! parse_fastservice_shares_email(pa -> text) )
		{
		fl_show_alert(\
	"Cannot parse fastservice email", pa -> subject, "command aborted", 1);
		free(petm);
		return 0;
		}
	
	/*
	XXXXXXX
	test if any of the portfolio entries are present in the notering list,
	if present mark entry in hashed list 
	*/
	fl_set_object_label(fdui -> portfolio_form_status_display,\
		"testing for double entries");
	XSync(fl_get_display(), 0);

	if(! mark_noteringen_already_present() )
		{
		fl_set_object_label(fdui -> portfolio_form_status_display,\
			"could not mark already present, aborted");
		XSync(fl_get_display(), 0);
	
		fl_show_alert("Cannot mark noteringen already present", "", "", 1);
		return 0;	
		}

	/* set last updated date (for each file) */
	sprintf(temp, "%lu", beurs_time);
	if(! save_setting("fastservice_time", temp) )
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"add_fastservice_shares_to_notering_list():\n\
			save_setting() failed for temp=%s aborting\n", temp);
			}
		free(petm);
		return(0);
		}
	}/* end for all entries in mail list */

if(! found_fastservice_email_flag)
	{
	fl_show_alert("No (new) fastservice emails found", "", "", 1);
	}

free(petm);

return 1;
}/* end function add_fastservice_shares_to_notering_list */


int parse_all_fastservice_shares_emails_and_update_krs_files()
{
int a;
struct mail *pa;
char temp[512];
int found_fastservice_email_flag;
time_t beurs_time;
time_t last_update_time;
char *data_read;
char *ptr;
int year, month, monthday;
struct tm *petm;

if(debug_flag)
	{
	fprintf(stdout,\
	"parse_all_fastservice_emails_and_update_krs_files(): arg none\n");
	}

/* allocate space for a structure time */
petm = (struct tm *) malloc(sizeof(struct tm) );
if(! petm)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_all_fastservice_emails_and_update_krs_files():\n\
		malloc struct tm failed, aborting command\n");
		}
	return 0;
	}

found_fastservice_email_flag = 0;
for(pa = mailtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* test if from fastservice */
	if(! strcasestr(pa -> from, "<stocks@fastservice.com>") ) continue;

	/* test if subject 'beurs' (Subject: Beurs 99/08/19 17:27, AEX 560.52) */
	if(strncasecmp(pa -> subject, "Beurs", 5) != 0) continue;
	
	/* compute beurs_time */
	a = sscanf(pa -> subject, "Beurs %d/%d/%d",\
	&year, &month, &monthday);	
	if(a < 3)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_all_fastservice_emails_and_update_krs_files():\n\
			could not get date from %s\n scipping", pa -> subject); 
			}
		continue;
		}		
	petm -> tm_sec = 0;
	petm -> tm_min = 0;
	petm -> tm_hour = 0;
	petm -> tm_mday = monthday;
	petm -> tm_mon = month - 1;
	
	/* fix year 2000 problem */
	if(year == 99) petm -> tm_year = year;
	else petm -> tm_year = year + 100;
	
	petm -> tm_isdst = -1;/* daylight savings time n.a. */
	/* the following are not implemented (see /usr/include/time.h) */
	/* petm -> tm_gmtoff = 0;*/
	/* strcp(petm -> tm_zone, ""); */ /* time zone 3 letters */
	beurs_time = mktime(( struct tm *) petm);
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_all_fastservice_emails_and_update_krs_files():\n\
		beursTime=%lu\n", (long)beurs_time);
		}
	if(beurs_time == -1)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_all_fastservice_emails_and_update_krs_files():\n\
			mktime() failed scipping\n");
			}
		continue;
		}

	/* get last_update_time */
	data_read = (char *) load_setting("fastservice_krs_time");
	if(! data_read)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_all_fastservice_emails_and_update_krs_files():\n\
			could not load fastservice_time\n");
			}
		fl_show_alert(\
		"Could not read fastservice time file", "creating it", "", 1);

		last_update_time = 0;
		}
	else
		{
		last_update_time = atol(data_read);
		free(data_read);
		}

	/* scip if mail beurs time <= last update time */
	if(beurs_time <= last_update_time) continue;
	
	found_fastservice_email_flag = 1;

	/* convert to normal year */
	year += 1900;

	/* ad the data to the notering list */
	if(! parse_fastservice_shares_email_and_update_krs_files(\
		pa -> text, monthday, month, year) )
		{
		fl_show_alert(\
	"Cannot parse fastservice email", pa -> subject, "command aborted", 1);
		free(petm);
		return 0;
		}
	
	/* set last updated date (for each file) */
	sprintf(temp, "%lu", beurs_time);
	if(! save_setting("fastservice_krs_time", temp) )
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"parse_all_fastservice_emails_and_update_krs_files():\n\
			save_setting() failed for temp=%s aborting\n", temp);
			}
		free(petm);
		return(0);
		}
	}/* end for all entries in mail list */

if(! found_fastservice_email_flag)
	{
	fl_show_alert("No (new) fastservice emails found", "", "", 1);
	}

free(petm);

return 1;
}/* end function parse_all_fastservice_shares_emails_and_update_krs_files */


