/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xkra.h"

FD_xkra *fdui;


int show_option_evaluation_form()
{
char temp[512];
char filename[512];
char *ptr;
char datestr[40];
char weekday[20];
char monthstr[20];
int year, month, monthday, hour, minute, second;

/* create a default filename in the format 19981130.CSV */
ogettime(weekday, monthstr, &monthday, &hour, &minute, &second, &year);
month = month_to_digit(monthstr);
if( (month < 1) || (month > 12) )
	{
	fl_show_alert("Ilegal month read from computer", monthstr, "", 1);
	return 0;
	}

sprintf(datestr, "%4d%02d%02d", year, month, monthday);
sprintf(temp, "%s%s.CSV.options", file_path, datestr);

/* set global, changing something in the file selector menu will also change this global */
if(option_evaluation_filename) free(option_evaluation_filename);
option_evaluation_filename = strsave(temp);
if(! option_evaluation_filename) return 0;

if(! load_options() )
	{
	fl_show_alert("No options are loaded", "use import menu", "", 1);
/*	return 0;*/
	}

if(! show_options(SHOW_TOP) )
	{
	fl_show_alert("could not show sorted options", "", "", 1);
/*	return 0;*/
	}

fl_show_form(\
	fdui -> option_evaluation_form, FL_PLACE_CENTER, FL_UP_FRAME, "");

}/* end function show_option_evaluation_form */


int fill_stock_list(char *filename)
{
int a;
char temp[512];
FILE *fptr;

if(debug_flag)
	{
	fprintf(stdout, "fill_stock_list(): arg filename=%s\n",\
	filename);
	}

fptr = fopen(filename, "r");
if(! fptr)
	{
	fl_show_alert("Cannot open file", filename, "for read", 1);

	return 0;
	}

while(1)
	{
	a = readline(fptr, temp);
	if(a == EOF)
		{
		return 1;
		} 
	/* ignore comments */
	if(temp[0] == '#') continue;
	
	/* scip lines starting with spaces */
	if(temp[0] == ' ') continue;
	if(! add_stock(temp) )
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"fill_stock_list(): cannot add stock %s\n", temp);
			}
		return 0;
		}
	}/* end while all lines */

return 1;
}/* end function fill_stock_list */


int import_and_evaluate_davieff_options_file()
{
char temp[512];
char filename[512];
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"import_and_evaluate_davieff_optionsfile(): arg none\n");
	}

/* get list of stocks to process from filepath/fund_list.dat */

sprintf(filename, "%sfund_list.dat", file_path);

sprintf(temp,\
	"reading list of funds to process from %s", filename);
fl_set_object_label(fdui -> option_evaluation_form_status_display,\
	temp);
XSync(fl_get_display(), 0);

if(! fill_stock_list(filename) )
	{
	sprintf(temp, "cannot read file stock list");
	fl_set_object_label(fdui -> option_evaluation_form_status_display,\
		temp);
	XSync(fl_get_display(), 0);
	return 0;
	}

fl_set_object_label(fdui -> option_evaluation_form_status_display, "");
XSync(fl_get_display(), 0);

/* get filename from user */
ptr = (char *) fl_show_fselector(\
	"Davieff options file?", home_dir, "*", option_evaluation_filename);
if(! ptr) return;

/* modify global for latest entry, so if wrong, user does not have to retype everything */
if(option_evaluation_filename) free(option_evaluation_filename);
option_evaluation_filename = strsave(ptr);
if(! option_evaluation_filename) return 0;

/* parse the file (this will add the entries to the data base) */
sprintf(temp, "parsing %s", ptr);
fl_set_object_label(fdui -> option_evaluation_form_status_display,\
temp);
XSync(fl_get_display(), 0);
if(! parse_davieff_option_file_for_evaluation(ptr) )
	{
	sprintf(temp, "could not parse %s", ptr);
	fl_set_object_label(fdui -> option_evaluation_form_status_display,
	temp);
	XSync(fl_get_display(), 0);
	return 0;
	}

sprintf(temp, "sorting options");
fl_set_object_label(fdui -> option_evaluation_form_status_display, temp);
XSync(fl_get_display(), 0);
if(! sort_options() )
	{
	sprintf(temp, "could not sort options");
	fl_set_object_label(fdui -> option_evaluation_form_status_display,\
	temp);
	XSync(fl_get_display(), 0);
	return 0;
	}

if(! show_options(SHOW_TOP) )
	{
	sprintf(temp, "could not show sorted options");
	fl_set_object_label(fdui -> option_evaluation_form_status_display,\
	temp);
	XSync(fl_get_display(), 0);
	return 0;
	}

/* clear status display */
fl_set_object_label(fdui -> option_evaluation_form_status_display, "");
XSync(fl_get_display(), 0);

return 1;
}/* end function import_and_evaluate_davieff_options_file */


