/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xkra.h"

FD_xkra *fdui;

extern FD_xkra *fd_main_form;

char *selected_item;
int portfolio_form_show_have_only_flag;


struct item
	{
	char *name; /* combination of properties so it is unique */
	int status; /* not used was NEW_ITEM HAVE_ITEM */
	int browser_line;	
	time_t last_updated_time;

	float trade_price;
	float quantity;

	int type; /* TYPE_CALL TYPE_PUT TYPE_STOCK TYPE_BOND */
	int expiration_month; /* 1 through 12 */
	int expiration_year;
	float strike_price;
	
	struct item *nxtentr;
	struct item *prventr;
	};
struct item *itemtab[2]; /* first element points to first entry,
			second element to last entry */


struct item *lookup_item(char *name)
{
struct item *pa;

/*pa points to next entry*/
for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return(0);/*not found*/
}/* end function lookup_item */


struct item *install_item_at_end_of_list(char *name)
{
struct item *plast, *pnew;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_item_at_end_off_list(): arg name=%s\n", name);
	}

/*pnew = lookup_item(name);*/
/*if(pnew)*/
/*	{*/
	/* free previous definition */
/*	free(pnew -> subject); */
/*	return(pnew); */ /* already there */
/*	}*/

/* create new structure */
pnew = (struct item *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = itemtab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !itemtab[0] ) itemtab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
itemtab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_item_at_en_of_list */


int delete_item(char *name)/* delete entry from double linked list */
{
struct item *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stdout, "delete_item(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return(0);

pa = itemtab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return(0);

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify itemtab[0] */
	if(pprev == 0) itemtab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify itemtab[1] */
	if(pnext == 0) itemtab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return(1);
	}/* end for all structures */
}/* end function delete_item */


int delete_all_items()/* delete all entries from table */
{
struct item *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_items() arg none\n");
	}

while(1)
	{	
	pa = itemtab[0];
	if(! pa) break;
	itemtab[0] = pa -> nxtentr;
	free(pa -> name);
	free(pa);/* free structure */
	}/* end while all structures */

itemtab[1] = 0;
return(1);
}/* end function delete_all_items */


int swap_item_position(struct item *ptop, struct item *pbottom)
{
struct item *punder;
struct item *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_item_position(): swapping top=%lu bottom=%lu\n", ptop, pbottom);
	}

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_item_position(): punder=%lu\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_item_position(): pabove=%lu\n", pabove);
	}

/* the next pointer in punder (or fundtab[0]) must now point to ptop */
if(! punder)
	{
	itemtab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in fundtab */
if(! pabove)
	{
	itemtab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_item_position */


int load_items()
{
int a;
FILE *items_dat_file;
char temp[READSIZE];
char pathfilename[TEMP_SIZE];
struct item *pa;
int first_item_flag;
int status;
int browser_line; /* used to lookup name (= id) if clicked in display,
						not saved on disk, assigned each time list
						is displayed */
int status_line;
time_t last_updated_time;
int some_new_flag;
float quantity;
int month;
int year;
float strike_price;
float trade_price;
int type;

if(debug_flag)
	{
	fprintf(stdout, "load_items() arg none using selected_item=%s\n",\
	selected_item);
	}
	
delete_all_items();

sprintf(pathfilename, "%s/items.dat", file_path);
items_dat_file = fopen(pathfilename, "r");
if(! items_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}
	fl_show_alert("Cannot open file", pathfilename, "for read", 1);

	return(0);
	} 
status_line = 1;
first_item_flag = 1;
while(1)
	{
	/* read header line or first line of entry */
	a = readline(items_dat_file, temp);

	if(status_line)
		{
		sscanf(temp,\
		"%d %f %f",\
		&some_new_flag, &portfolio_form_value, &portfolio_form_cash);
		status_line = 0;
		if(a == EOF) return 1;/* only this line, no entries */
		continue;
		}/* end if status line */

	if(a == EOF) break;/* file format error, second line entry missing  */

	/* this first, temp used in next readline */
	/* create a structure entry */
	pa = install_item_at_end_of_list(temp);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_items: cannot install item %s\n", temp);
			}
		return(0);
		}

	/* read second line of entry */
	a = readline(items_dat_file, temp);
	if(a == EOF) break;
	sscanf(temp,\
	"%d %d %lu %f %d %d %f %f %d",\
	&status,\
	&browser_line,
	&last_updated_time,\
	&quantity,\
	&month,\
	&year,\
	&strike_price,\
	&trade_price,\
	&type\
	);
	
	/* data to structure */
	pa -> status = status;
	pa -> browser_line = browser_line;
	pa -> last_updated_time = last_updated_time;

	pa -> quantity = quantity;
	pa -> expiration_month = month;
	pa -> expiration_year = year;
	pa -> strike_price = strike_price;
	pa -> trade_price = trade_price;
	pa -> type = type;
		
	if(first_item_flag)
		{
		selected_item = strsave(pa -> name);
		first_item_flag = 0;
		}

	/* if no more lines, this was the last entry, return OK */
/*	if(a == EOF) return 1;*/
	}/* end while all lines in items.dat */

/* return error */
return 0;
}/* end function load items */


int save_items()
{
FILE *items_dat_file;
char pathfilename[TEMP_SIZE];
char pathfilename2[TEMP_SIZE];
struct item *pa;
extern char *strsave();
int some_new_flag;
FILE *fptr;

if(debug_flag)
	{
	fprintf(stdout, "save_items() arg none\n");
	}

sprintf(pathfilename, "%s/items.tmp", file_path);
items_dat_file = fopen(pathfilename, "w");
if(! items_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"could not open file %s for write\n", pathfilename);
		}
	fl_show_alert("Cannot open file", pathfilename, "for write", 1);	

	return(0);
	} 

some_new_flag = 0;
fprintf(items_dat_file, "%d %.2f %.2f\n",\
some_new_flag, portfolio_form_value, portfolio_form_cash);

for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(items_dat_file,\
	"%s\n%d %d %lu %.2f %d %d %.2f %.2f %d\n",\
	pa -> name,\
	pa -> status,\
	pa -> browser_line,\
	/* used to lookup name (= id) if clicked in display,
							not saved on disk, assigned each time list
							is displayed */
	pa -> last_updated_time,\
	
	pa -> quantity,\
	pa -> expiration_month,\
	pa -> expiration_year,\
	pa -> strike_price,\
	pa -> trade_price,\
	pa -> type);
	
	}/* end while all elements in list */

fclose(items_dat_file);

if(debug_flag)
	{
	fprintf(stdout, "save_items(): backing up items.dat\n");
	}

/* set some path file names */
sprintf(pathfilename, "%s/items.dat", file_path);
sprintf(pathfilename2, "%s/items.dat~", file_path);

/* unlink the old .dat~ */
unlink(pathfilename2);

/* test if there is a /.dat file (not first time) */
fptr = fopen(pathfilename, "r");
if(fptr)
	{
	fclose(fptr);
	/* rename .dat to .dat~ */
	if( rename(pathfilename, pathfilename2) == -1)
		{
		if(debug_flag)
			{
			fprintf(stdout, "save_items(); rename %s into %s failed\n",\
			pathfilename, pathfilename2);
			}
		return(0);
		}
	}/* end if .dat file existed */
else
	{
	fl_show_alert("Warning", pathfilename, "does not exist, creating it", 1);
	}		

/* rename .tmp to .dat */
sprintf(pathfilename2, "%s/items.tmp", file_path);
if( rename(pathfilename2, pathfilename) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_items(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return(0);
	}

return(1);/* ok */
}/* end function save_items */


int show_portfolio_form()
{
char temp[512];
char datestr[40];
char weekday[20];
char monthstr[20];
int year, month, monthday, hour, minute, second;

if(debug_flag)
	{
	fprintf(stdout, "show_portfolio_form(): arg none\n");
	}

load_items();
load_transactions();

/*
if(! delete_expired_options() )
	{
	fl_show_alert("Could not delete expired options", "ignoring", "", 1);
	}
*/

/*
fill in input field defaults for davieff shares and davieff options import
file selectors.
*/
/* create a default filename in the format  19981130.CSV */
ogettime(weekday, monthstr, &monthday, &hour, &minute, &second, &year);
month = month_to_digit(monthstr);
if( (month < 1) || (month > 12) )
	{
	fl_show_alert("Ilegal month read from computer", monthstr, "", 1);
	return 0;
	}

sprintf(datestr, "%4d%02d%02d", year, month, monthday);

/* for shares */
sprintf(temp, "%s%s.CSV.shares", file_path, datestr);
/*
set global, changing something in the file selector menu will also change
this global
*/
if(davieff_shares_filename) free(davieff_shares_filename);
davieff_shares_filename = strsave(temp);
if(! davieff_shares_filename) return 0;

/* for options */
sprintf(temp, "%s%s.CSV.options", file_path, datestr);
/*
set global, changing something in the file selector menu will also change
this global
*/
if(davieff_options_filename) free(davieff_options_filename);
davieff_options_filename = strsave(temp);
if(! davieff_options_filename) return 0;

/* select market browser as visible */
select_portfolio_form_browser(MARKET_BROWSER);

fl_show_form(fd_main_form -> portfolio_form, FL_PLACE_CENTER, FL_UP_FRAME, "");

portfolio_form_modified_flag = 0;

return(1);
}/* end function show_portfolio_form */


int item_to_line(char *url, int *line)
{
struct item *pa;

if(debug_flag)
	{
	fprintf(stdout, "item_to_line(): arg url=%s\n", url);
	}

/* argument check */
if(! url) return(0);

pa = lookup_item(url);
if(! pa) return(0);

*line = pa -> browser_line;
return(1);
}/* end function item_to_line */


int show_items(int position)
{
char temp[TEMP_SIZE];
struct item *pa;
char formatstr[20];
int browser_line;
int browser_topline;
char *strsave();
int first_new_item_line;
int find_first_new_item_flag;
int maxline;
int a, i;
char *ptra;
char *ptrb;
char *ptrc;
int match_failed_flag;
int last_string_flag;
char *filter_keywords;
int text_length;
char temp2[80];
float item_value;
int double_quote_flag;
float account_value;
float shares_value;
float options_value;

if(debug_flag)
	{
	fprintf(stdout, "show_items(): arg position=%d\n", position);
	}

filter_keywords =\
	(char*)fl_get_input(fdui -> portfolio_form_market_filter_input_field);

/* remember vertical slider position */
browser_topline =\
	fl_get_browser_topline(fdui -> portfolio_form_market_browser);

fl_set_object_label(fdui -> portfolio_form_status_display,\
	"building display, this may take a while");
XSync(fl_get_display(), 0);

fl_freeze_form(fd_main_form -> portfolio_form);

fl_clear_browser(fdui -> portfolio_form_market_browser);

first_new_item_line = -1;/* keep gcc -Wall from complaining */
find_first_new_item_flag = 1;
browser_line = 1;
for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(portfolio_form_show_owned_only_flag)
		{
		if(pa -> quantity == 0)
			{
			/* to prevent false finds in double click mark not existing */
			pa -> browser_line = 0;
			continue;
			}
		}

	/* test if any filter keywords */
	if(filter_keywords[0])/* some filter */
		{
		/* test if filter keywords present in item name */
		/* for all entries in filter */
		double_quote_flag = 0;
		ptra = filter_keywords;
		ptrb = strsave(filter_keywords);/* create space */
		match_failed_flag = 0;
		last_string_flag = 0;
		while(1)
			{
			ptrc = ptrb;/* point to start availeble space */
			while(1)/* string in filter to ptrb */
				{
				*ptrc = *ptra;
				if(! *ptra) break;/* end filter */
				
				if(*ptra == '"')/* double quote */
					{
					double_quote_flag = 1 - double_quote_flag;
					ptra++;
					continue;
					}
				if(! double_quote_flag)
					{
					if(*ptra == ' ')/* end of string in filter */
						{
						*ptrc = 0;
						ptra++;/* scip the space */
						break;
						}
					}
				ptra++;
				ptrc++;
				}/* end while all characters in string */
			/* test if string present in name */
			/* test for case sensitive */
			if(portfolio_form_market_filter_case_sensitive_flag)
				{
				if(! strstr(pa -> name, ptrb) )
					{
					match_failed_flag = 1;
					break;
					}
				}
			else /* not case sensitive */
				{
				if(! strcasestr(pa -> name, ptrb) )
					{
					match_failed_flag = 1;
					break;
					}
				}
			if(! *ptra) break;/* end filter */
			}/* end while all strings in filter */	
		free(ptrb);/* release space */
		if(match_failed_flag)
			{
			/* to prevent false finds in double click mark not existing */
			pa -> browser_line = 0;
			continue;
			}
		}/* end if filter */

	if(position == SHOW_UNREAD_AT_TOP)
		{
		/* want to show the first unaccessed at top of display */
		if(find_first_new_item_flag)
			{
			if(pa -> quantity == 0)
				{
				first_new_item_line = browser_line;
				find_first_new_item_flag = 0;
				}
			}
		}
	
	strcpy(formatstr, "@f");

	if(pa -> quantity > 0) strcat(formatstr, "@C18"); /* long is dark green */
	else if(pa -> quantity < 0) strcat(formatstr, "@C1"); /* short is red */
	else if(pa -> quantity == 0) strcat(formatstr, "@C0"); /* black */

	/* do some formatting */
	sprintf(temp, "%s %7.2f", formatstr, pa -> quantity);

	#define START_QUANTITY	0
	#define START_NAME		10
	#define START_PRICE		46

	/* not format string has different length for different colors */
	text_length = strlen(temp) - strlen(formatstr);
	i = 0;
	while(1)
		{
		if(i > START_QUANTITY + text_length) strcat(temp, " ");
		if(i >= START_NAME) break;
		i++;
		}

	strcat(temp, pa -> name);
	
	text_length = strlen(temp) - strlen(formatstr);
	i = 0;
	while(1)
		{
		if(i > START_NAME + text_length) strcat(temp, " ");
		if(i >= START_PRICE) break;
		i++;
		}

	item_value = pa -> quantity * pa -> trade_price;
	sprintf(temp2, "prc %7.2f  vl %7.2f",\
		pa -> trade_price, item_value);

	strcat(temp, temp2);

	fl_add_browser_line(fdui -> portfolio_form_market_browser, temp);
	pa -> browser_line = browser_line;/* first one is 1 */
	browser_line++;
	}/* end for all elements in list */
	
/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(\
		fdui -> portfolio_form_market_browser, first_new_item_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last item visible */
	maxline = fl_get_browser_maxline(fdui -> portfolio_form_market_browser);
	fl_set_browser_topline(fdui -> portfolio_form_market_browser, maxline);
	}
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> portfolio_form_market_browser, 1);
	}
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(\
		fdui -> portfolio_form_market_browser, browser_topline);
	}	

fl_unfreeze_form(fd_main_form -> portfolio_form);

fl_set_object_label(fdui -> portfolio_form_status_display, "");
XSync(fl_get_display(), 0);

if(! calculate_portfolio_value(\
	&portfolio_form_value, &account_value, &shares_value, &options_value) )
	{
	fl_show_alert("Cannot calculate portfolio value", "", "", 1);
	return 0;
	}

sprintf(temp, "%.2f", portfolio_form_value);
fl_set_input(fdui -> portfolio_form_value_input_field, temp);

sprintf(temp, "%.2f", portfolio_form_cash);
fl_set_input(fdui -> portfolio_form_cash_input_field, temp);

return(1);
}/* end function show_items */


char *line_to_item(int line)
{
struct item *pa;

if(debug_flag)
	{
	fprintf(stdout, "line_to_item(): arg line=%d\n", line);
	}

/* argument check */
if(line < 0) return(0);

for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line == line)
		{
		return(pa -> name);
		}
	}

/* no save_items, nothing was changed */
return(0);
}/* end function line_to_item_id */


void replace(char *text, char *string, char *replace, char *result)
{
if(debug_flag)
	{
	fprintf(stdout,\
	"replace(): arg text=%s string=%s replace=%s\n",\
	text, string, replace);
	}

while(1)
	{
	if( (char *)strstr(text, string) == text)
		{
		*result = 0;
		strcat(result, replace);
		text += strlen(string);
		result += strlen(replace);
		}
	*result = *text;
	if(*text == 0)break;
	result++;
	text++;
	}
}/* end function replace */


int set_item_status(char *item, int status)
{
struct item *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_item_status(): arg item=%s status=%d\n", item, status);
	}

/* argument check */
if(! item) return(0);

pa = lookup_item(item);
if(! pa) return(0);

pa -> status = status;

return(1);
}/* end function set_item_status */
 

int get_item_status(char *item, int *status)
{
struct item *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"get_item_status(): arg item=%s\n", item);
	}

/* argument check */
if(! item) return(0);

pa = lookup_item(item);
if(! pa) return(0);

*status = pa -> status;

return(1);
}/* end function get_item_status */
 

int set_item_update_time(char *item_in_progress, time_t now)
{
struct item *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_item_update_time(): arg item_in_progress=%s now=%ld\n",\
	item_in_progress, now);
	}

if(! item_in_progress) return(0);

pa = lookup_item(item_in_progress);
if(! pa) return(0);

pa -> last_updated_time = now;

return(1);
}/* end function set_item_update_time */


char *digit_to_month(int month)
{
char *mstr[]=\
{
"ILLEGAL", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",\
"Aug", "Sept", "Oct", "Nov", "Dec"};
char temp[20];

if(debug_flag)
	{
	fprintf(stdout,\
	"digit_to_month(): arg month=%d\n",\
	month);
	}

if( (month > 0) && (month < 13) ) return strsave(mstr[month]);


sprintf(temp, "%d", month);
if(debug_flag)
	{
	fprintf(stdout, "digit_to_month(): %d illegal month\n", month);
	}
fl_show_alert("digit_to_month():", temp, "illegal month", 1);

return 0;
}/* end function digit_to_month */


int add_item_to_portfolio(\
char *fund, int type, int month, int year, float strike_price,\
int test_exist_flag, float transaction_price)
{
int a, i;
struct item *pa;
char temp[READSIZE];
char *fundname_lower_case;
char *month_name;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_item_to_portfolio(): arg\n\
	fund=%s type=%d month=%d year=%d strike_price=%.2f\n\
	transaction_price=%.2f\n",\
	fund, type, month, year, strike_price, transaction_price);
	}

/* argument check */
if(! fund) return 0;
if( (type != TYPE_CALL) && (type != TYPE_PUT) && (type != TYPE_STOCK) &&\
(type != TYPE_BOND) )
	{
	fl_show_alert("Unkown type", "Legal are: call put stock bond", "", 1);
	return 0;
	}

if( (type == TYPE_CALL) || (type == TYPE_PUT) || (type == TYPE_BOND) )
	{
	if(year < 1900) return 0;

	if(month == 0) return 0;
	month_name = digit_to_month(month);
	if(! month_name) return 0;

	if(strike_price < 0) return 0;
	}

/* fund to lower case */
fundname_lower_case = strsave(fund);/* should be same length  */
if(! fundname_lower_case) return 0;
i = 0;
while(1)
	{
	fundname_lower_case[i] = tolower(fund[i]);
	if(fund[i] == 0) break;
	i++;
	}

/* create an unique identifier */
if(type == TYPE_STOCK)
	{
	/* aab */
	sprintf(temp,\
	"%s",\
	fundname_lower_case);
	free(fundname_lower_case);
	}
else if(type == TYPE_CALL)
	{
	/* aab call oct 1999 22.00 */
	sprintf(temp,\
	"%s call %s %d %.2f",\
	fundname_lower_case, month_name, year, strike_price);
	free(fundname_lower_case);
	free(month_name);
	}
else if(type == TYPE_PUT)
	{
	/* aab put oct 1999 22.00 */
	sprintf(temp,\
	"%s put %s %d %.2f",\
	fundname_lower_case, month_name, year, strike_price);
	free(fundname_lower_case);
	free(month_name);
	}
else if(type == TYPE_BOND)
	{
	/* somebond */
	sprintf(temp,\
	"%s bond %s %d %.2f",\
	fundname_lower_case, month_name, year, strike_price);
	free(fundname_lower_case);
	free(month_name);
	}
else
	{
	free(fundname_lower_case);
	return 0;
	}

if(test_exist_flag)
	{
	/* test entry already if exists */
	pa = lookup_item(temp);
	if(pa)
		{
		fl_show_alert("Entry already exists", temp, "", 1);
		return 0;
		}
	}/* end if test_exist_flag */
	
/* create new entry */
pa = install_item_at_end_of_list(temp);
if(! pa) return 0;

pa -> status = 0; /* not used */
pa -> browser_line = 0;	
pa -> last_updated_time = time(0);
pa -> type = type;
pa -> quantity = 0;
pa -> trade_price = transaction_price;

if(type == TYPE_CALL)
	{
	pa -> expiration_month = month;
	pa -> expiration_year = year;
	pa -> strike_price = strike_price;
	}
if(type == TYPE_PUT)
	{
	pa -> expiration_month = month;
	pa -> expiration_year = year;
	pa -> strike_price = strike_price;
	}
else if(type == TYPE_STOCK)
	{
	pa -> expiration_month = 0;
	pa -> expiration_year = 0;
	pa -> strike_price = 0;
	}
else if(type == TYPE_BOND)
	{
	pa -> expiration_month = month;
	pa -> expiration_year = year;
	pa -> strike_price = strike_price;
	}	

if(! test_exist_flag) return 1;

if(! show_items(SHOW_BOTTOM) ) return 0;

return 1;
}/* end function add_item_to_portfolio */


int modify_portfolio_item_price(\
char *fund, int type, int month, int year, float strike_price,\
float transaction_price)
{
int a, i;
struct item *pa;
char temp[READSIZE];
char *fundname_lower_case;
char *month_name;

if(debug_flag)
	{
	fprintf(stdout,\
	"modify_portfolio_item_price(): arg\n\
	fund=%s type=%d month=%d year=%d strike_price=%.2f\n\
	transaction_price=%.2f\n",\
	fund, type, month, year, strike_price, transaction_price);
	}

/* argument check */
if(! fund) return 0;
if( (type != TYPE_CALL) && (type != TYPE_PUT) && (type != TYPE_STOCK) &&\
(type != TYPE_BOND) )
	{
	fl_show_alert("Unkown type", "Legal are: call put stock bond", "", 1);
	return 0;
	}

if( (type == TYPE_CALL) || (type == TYPE_PUT) || (type == TYPE_BOND) )
	{
	if(year < 1900) return 0;

	if(month == 0) return 0;
	month_name = digit_to_month(month);
	if(! month_name) return 0;

	if(strike_price < 0) return 0;
	}

/* fund to lower case */
fundname_lower_case = strsave(fund);/* should be same length  */
if(! fundname_lower_case) return 0;
i = 0;
while(1)
	{
	fundname_lower_case[i] = tolower(fund[i]);
	if(fund[i] == 0) break;
	i++;
	}

/* create an unique identifier */
if(type == TYPE_STOCK)
	{
	/* aab */
	sprintf(temp,\
	"%s",\
	fundname_lower_case);
	free(fundname_lower_case);
	}
else if(type == TYPE_CALL)
	{
	/* aab call oct 1999 22.00 */
	sprintf(temp,\
	"%s call %s %d %.2f",\
	fundname_lower_case, month_name, year, strike_price);
	free(fundname_lower_case);
	free(month_name);
	}
else if(type == TYPE_PUT)
	{
	/* aab put oct 1999 22.00 */
	sprintf(temp,\
	"%s put %s %d %.2f",\
	fundname_lower_case, month_name, year, strike_price);
	free(fundname_lower_case);
	free(month_name);
	}
else if(type == TYPE_BOND)
	{
	/* somebond */
	sprintf(temp,\
	"%s bond %s %d %.2f",\
	fundname_lower_case, month_name, year, strike_price);
	free(fundname_lower_case);
	free(month_name);
	}
else
	{
	free(fundname_lower_case);
	return 0;
	}

/* lookup entry */
pa = lookup_item(temp);
if(! pa)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"modify_portfolio_item_price(): could not look up %s returning 0\n",\
		temp);
		}
	return 0;
	}

pa -> trade_price = transaction_price;

return 1;
}/* end function modify_portfolio_item_price */


int sort_items(int mode)
/*
sorts the double linked list with criterium set by  mode.
doing some sort of bubble sort.
*/
{
struct item *pa;
struct item *pb;
int swap_flag;

if(debug_flag)
	{
	fprintf(stdout,\
	"sort_items(): arg mode=%d\n",\
	mode);
	}

while(1)/* go through list again and again */
	{
	swap_flag = 0;
	for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa -> prventr;
		if(pb)
			{
			if(mode == SORT_ALPHA)
				{
				if(strcmp(pa -> name, pb -> name) < 0)
					{
					swap_flag = swap_item_position(pa , pb);
					/* indicate position was swapped */
					}
				}/* end if sort by alpha */
 			else if(mode == SORT_PRICE)
				{
				if(pa -> trade_price < pb -> trade_price)
					{
					swap_flag = swap_item_position(pa , pb);
					/* indicate position was swapped */
					}/* end if condition */
				}/* end if sort by price */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_items */


int buy_item(char *name, float quantity, float price)
{
struct item *pa;
float transaction_price;
float transaction_cost;
float money_spend;

if(debug_flag)
	{
	fprintf(stdout,\
	"buy_item(): arg name=%s quantity=%.2f price=%.2f\n",\
	name, quantity, price);
	}

if(! name) return 0;
if(quantity <= 0) return 0;
if(price <= 0) return 0;

pa = lookup_item(name);
if(! pa) return 0;

/* calculate transaction price */
transaction_price = quantity * price;

/*
calculate the transaction cost from the share_rates form or the option_rates
form.
*/
if(pa -> type == TYPE_STOCK)
	{
	if(! get_share_transaction_cost(\
		BUY, quantity, price, transaction_price, &transaction_cost) )
			{
			return(0);
			}

	}/* end if type stock  */
else if( (pa -> type == TYPE_CALL) || (pa -> type == TYPE_PUT) ||\
 (pa -> type == TYPE_BOND) )
	{
	if(! get_option_transaction_cost(\
		BUY, quantity, price, transaction_price, &transaction_cost) )
			{
			return(0);
			}

	}/* end if type stock  */
else/* not a known type */
	{
	return 0;
	}

/* calculate money spend */
money_spend = transaction_price + transaction_cost;

if(debug_flag)
	{
	fprintf(stdout,\
	"buy_item(): type=%d\n\
	transaction_price=%.2f transaction_cost=%.2f money_spend=%.2f\n",\
	pa -> type,\
	transaction_price, transaction_cost, money_spend);
	}

/* negative quantity is long */

/* update cash */
portfolio_form_cash -= money_spend;

if(portfolio_form_cash < 0)
	{
	fl_show_alert("Your account is negative", "", "credit?", 1);
	}

if( (portfolio_form_cash + portfolio_form_value) < 0)
	{
	fl_show_alert("Total assets are negative", "margin?", "credit?", 1);
	}

return 1;
}/* end function buy_item */


int sell_item(char *name, float quantity, float price)
{
struct item *pa;
char temp[80];
float transaction_price;
float transaction_cost;
float money_received;

if(debug_flag)
	{
	fprintf(stdout,\
	"sell_item(): arg name=%s quantity=%.2f price=%.2f\n",\
	name, quantity, price);
	}

if(! name) return 0;
if(quantity <= 0) return 0;
if(price <= 0) return 0;

pa = lookup_item(name);
if(! pa) return 0;

/* warn user for going short */
if(quantity > pa -> quantity)
	{
	sprintf(temp, "%.2f", pa -> quantity);
	fl_show_alert("You will go short you own", temp, pa -> name, 1);
	}

/* calculate transaction price */
transaction_price = quantity * price;

/*
calculate the transaction cost from the share_rates form or the option_rates
form.
*/
if(pa -> type == TYPE_STOCK)
	{
	if(! get_share_transaction_cost(\
		SELL, quantity, price, transaction_price, &transaction_cost) )
			{
			return(0);
			}

	}/* end if type stock  */
else if( (pa -> type == TYPE_CALL) || (pa -> type == TYPE_PUT) ||\
(pa -> type == TYPE_BOND) )
	{
	if(! get_option_transaction_cost(\
		SELL, quantity, price, transaction_price, &transaction_cost) )
			{
			return(0);
			}

	}/* end if type stock  */
else/* not a known type */
	{
	return 0;
	}

/* calculate money received */
money_received = transaction_price - transaction_cost;

if(debug_flag)
	{
	fprintf(stdout,\
	"sell_item(): type=%d\n\
	transaction_price=%.2f transaction_cost=%.2f money_received=%.2f\n",\
	pa -> type,\
	transaction_price, transaction_cost, money_received);
	}

/* if quantity goes negative, we are short */

/* update cash */
portfolio_form_cash += money_received;

return 1;
}/* end function sell_item */


int calculate_portfolio_value(\
float *value, float *account_value,\
float *shares_value, float *options_value)
{
struct item *pa;
float item_value;
char *ptra;
char *ptrb;
char *ptrc;
int match_failed_flag;
int last_string_flag;
char *filter_keywords;

if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_portfolio_value(): arg *value=%.2f\n",\
	*value);
	}

filter_keywords =\
	(char*)fl_get_input(fdui -> portfolio_form_market_filter_input_field);

*value = 0;
*shares_value = 0;
*options_value = 0;
for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* count only what we have */
	if(pa -> quantity != 0)
		{
		/* test if any filter keywords */
		if(filter_keywords[0])/* some filter */
			{
			/* test if filter keywords present in item name */
			/* for all entries in filter */
			ptra = filter_keywords;
			ptrb = strsave(filter_keywords);/* create space */
			match_failed_flag = 0;
			last_string_flag = 0;
			while(1)
				{
				ptrc = ptrb;/* point to start availeble space */
				while(1)/* string in filter to ptrb */
					{
					*ptrc = *ptra;
					if(! *ptra) break;/* end filter */
					if(*ptra == ' ')/* end of string in filter */
						{
						*ptrc = 0;
						ptra++;/* scip the space */
						break;
						}
					ptra++;
					ptrc++;
					}/* end while all characters in string */
				/* test if string present in name */
				/* test for case sensitive */
				if(portfolio_form_market_filter_case_sensitive_flag)
					{
					if(! strstr(pa -> name, ptrb) )
						{
						match_failed_flag = 1;
						break;
						}
					}
				else /* not case sensitive */
					{
					if(! strcasestr(pa -> name, ptrb) )
						{
						match_failed_flag = 1;
						break;
						}
					}
				if(! *ptra) break;/* end filter */
				}/* end while all strings in filter */	
			free(ptrb);/* release space */
			if(match_failed_flag) continue;
			}/* end if filter */

		/*
		item value may be negative, because quantity may be negative (short)
		*/
		item_value = pa -> quantity * pa -> trade_price;

		/* value may be negative */
		*value += item_value;
		
		if( (pa -> type == TYPE_CALL) || (pa -> type == TYPE_PUT) ||\
		(pa -> type == TYPE_BOND) )
			{
			*options_value += item_value;
			}
		if(pa -> type == TYPE_STOCK)
			{
			*shares_value += item_value;
			}		
		}
	}/* end for all entries */

*account_value = portfolio_form_cash;

return 1;
}/* end function calculate_portfolio_value */


char *strcasestr(char *str1, char *str2)
{
char *ptr1;
char *ptr2;
char *position;

position = str1;
while(1)
	{
	if(! *position) return(0);

	/* compare from position start */
	ptr1 = position;
	ptr2 = str2;
	while(1)
		{
		/* test for end string 2 first, that would be found */
		if(! *ptr2) return(position);

		/* test for end string 1 first, that would be failed */
		if(! *ptr1) break;

		if(tolower(*ptr1) == tolower(*ptr2) )
			{
			ptr1++;
			ptr2++;
			}
		else break;
		}/* end inner while */
	position++;
	}/* end outher while */
}/* end function strcasestr */


int select_portfolio_form_browser(int browser)
{
if(debug_flag)
	{
	fprintf(stdout,\
	"select_portfolio_form_browser(): arg browser=%d\n",\
	browser);
	}

if(! directory_changed_flag)
	{
/*	if(browser == portfolio_form_selected_browser) return 1;*/
	}

/* only do something if not already selected (initial start = 0) */
portfolio_form_selected_browser = browser;

if(portfolio_form_selected_browser == MARKET_BROWSER)
	{
	fl_hide_object(fdui -> portfolio_form_transaction_group);
	fl_show_object(fdui -> portfolio_form_market_group);

	fl_set_object_label(\
		fdui -> portfolio_form_browser_select_button, "MARKET");

	if(! show_items(SHOW_BOTTOM) ) return; 

	/* set button to right state (color) */
	fl_set_button(fdui -> portfolio_form_browser_select_button, 0);
	}
else if(portfolio_form_selected_browser == TRANSACTION_BROWSER)
	{	
	fl_hide_object(fdui -> portfolio_form_market_group);
	fl_show_object(fdui -> portfolio_form_transaction_group);

	fl_set_object_label(\
		fdui -> portfolio_form_browser_select_button, "TRANSACTIONS");

	if(! show_transactions(SHOW_BOTTOM) ) return; 

	fl_set_button(fdui -> portfolio_form_browser_select_button, 1);
	}
/* other elses perhaps */

return 1;
}/* end function select_portfolio_form_browser */


int update_and_get_owned_quantity(char *item, float *owned_quantity, float quantity)
{
/*
This updates the quantity, if this is the last entry for that item,
updates the price, returns 0 if item not present. in item list.
*/
struct item *pa;

if(debug_flag)
	{
	fprintf(stdout,\
		"update_and_get_owned_quantity(): arg item=%s quantity=%.2f\n",\
		item, quantity);
	}

/* argument check */
if(! item) return 0;

pa = lookup_item(item);
if(! pa) return 0;

pa -> quantity += quantity;
*owned_quantity = pa -> quantity;

return 1;
}/* end function update_and_get_owned_quantity */


int mark_noteringen_already_present()
{
struct item *pa;
int present_flag;

if(debug_flag)
	{
	fprintf(stdout, "mark_noteringen_already_present(): arg none\n");
	}

for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(! mark_if_notering_present_in_list(pa -> name, &present_flag) )
		{
		continue;
		}
	}

return 1;
}/* end function mark_noteringen_already_present */


int set_item_price(char *item, float price)
{
struct item *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_item_price(): arg item=%s price=%.2f\n",\
	item, price);
	}

/* argument check */
if(! item) return(0);

pa = lookup_item(item);
if(! pa) return(0);

pa -> trade_price = price;

return(1);
}/* end function set_item_price */


int delete_expired_options()
{
struct item *pa;
float days_to_expiration;
char temp[80];
float percentage;
int size;
int done;
float old_percentage;
size = 0;
for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	size++;
	}

if(debug_flag)
	{
	fprintf(stdout, "delete_expired_options(): arg none\n");
	}

old_percentage = 0;
done = 0;
for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	percentage = ((float)done / (float)size) * 100.0;
	if( (percentage - old_percentage) > 1.0)
		{
		sprintf(temp,\
		"deleting expired items %2.0f%c done", percentage, '%');
		fl_set_object_label(fdui -> portfolio_form_status_display, temp);
		XSync(fl_get_display(), 0);
		old_percentage = percentage;
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"delete_expired_options(): processing %s\n", pa -> name);
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"delete_expired_options(): pa -> type=%d\n",\
		pa -> type);
		}

	if( (pa -> type != TYPE_CALL) && (pa -> type != TYPE_PUT) &&\
	(pa -> type != TYPE_BOND) )
		{
		done++;
		continue;
		}

	if(! get_days_to_expiration2(\
	pa -> expiration_month, pa -> expiration_year, &days_to_expiration) )
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"delete_expired_options():\n\
			cannot get days to expiration for %s scipping\n",\
			pa -> name);
			}
		fl_show_alert(\
		"Could not get days to expiration for", pa -> name, "scipping", 1);

		done++;
		continue;
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"delete_expired_options(): days to expiration for %s is %.2f\n",\
		pa -> name, days_to_expiration);
		}

	if(days_to_expiration < 0)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"delete_expired_options(): deleting %s\n", pa -> name);
			}

		if(! delete_item(pa -> name) )
			{
			if(debug_flag)
				{
				fprintf(stdout,\
				"delete_expired_options():\\n\
				could not delete expired option %s scipping\n", pa -> name);
				}
			fl_show_alert(\
			"Could not delete expired option", pa -> name, "scipping", 1);

			done++;
			continue;
			}
		}/* end if days_to_expiration < 0 */

	done++;
	}/* end for all entries in item tab */

return 1;
}/* end function delete_expired_options */


int show_where_the_money_is_chart()
{
float value;/* shares plus options */
float account_value;
float shares_value;
float options_value;
char temp[512];

/*
fl_freeze_form

fl_clear_chart
fl_add_chart_value(FL_OBJECT *obj, double val, const char *text, int col)
fl_insert_chart_value(FL_OBJECT *obj, int index,
                           double val, const char *text, int col)</PRE>
fl_replace_chart_value(FL_OBJECT *obj, int index,
                               double val, const char *text, int col)</PRE>
void fl_set_chart_bounds(FL_OBJECT *obj, double min, double max)
fl_set_chart_autosize(FL_OBJECT *obj, int autosize)
fl_set_chart_maxnumb(FL_OBJECT *obj, int maxnumb)
fl_unfreeze_form
*/

fl_set_object_label(\
	fdui -> portfolio_form_special_pie_chart, "Where your money is");
	
if(! calculate_portfolio_value(\
	&value, &account_value, &shares_value, &options_value) )
		{
		fl_show_alert(\
		"Cannot calculate portfolio value", "command aborted", "", 1);
		return;
		}

fl_clear_chart(fdui -> portfolio_form_special_pie_chart);

if(account_value > 0)
	{
	sprintf(temp, "account %.2f", account_value);
	fl_add_chart_value(fdui -> portfolio_form_special_pie_chart,\
		account_value, temp,  FL_GREEN);
	}
else
	{
	sprintf(temp, "account %.2f", account_value);
	fl_add_chart_value(fdui -> portfolio_form_special_pie_chart,\
		fabs(account_value), temp,  FL_RED);
	}
if(options_value > 0)
	{
	sprintf(temp, "options %.2f", options_value);
	fl_add_chart_value(fdui -> portfolio_form_special_pie_chart,\
		options_value, temp,  FL_CYAN);
	}
else
	{
	sprintf(temp, "options %.2f", options_value);
	fl_add_chart_value(fdui -> portfolio_form_special_pie_chart,\
		fabs(options_value), temp,  FL_YELLOW);
	}	
if(shares_value > 0)
	{
	sprintf(temp, "shares %.2f", shares_value);
	fl_add_chart_value(fdui -> portfolio_form_special_pie_chart,\
		shares_value, temp, FL_BLUE);
	}
else
	{
	sprintf(temp, "shares %.2f", shares_value);
	fl_add_chart_value(fdui -> portfolio_form_special_pie_chart,\
		fabs(shares_value), temp, FL_MAGENTA);
	}

fl_show_object(fdui -> portfolio_form_special_pie_chart);
/*
fl_show_object(fdui -> portfolio_form_pie_chart);
fl_show_object(fdui -> portfolio_form_bar_chart);
*/

show_where_the_money_is_chart_shown_flag = 1;

return 1;
}/* end function show_where_the_money_is_chart */


int update_krs_files_from_portfolio()
{
struct item *pa;
char temp[1024];
char datestr[40];
char weekday[20];
char monthstr[20];
int year, month, monthday, hour, minute, second;
char filename[1024];

if(debug_flag)
	{
	fprintf(stdout,\
	"update_krs_files_from_portfolio(): arg none\n");
	}

sprintf(filename, "%s/fund_list.dat", file_path);
sprintf(temp,\
	"reading list of funds to process from %s", filename);
fl_set_object_label(fdui -> portfolio_form_status_display, temp);
XSync(fl_get_display(), 0);

if(! fill_stock_list(filename) )
	{
	sprintf(temp, "cannot read file stock list");
	fl_set_object_label(fdui -> portfolio_form_status_display, temp);
	XSync(fl_get_display(), 0);
	return 0;
	}

/*pa points to next entry*/
for(pa = itemtab[0]; pa != 0; pa = pa -> nxtentr)
	{
/*
	int update_fund_file(\
char *fundname, float close_price, long volume,\
int monthday, int month, int year)
*/	
	if(pa -> type == TYPE_STOCK)
		{

		/* test if fund in list to be processed */
		if(! test_stock(pa -> name) ) continue;
		
		/* get computer time */
		ogettime(\
			weekday, monthstr, &monthday, &hour, &minute, &second, &year);
		month = month_to_digit(monthstr);
		if( (month < 1) || (month > 12) )
			{
			fl_show_alert("Ilegal month read from computer", monthstr, "", 1);
			return 0;
			}

		/* update fund files (.krs and .csv) */
		if(! update_fund_file(\
		pa -> name, pa -> trade_price, 0, monthday, month, year)
		)
			{
			if(debug_flag)
				{
				fprintf(stdout,\
				"Could not update stock %s scipping\n",\
				pa -> name);
				}
			continue;
			}/* end if update_fund_file failed */
		}/* end if type is stock */
	}/* end for all items in list */

return 1;
}/* end function update_krs_files_from_portfolio */

