/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xkra.h"

FD_xkra *fdui;

extern FD_xkra *fd_main_form;

int show_transaction_add_form()
{

fl_show_form(fdui -> transaction_add_form, FL_PLACE_CENTER, FL_NOBORDER, "");

transaction_add_form_shown_flag = 1;

}/* end function show_transaction_add_form */


int add_transaction_to_transaction_list(\
	int year, int month, int day, int hour, int buy_sell,\
	char *fund, int type, int expiration_month, int expiration_year,\
	float strikeprice,\
	float quantity, float price, float account, char *report\
)
{
int i;
char fundname[80];
char description[512];
struct tm *pttm;/* time transaction took place */
time_t transaction_time;
char *expiration_monthstr;

if(debug_flag)
	{
	fprintf(stdout, "add_transaction_to_transaction_list(): arg\n\
	year=%d month=%d day=%d hour=%d buy_sell=%d\n\
	fund=%s type=%d expiration_month=%d expiration_year=%d\n\
	strikeprice=%.2f quantity=%.2f price=%.2f account=%.2f\n",\
	year, month, day, hour, buy_sell,\
	fund, type, expiration_month, expiration_year,\
	strikeprice, quantity, price, account\
	);
	}

/* argument check */
if(strlen(fund) == 0)
	{
	strcpy(report, "no fund name specified");
	return 0;
	}
if(year < 1900)
	{
	strcpy(report, "invalid year");
	return 0;
	}
if( (type != TYPE_CALL) && (type != TYPE_PUT) && (type != TYPE_STOCK) &&\
(type != TYPE_BOND) )
	{
	strcpy(report, "invalid type");
	return 0;
	}
	
if( (type == TYPE_PUT) || (type == TYPE_CALL) || (type == TYPE_BOND) )
	{
	/* create a standard month name */
	expiration_monthstr = digit_to_month(expiration_month);
	if(! expiration_monthstr)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
"add_transaction_to_transaction_list(): digit_to_month() failed returning\n");
			}
		strcpy(report, "cannot convert to month");
		return 0;
		}
	}

/* calculate transaction_time */
/* allocate space for a structure */
pttm = (struct tm *) malloc(sizeof(struct tm) );
if(! pttm)
	{
	strcpy(report, "cannot malloc structure pttm");
	return 0;
	}

pttm -> tm_sec = 0;
pttm -> tm_min = 0;
pttm -> tm_hour = hour;
pttm -> tm_mday = day;
pttm -> tm_mon = month - 1;
pttm -> tm_year = year - 1900;
pttm -> tm_isdst = -1;/* daylight savings time n.a. */
/* the following are not implemented (see /usr/include/time.h) */
/* pttm -> tm_gmtoff = 0;*/
/* strcp(pttm -> tm_zone, "");/* time zone 3 letters */
transaction_time = mktime(( struct tm *) pttm);

free(pttm);

if(transaction_time == -1)
	{
	strcpy(report, "cannot create time");
	}
if(debug_flag)
	{
	fprintf(stdout,\
	"add_transaction_to_transaction_list(): transaction_time=%lu\n",\
	(long)transaction_time);
	}

/* fundname to lower case */
strcpy(fundname, fund);
i = 0;
while(1)
	{
	fundname[i] = tolower(fundname[i]);
	if(fundname[i] == 0) break;
	i++;
	}

/* create an unique identifier in description */
if(type == TYPE_STOCK)
	{
	/* aab */
	sprintf(description,\
	"%s",\
	fundname);
	}
else if(type == TYPE_CALL)
	{
	/* aab call oct 1999 22.00 */
	sprintf(description,\
	"%s call %s %d %.2f",\
	fundname, expiration_monthstr, expiration_year, strikeprice);
	free(expiration_monthstr);
	}
else if(type == TYPE_PUT)
	{
	/* aab put oct 1999 22.00 */
	sprintf(description,\
	"%s put %s %d %.2f",\
	fundname, expiration_monthstr, expiration_year, strikeprice);
	free(expiration_monthstr);
	}
else if(type == TYPE_BOND)
	{
	/* somebond */
	sprintf(description,\
	"%s bond %s %d %.2f",\
	fundname, expiration_monthstr, expiration_year, strikeprice);
	free(expiration_monthstr);
	}

if(! add_transaction_to_portfolio(\
	transaction_time, description, quantity, price, buy_sell, account) )
	{
	strcpy(report, "cannot add transaction");
	return 0;
	}

return 1;
}/* end function add_transaction_to_transaction_list */
