/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xkra.h"

FD_xkra *fdui;

extern FD_xkra *fd_main_form;

char *selected_transaction;
int portfolio_form_show_have_only_flag;


struct transaction
	{
	char *name; /* time as a string */
	int status; /* not used was NEW_ITEM HAVE_ITEM */
	int browser_line;	
	float quantity;
	float price;
	char *description;
	int buy_sell;
	float owned_quantity;
	float account;
				
	struct transaction *nxtentr;
	struct transaction *prventr;
	};
struct transaction *transactiontab[2]; /* first element points to first entry,
			second element to last entry */


struct transaction *lookup_transaction(char *name)
{
struct transaction *pa;

/*pa points to next entry*/
for(pa = transactiontab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return(0);/*not found*/
}/* end function lookup_transaction */


struct transaction *install_transaction_at_end_of_list(char *name)
{
struct transaction *plast, *pnew;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_transaction_at_end_off_list(): arg name=%s\n", name);
	}

/*pnew = lookup_transaction(name);*/
/*if(pnew)*/
/*	{*/
	/* free previous definition */
/*	free(pnew -> subject); */
/*	return(pnew); */ /* already there */
/*	}*/

/* create new structure */
pnew = (struct transaction *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = transactiontab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !transactiontab[0] ) transactiontab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
transactiontab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_transaction_at_en_off_list */


int delete_transaction(char *name)/* delete entry from double linked list */
{
struct transaction *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stdout, "delete_transaction(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return(0);

pa = transactiontab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return(0);

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify transactiontab[0] */
	if(pprev == 0) transactiontab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify transactiontab[1] */
	if(pnext == 0) transactiontab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pa -> description);
	
	free(pdel); /* free structure */

	/* return OK deleted */
	return(1);
	}/* end for all structures */
}/* end function delete_transaction */


int delete_all_transactions()/* delete all entries from table */
{
struct transaction *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_transactions() arg none\n");
	}

while(1)
	{	
	pa = transactiontab[0];
	if(! pa) break;
	transactiontab[0] = pa -> nxtentr;
	free(pa -> name);
	free(pa -> description);
	free(pa);/* free structure */
	}/* end while all structures */

transactiontab[1] = 0;
return(1);
}/* end function delete_all_transactions */


int swap_transaction_position(struct transaction *ptop, struct transaction *pbottom)
{
struct transaction *punder;
struct transaction *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): swapping top=%lu bottom=%lu\n", ptop, pbottom);
	}

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): punder=%lu\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): pabove=%lu\n", pabove);
	}

/* the next pointer in punder (or transactiontab[0]) must now point to ptop */
if(! punder)
	{
	transactiontab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in transactiontabtab */
if(! pabove)
	{
	transactiontab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_transaction_position */


int load_transactions()
{
int a;
FILE *transactions_dat_file;
char temp[READSIZE];
char pathfilename[TEMP_SIZE];
struct transaction *pa;
int first_transaction_flag;
int status;
int browser_line; /* used to lookup name (= id) if clicked in display,
						not saved on disk, assigned each time list
						is displayed */
int status_line;
int some_new_flag;
float quantity;
float owned_quantity;
float price;
int buy_sell;
char time_str[40];
int type;
float account;
 
if(debug_flag)
	{
	fprintf(stdout, "load_transactions() arg none using selected_transaction=%s\n",\
	selected_transaction);
	}
	
delete_all_transactions();

sprintf(pathfilename, "%s/transactions.dat", file_path);
transactions_dat_file = fopen(pathfilename, "r");
if(! transactions_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}
	fl_show_alert("Cannot open file", pathfilename, "for read", 1);

	return(0);
	} 
status_line = 1;
first_transaction_flag = 1;
while(1)
	{
	/* read header line or first line of entry */
	a = readline(transactions_dat_file, temp);

	if(status_line)
		{
		sscanf(temp, "%d", &some_new_flag);
		status_line = 0;
		if(a == EOF) return 1;/* only this line, no entries */
		continue;
		}/* end if status line */

	if(a == EOF) break;/* file format error, second line entry missing  */

	/* this first, temp used in next readline */
	
	sscanf(temp, "%s %d %d %d %f %f %f %f",\
	time_str, &status, &browser_line, &buy_sell, &quantity, &price,\
	&owned_quantity, &account);
	
	/* create a structure entry */
	pa = install_transaction_at_end_of_list(time_str);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_transactions: cannot install transaction %s\n", temp);
			}
		return(0);
		}

	/* read second line of entry */
	a = readline(transactions_dat_file, temp);
	if(a == EOF) break;/* file format error */
	
	/* data to structure */
	pa -> name = strsave(time_str);
	if(! pa -> name) return 0;
	pa -> status = status;
	pa -> browser_line = browser_line;
	pa -> buy_sell = buy_sell;
	pa -> quantity = quantity;
	pa -> price = price;
	pa -> owned_quantity = owned_quantity;
	pa -> account = account;
	pa -> description = strsave(temp);
	if(! pa -> description) return 0;
			
	if(first_transaction_flag)
		{
		selected_transaction = strsave(pa -> name);
		first_transaction_flag = 0;
		}

	/* if no more lines, this was the last entry, return OK */
	}/* end while all lines in transactions.dat */

/* return error */
return 0;
}/* end function load transactions */


int save_transactions()
{
FILE *transactions_dat_file;
char pathfilename[TEMP_SIZE];
char pathfilename2[TEMP_SIZE];
struct transaction *pa;
int some_new_flag;
FILE *fptr;

if(debug_flag)
	{
	fprintf(stdout, "save_transactions() arg none\n");
	}

sprintf(pathfilename, "%s/transactions.tmp", file_path);
transactions_dat_file = fopen(pathfilename, "w");
if(! transactions_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"could not open file %s for write\n", pathfilename);
		}
	fl_show_alert("Cannot open file", pathfilename, "for write", 1);	

	return(0);
	} 

some_new_flag = 0;
fprintf(transactions_dat_file, "%d\n", some_new_flag);

for(pa = transactiontab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(transactions_dat_file,\
	"%s %d %d %d %.2f %.2f %.2f %.2f\n%s\n",\
	pa -> name,\
	pa -> status,\
	pa -> browser_line,\
	pa -> buy_sell,\
	pa -> quantity,\
	pa -> price,\
	pa -> owned_quantity,\
	pa -> account,\
	pa -> description);

	}/* end while all elements in list */

fclose(transactions_dat_file);

if(debug_flag)
	{
	fprintf(stdout, "save_transactions(): backing up transactions.dat\n");
	}

/* set some path file names */
sprintf(pathfilename, "%s/transactions.dat", file_path);
sprintf(pathfilename2, "%s/transactions.dat~", file_path);

/* unlink the old .dat~ */
unlink(pathfilename2);

/* test if there is a /.dat file (not first time) */
fptr = fopen(pathfilename, "r");
if(fptr)
	{
	fclose(fptr);
	/* rename .dat to .dat~ */
	if( rename(pathfilename, pathfilename2) == -1)
		{
		if(debug_flag)
			{
			fprintf(stdout, "save_transactions(); rename %s into %s failed\n",\
			pathfilename, pathfilename2);
			}
		return(0);
		}
	}/* end if .dat file existed */
else
	{
	fl_show_alert("Warning", pathfilename, "does not exist, creating it", 1);
	}		

/* rename .tmp to .dat */
sprintf(pathfilename2, "%s/transactions.tmp", file_path);
if( rename(pathfilename2, pathfilename) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_transactions(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return(0);
	}

return(1);/* ok */
}/* end function save_transactions */


int show_transactions(int position)
{
char temp[TEMP_SIZE];
struct transaction *pa;
char formatstr[20];
int browser_line;
int browser_topline;
int maxline;
int a;
char *ptra;
char *ptrb;
char *ptrc;
int match_failed_flag;
int last_string_flag;
char *filter_keywords;
char timestr[512];
time_t transaction_time;
int double_quote_flag;

if(debug_flag)
	{
	fprintf(stdout, "show_transactions(): arg position=%d\n", position);
	}

fl_set_object_label(fdui -> portfolio_form_status_display,\
	"sorting alpha, this may take a while");
XSync(fl_get_display(), 0);

if(! sort_transactions(SORT_DATE) ) return 0;

if(position == SHOW_UNREAD_AT_TOP) return 0;/* not supported */

filter_keywords =\
	(char*)fl_get_input(\
	fdui -> portfolio_form_transaction_filter_input_field);

/* remember vertical slider position */
browser_topline =\
	fl_get_browser_topline(fdui -> portfolio_form_transaction_browser);

fl_set_object_label(fdui -> portfolio_form_status_display,\
	"building display, this may take a while");
XSync(fl_get_display(), 0);

fl_freeze_form(fd_main_form -> portfolio_form);

fl_clear_browser(fdui -> portfolio_form_transaction_browser);

browser_line = 1;
for(pa = transactiontab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	/* test if any filter keywords */
	if(filter_keywords[0])/* some filter */
		{
		/* test if filter keywords present in transaction name */
		/* for all entries in filter */
		ptra = filter_keywords;
		ptrb = strsave(filter_keywords);/* create space */
		match_failed_flag = 0;
		last_string_flag = 0;
		while(1)
			{
			ptrc = ptrb;/* point to start availeble space */
			while(1)/* string in filter to ptrb */
				{
				*ptrc = *ptra;
				if(! *ptra) break;/* end filter */

				if(*ptra == '"')/* double quote */
					{
					double_quote_flag = 1 - double_quote_flag;
					ptra++;
					continue;
					}
				if(! double_quote_flag)
					{
					if(*ptra == ' ')/* end of string in filter */
						{
						*ptrc = 0;
						ptra++;/* scip the space */
						break;
						}
					}
				ptra++;
				ptrc++;
				}/* end while all characters in string */
			/* test if string present in name */
			/* test for case sensitive */
			if(portfolio_form_transaction_filter_case_sensitive_flag)
				{
				if(! strstr(pa -> description, ptrb) )
					{
					match_failed_flag = 1;
					break;
					}
				}
			else /* not case sensitive */
				{
				if(! strcasestr(pa -> description, ptrb) )
					{
					match_failed_flag = 1;
					break;
					}
				}
			if(! *ptra) break;/* end filter */
			}/* end while all strings in filter */	
		free(ptrb);/* release space */
		if(match_failed_flag)
			{
			/* to prevent false finds in double click mark not existing */
			pa -> browser_line = 0;
			continue;
			}
		}/* end if filter */
	
	strcpy(formatstr, "@f");

	if(pa -> owned_quantity > 0)\
		strcat(formatstr, "@C18"); /* long is dark green */
	else if(pa -> owned_quantity < 0)\
		strcat(formatstr, "@C1"); /* short is red */
	else if(pa -> owned_quantity == 0) strcat(formatstr, "@C0"); /* black */

/*	if(pa -> status & HAVE_ITEM) strcat(formatstr, "@C4");*/ /* blue */
/*	if(pa -> status & HAVE_ITEM) strcat(formatstr, "@C5");*/ /* magenta */
	
	/* create time as string */
	sscanf(pa -> name, "%lu", &transaction_time);
	sprintf(timestr, "%s", ctime(&transaction_time) );
	/* Note ctime prints \n, do not want that */
	a = strlen(timestr);	
	/* cut off \n */
	timestr[a - 1] = 0; 
	
	if(pa -> buy_sell == BUY)
		{
		sprintf(temp,\
		"%s %s bought %.2f %s at %.2f own %.2f",\
		formatstr, timestr, pa -> quantity, pa -> description, pa -> price,\
		pa-> owned_quantity);
		}
	else if(pa -> buy_sell == SELL)
		{
		sprintf(temp,\
		"%s %s sold %.2f %s at %.2f own %.2f",\
		formatstr, timestr, pa -> quantity, pa -> description, pa -> price,\
		pa -> owned_quantity);
		}
	else
		{
		/* to prevent false finds in double click mark not existing */
		pa -> browser_line = 0;
		continue;
		}
	
	fl_add_browser_line(fdui -> portfolio_form_transaction_browser, temp);
	pa -> browser_line = browser_line;/* first one is 1 */
	browser_line++;
	}/* end for all elements in list */
	
/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last transaction visible */
	maxline =\
		fl_get_browser_maxline(fdui -> portfolio_form_transaction_browser);
	fl_set_browser_topline(\
		fdui -> portfolio_form_transaction_browser, maxline);
	}
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> portfolio_form_transaction_browser, 1);
	}
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(\
		fdui -> portfolio_form_transaction_browser, browser_topline);
	}	

fl_unfreeze_form(fd_main_form -> portfolio_form);

fl_set_object_label(fdui -> portfolio_form_status_display, "");
XSync(fl_get_display(), 0);

return(1);
}/* end function show_transactions */


char *line_to_transaction(int line)
{
struct transaction *pa;
extern long atol();

if(debug_flag)
	{
	fprintf(stdout, "line_to_transaction(): arg line=%d\n", line);
	}

/* argument check */
if(line < 0) return(0);

for(pa = transactiontab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line == line)
		{
		return(pa -> name);
		}
	}

/* no save_transactions, nothing was changed */
return(0);
}/* end function line_to_transaction_id */


int transaction_to_line(char *url, int *line)
{
struct transaction *pa;

if(debug_flag)
	{
	fprintf(stdout, "transaction_to_line(): arg url=%s\n", url);
	}

/* argument check */
if(! url) return(0);

pa = lookup_transaction(url);
if(! pa) return(0);

*line = pa -> browser_line;
return(1);
}/* end function transaction_to_line */


int sort_transactions(int mode)
/*
sorts the double linked list with criterium set by  mode.
doing some sort of bubble sort.
*/
{
struct transaction *pa;
struct transaction *pb;
int swap_flag;

while(1)/* go through list again and again */
	{
	swap_flag = 0;
	for(pa = transactiontab[0]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa -> prventr;
		if(pb)
			{
			if(mode == SORT_DATE)
				{
				if(atol(pa -> name) < atol(pb -> name) )
					{
					swap_flag = swap_transaction_position(pa , pb);
					/* indicate position was swapped */
					}/* end if condition */
				}/* end if sort by price */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_transactions */


int add_transaction_to_portfolio(\
time_t transactiontime, char *item, float quantity, float price,\
int buy_sell, float account)
{
int a, i;
struct transaction *pa;
char temp[READSIZE];
char timestr[512];
char buy_sell_str[20];
float owned_quantity;
float bought;
struct transaction *pb;
int latest_transaction_flag;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_transaction_to_portfolio(): arg\n\
	transactiontime=%lu item=%s quantity=%.2f price=%.2f\n\
	owned_quantity=%.2f account=%.2f\n",\
	transactiontime, item, quantity, price, owned_quantity, account);
	}

/* argument check */
if(! item) return 0;
if(price < 0) return 0;/* ?_____ */

if(buy_sell == BUY) bought = quantity;
if(buy_sell == SELL) bought = -1.0 * quantity;
  
/*
This updates the quantity in the item list (market browser),
returns 0 if item not present. in item list.
*/
if(! update_and_get_owned_quantity(item, &owned_quantity, bought) )
	{
	fl_show_alert(\
	"Item is not present in market list", item, "enter there first", 1);
	return 0;
	}

if(debug_flag)
	{
	fprintf(stdout,\
	"add_transaction_to_portfolio(): owned_quantity now %.2f\n",\
	owned_quantity);
	}

i = 0;
while(1)
	{
	/* create an unique identifier, using time */
	sprintf(timestr, "%lu", (long)transactiontime);
	pa = lookup_transaction(timestr);
	if(! pa) break;

	transactiontime++;
	i++;
	/* give up */
	if(i > 3600) return 0;
	}

if(i)
	{
	sprintf(temp, "shifted %d seconds", i);
	fl_show_alert(\
	"A transaction already happend", "at the same moment", temp, 1);
	}	

/* create a new entry */
pa = install_transaction_at_end_of_list(timestr);
if(! pa) return 0;

pa -> description = strsave(item);
if(! pa -> description) return 0;

pa -> status = 0; /* not used */
pa -> browser_line = 0;	
pa -> quantity = quantity;
pa -> price = price;
pa -> buy_sell = buy_sell;
pa -> account = account;
pa -> owned_quantity = owned_quantity;

/*
If this is the last transaction for this item in the transaction browser,
set the price in the market browser to the current price.
*/
latest_transaction_flag = 1;
for(pb = transactiontab[0]; pb != 0; pb = pb -> nxtentr)
	{
	if(strcmp(item, pa -> description) == 0) 
		{
		if(atol(pb -> name) > transactiontime)
			{
			latest_transaction_flag = 0;
			break;
			}/* end if later */
		}/* end if same description */	
	}/* end for all transactions in double linked list */

if(latest_transaction_flag)
	{
	if(! set_item_price(item, price) ) return 0;
	}

if(! show_transactions(SHOW_SAME_POSITION) ) return 0;

return 1;
}/* end function add_transaction_to_portfolio */


char *get_transaction_description(char *name)
{
struct transaction *pa;

if(debug_flag)
	{
	fprintf(stdout, "get_transaction_description(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;

pa = lookup_transaction(name);
if(! pa) return 0;

return pa -> description;
}/* end function get_transaction_sdescription */


