/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xkra.h"

FD_xkra *fdui;


struct stock
	{
	char *name;
	
	struct stock *nxtentr;
	struct stock *prventr;
	};
struct stock *stocktab[2];


struct stock *lookup_stock(char *name)
{
struct stock *pa;

if(debug_flag) 
	{
	fprintf(stdout, "lookup_stock(): arg name=%s\n", name);
	}

for(pa = stocktab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return(0); /* not found */
}


struct stock *install_stock_at_end_of_list(char *name)
{
struct stock *plast, *pnew;
struct stock *lookup_stock();

if(debug_flag)
	{
	fprintf(stdout, "install_stock_at_end_of_list(): arg name=%s\n", name);
	}

pnew = lookup_stock(name);
if(pnew)
	{
	/* else already there */
	if(debug_flag)
		{
		fprintf(stdout,\
		"install_stock_at_end_of_list(): already there ignored\n");
		}
	return(pnew);/* allow double entries in file, return old address */
	}

/* create new structure */
pnew = (struct stock *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = stocktab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !stocktab[0] ) stocktab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
stocktab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_stock_at_end_of_list */


int delete_stock(char *name)/* delete entry from double linked list */
{
struct stock *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stdout, "delete_stock(): arg name=%s\n", name);
	}

/* parameter check */
if(! name) return(0);

pa = stocktab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return(0);

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify stocktab[0] */
	if(pprev == 0) stocktab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify stocktab[1] */
	if(pnext == 0) stocktab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	/* delete structure data */	
	free(pdel -> name);/* free name */
	free(pdel); /* free structure */

	/* return OK deleted */
	return(1);
	}/* end for all structures */
}/* end function delete_stock */


int delete_all_stocks()/* delete all entries from table */
{
struct stock *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_stocks(): arg none\n");
	}

while(1)
	{
	pa = stocktab[0];
	if(! pa) break;
	stocktab[0] = pa -> nxtentr;/* stocktab entry points to next one,
												            this could be 0 */
	free(pa -> name);/* free name */

	free(pa);/* free structure */
	}/* end for all structures */
stocktab[1] = 0;

return 1;
}/* end function delete_all_stocks */


int add_stock(char *stock)
{
struct stock *pa;

if(debug_flag)
	{
	fprintf(stdout, "add_stock(): arg stock=%s\n", stock);
	}

/* argument check */
if(! stock) return 0;

pa = install_stock_at_end_of_list(stock); 
if(! pa) return 0;

return 1;
}/* add_function add_stock */


int test_stock(char *stock)
{
struct stock *pa;

if(debug_flag)
	{
	fprintf(stdout, "test_stock(): arg stock=%s\n", stock);
	}
	
/* argument check */
if(! stock) return 0;

pa = lookup_stock(stock);
if(! pa) return 0;

return 1;
}/* end function test_stock */


int print_volatilities()
{
struct stock *pa;
double volatility;
char temp[512];
float ddata[DATA_SIZE];
int data_points;
double daily_standard_deviation;
double yearly_standard_deviation;

if(debug_flag)
	{
	fprintf(stdout, "print_volatilities(): arg none\n");
	}

for(pa = stocktab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* get the historical close prices */
	sprintf(temp, "%s%s.krs", "/root/stockvac/", pa -> name);
	if(! load_stock_prices(temp, ddata, &data_points) )
		{
		fprintf(stdout,\
		"print_volatilities(): cannot load stockprices for fund %s\n",\
		temp);
		
		continue;
		/*return 0;*/
		}
	if(debug_flag)
		{
		fprintf(stdout,\
"print_volatility(): historical stock prices %d data points loaded\n",\
		data_points);
		}
	
	if(data_points < 2) continue;

	/* calculate the daily standard deviation */
	daily_standard_deviation = stddev(ddata, data_points);
	if(debug_flag)
		{
		fprintf(stdout,\
		"print_volatility(): daily standard deviation=%f\n",\
		daily_standard_deviation);
		}

	/* calculate the yearly standard deviation */
	/* there are 252 trading days in a year */
	yearly_standard_deviation = sqrt(252) * daily_standard_deviation;
	if(debug_flag)
		{
		fprintf(stdout,\
		"calculate_option_price(): yearly standard deviation=%f\n",\
		(float)yearly_standard_deviation);
		}

	fprintf(stdout,\
	"fund=%s data points=%d\n\
	daily standard deviation=%f\n\
	yearly_standard_deviation=%f\n",\
	pa -> name, data_points,\
	daily_standard_deviation,\
	yearly_standard_deviation); 
	}/* end for all entries */

return 1;
} /* end function print_volatilities */

