/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xkra.h"
#include "black_scholes_proto.h"

extern FD_xkra *fdui;
extern FD_xkra *fd_main_form;

FL_APPEVENT_CB optimize_idle_cb;

struct timeval timeout;
time_t idletimer; /* idle time for user */
char *present_time();

char ny_time[80];
int new_york_time();
int acquire_data(char *);
int update_minutes = 1;
time_t last_update_time = 0;

float tx[DATA_SIZE];
char ttext[DATA_SIZE][80];
float xvolume[DATA_SIZE], yvolume[DATA_SIZE];
float xclose[DATA_SIZE], yclose[DATA_SIZE];
float xrelative[DATA_SIZE], yrelative[DATA_SIZE];
float xmomentum[DATA_SIZE], ymomentum[DATA_SIZE];
float xshares[DATA_SIZE], yshares[DATA_SIZE];
float xvermogen[DATA_SIZE], yvermogen[DATA_SIZE];
float x[DATA_SIZE], y[DATA_SIZE];/* close prices */
/*static float xo1[DATA_SIZE], yo1[DATA_SIZE];*/ /* close prices not used */
float xo2[DATA_SIZE], yo2[DATA_SIZE];/* moving average 1 */
float xo3[DATA_SIZE], yo3[DATA_SIZE];/* moving average 2 */
float xo4[DATA_SIZE], yo4[DATA_SIZE];/* upper trading band */
float xo5[DATA_SIZE], yo5[DATA_SIZE];/* lower trading band */
float xo6[DATA_SIZE], yo6[DATA_SIZE];/* volume */
float xo7[DATA_SIZE], yo7[DATA_SIZE];/* relative strength */
float xo8[DATA_SIZE], yo8[DATA_SIZE];/* momentum */
float xo9[DATA_SIZE], yo9[DATA_SIZE];/* shares */
float xo10[DATA_SIZE], yo10[DATA_SIZE];/* vermogen */
float fymink, fymaxk;
float fyrelmink, fyrelmaxk;
int valuesread = -1/* not defined yet */;
int relative_valuesread = -1;/* not defined yet */
char fund_name[80], path_filename[512], relative_path_filename[80];
int volume_availeble_flag;
float last_close, total_capital;
int transactions;

float old_mawei1, old_mawei2;
int old_maday1, old_maday2;
int old_momentum_days;
float old_gain;
char last_year[1024];
char last_month[1024];
int last_day;


char *strsave(char *s) /*save char array s somewhere*/
{
char *p, *malloc();
if(p = malloc( strlen(s) +  1) )strcpy(p, s);
return(p);
}


int buy_sell_simulation(int buy_sell, float share_price, int position)
{
float shares_bought, shares_sold;
float transaction_price, transaction_cost;
float money_spend, money_received;
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"buy_sell_simulation(): buy_sell=%d share_price=%.2f position=%d\n",\
	buy_sell, share_price, position);
	}

if(buy_sell == BUY)
	{
	if(debug_flag)
		{
		fprintf(stdout, "i=%d\n", position);
		fprintf(stdout,
		"BUY enter: share_price=%4.2f present_cap=%4.2f shares=%4.2f\n",
		share_price, present_capital, present_shares);
		}

	/* avoid floating point exeption */
	if(share_price == 0) return(1);
	shares_bought = present_capital / share_price; 
	if(shares_bought == 0) return(1);
	while(1)/* iteration loop */
		{
		transaction_price = shares_bought * share_price;

		/* calculate the rates (from the rates set in the rates form) */
		if(! get_share_transaction_cost(\
		BUY, shares_bought, share_price,\
		transaction_price, &transaction_cost) ) return(0);

		if(debug_flag)
			{
			fprintf(stdout,\
			"shares_bought=%.2f transaction_price=%4.2f transaction_cost=%4.2f\n",\
			shares_bought, transaction_price, transaction_cost);		
			}

		money_spend = transaction_price + transaction_cost;
		if(money_spend <= present_capital) break;

		if(debug_flag)
			{
			fprintf(stdout,"ITERATING shares_bought=%4.2f\n", shares_bought);
			}

		shares_bought--;
		if(shares_bought <= 0)
			{
			shares_bought = 0;
			money_spend = 0;
			return(1);
			}
		}

	if(debug_flag)
		{
		fprintf(stdout, "money_spend=%4.2f\n", money_spend); 
		}

	present_capital -= money_spend;
	present_shares += shares_bought;

	if(debug_flag)
		{
		fprintf(stdout,\
		"BUY exiting: shares_bought=%f pres_cap=%4.2f pres_shar=%4.2f\n",\
		shares_bought, present_capital, present_shares);
		}

	transactions++;
	return(1);
	}/* end if BUY */
if (buy_sell == SELL)
	{

	if(debug_flag)
		{
		fprintf(stdout, "i=%d\n", position);
		fprintf(stdout,\
		"SELL enter: share_price=%4.2f present_cap=%4.2f pres_shar=%4.2f\n",\
		share_price, present_capital, present_shares);
		}

	shares_sold = present_shares;
	transaction_price = shares_sold * share_price;

	if(! get_share_transaction_cost(\
	SELL, shares_sold, share_price,\
	transaction_price, &transaction_cost) ) return(0);

	if(debug_flag)
		{
		fprintf(stdout,\
		"shares_sold=%.2f transaction_price=%4.2f transaction_cost=%4.2f\n",\
		shares_sold, transaction_price, transaction_cost);		
		}

	if(transaction_cost > present_shares * share_price)
		{
		shares_sold = 0;
		money_received = 0;
		}
	else
		{
		money_received = transaction_price - transaction_cost;
		}

	if(debug_flag)
		{
		fprintf(stdout, "money_received=%4.2f\n", money_received);
		}

	present_capital += money_received;
	present_shares -= shares_sold;

	if(debug_flag)
		{
		fprintf(stdout,\
		"SELL exiting: share_sold=%4.2f present_cap=%4.2f pres_shar=%4.2f\n",\
		shares_sold, present_capital, present_shares);
		}

	transactions++;
	return(1);
	}/* end if SELL */

if(debug_flag)
	{
	fprintf(stdout,\
	"position=%d present_capital=%4.2f present_shares=%f transactions=%d\n",\
	position, present_capital, present_shares, transactions);
	}

return(1);/* no trade */
}/* end function buy_sell_simulation */


int load_file(int relative_file_flag)
{
int a, b, i, j;
FILE *stockfile;
char fiwd[80], fimo[80], si[80], temp[80];
float trend, fmink, fmaxk;
int fida, af, pg, sp, xpos, ypos, lenfie, lenfr, specialfield;
int special_field_type;
float  value, closeprice, openprice, minprice, maxprice, volume;
int header_flag;
char header[80];
int month, day, year;
char field[6][80];
char date[80];

if(relative_file_flag == 0)
	{
	stockfile = fopen(path_filename, "r");
	file_loaded_flag = 0;
/*
fprintf(stdout, "loading %s\n", path_filename); 
*/
	}
if(relative_file_flag == 1)
	{
	stockfile = fopen(relative_path_filename, "r");
	relative_file_loaded_flag = 0;
/*
fprintf(stdout, "loading %s\n", relative_path_filename); 
*/
	}
if(! stockfile) 
	{
	if(! relative_file_flag) 
		{
		valuesread = -1;/* signal to end_input_cb that end is undefined */
		}
	if(relative_file_flag)
		{
		relative_valuesread = 0;
		}	
	return (0);
	}

/* clear the data box */
fl_set_object_label(fdui -> outfield, "");

if(! relative_file_flag)
	{
	for(i = 0; i < DATA_SIZE; i++)
		{
		tx[i] = 0;
		ttext[i][0] = 0;
		}/* end for clear text arrays */
	}		

if(strcmp(file_type, "krs") == 0)
	{
	volume_availeble_flag = 0;/* volume not supported */
	/* get header from file */
	fscanf(stockfile,"%s %s %d %s %f %d %d %d %d %d %d %f %f %d",\
		fiwd,fimo,&fida,si,&trend,&af,&pg,&sp,&xpos,&ypos,&lenfie,\
		&fmaxk,&fmink,&lenfr);

	/* save in globals the last update date for display only */
	strcpy(last_year, fiwd);
	strcpy(last_month, fimo);
	last_day = fida;

	/* read data from file */
	if(! relative_file_flag)
		{
		fymink = 1000000000;
		fymaxk = 0;
		}
	else
		{
		fyrelmink = 100000000;
		fyrelmaxk = 0;
		}	
	specialfield = 0;
	if(! relative_file_flag) valuesread = 0;
	else relative_valuesread = 0;
	while(1)	
		{
		a = fscanf(stockfile, "%s", temp);
		if(a == EOF)
			{
			fclose(stockfile);
			break;
			}
		value = atof(temp);
		if(value < 0)
			{
			special_field_type = value;
			specialfield = 1;
			continue; /* get next field */
			}
		if(specialfield)
			{
			specialfield = 0;
			if(! relative_file_flag)
				{
				tx[valuesread] = special_field_type;
				strcpy(ttext[valuesread], temp);
				}
			continue; /* scip */
			}
		if(value > 0) /* if valid */
			{
			if(! relative_file_flag)
				{
				closeprice = value;
				xclose[valuesread] = valuesread;
				yclose[valuesread] = closeprice;
				if(closeprice > fymaxk) fymaxk = closeprice;
				if(closeprice < fymink) fymink = closeprice;
				}
			else
				{
				xrelative[relative_valuesread] = valuesread;
				yrelative[relative_valuesread] = value;
				if(value > fyrelmaxk) fyrelmaxk = value;
				if(value < fyrelmink) fyrelmink = value;
				}
			special_field_type = 0;
			if(! relative_file_flag) valuesread++;
			else relative_valuesread++;
			}
		}/* end while read values from file */
	}/* end if file_type is krs */
else if(strcmp(file_type, "csv") == 0)
	{
	if(! relative_file_flag) volume_availeble_flag = 0;
	/* read data from file */
	header_flag = 0;

	if(! relative_file_flag)
		{
		fymink = 1000000000;
		fymaxk = 0;
		valuesread = 0;
		}
	else
		{
		fyrelmink = 1000000000;
		fyrelmaxk = 0;
		relative_valuesread = 0;
		}
	specialfield = 0;
	while(1)	
		{
		a = fscanf(stockfile, "%s\n", temp);
		if(a == EOF)
			{
			fclose(stockfile);
			break;
			}
		if(! header_flag)
			{
			sscanf(temp, "%s", header);
			header_flag = 1;
			}
		else/* data */
			{
			b = 0;/* field ptr */
			j = 0;/* pointer in field */
			for(i = 0; i < strlen(temp); i++)
				{
				a = temp[i];
				if(a == ',')
					{
					field[b][j] = 0;/* string termination */
					b++;/* point to next field */
					j = 0;/* point to first character in field */
					continue;/* scip the comma */
					}
				else field[b][j] = a;
				j++;
				}
			field[b][j] = 0;/* string termination last field */

			for(i = 0; i < strlen(header); i++)
				{
				switch(toupper(header[i]) )
					{
					case 'D':
						sscanf(field[i], "%d/%d/%d", &month, &day, &year);
						if(! relative_file_flag)
							{
							tx[valuesread] = -1;/* month */
							sprintf(ttext[valuesread], "%d/%d", month, year);
							}
						break;
					case 'C':
						sscanf(field[i], "%f", &closeprice);
						if(! relative_file_flag)
							{
/*
fprintf(stdout, "i=%d close=%f\n", valuesread, closeprice);
*/
							xclose[valuesread] = valuesread;
							yclose[valuesread] = closeprice;
							if(closeprice > fymaxk) fymaxk = closeprice;
							if(closeprice < fymink) fymink = closeprice;
							}
						else
							{
							xrelative[relative_valuesread] = valuesread;
							yrelative[relative_valuesread] = closeprice;
							if(closeprice > fyrelmaxk) fyrelmaxk = closeprice;
							if(closeprice < fyrelmink) fyrelmink = closeprice;
							}
						break;
					case 'V':
						if(! relative_file_flag)
							{
							sscanf(field[i], "%f", &volume);
							xvolume[valuesread] = valuesread;
							yvolume[valuesread] = volume;
							volume_availeble_flag = 1;
							}
						break;
					}/* end switch character in header */
				}/* end for all characters in header */
			if(! relative_file_flag) valuesread++;
			else relative_valuesread++;
			}/* end if data */
		}/* end while read in all fields */
	}/* end if file_type is csv */
/*	
fprintf(stdout, "fymink=%f fymaxk=%f\n", fymink, fymaxk);
*/
/* the following will be overruled by using start_input_cb and end_input_cb */
if(! relative_file_flag)
	{
	start = start;/* set in xkra.h defaults */
	end = valuesread;
	sprintf(temp, "%d", start);
	fl_set_input(fdui -> start_input, temp);
	sprintf(temp, "%d", end);
	fl_set_input(fdui -> end_input, temp);

	file_loaded_flag = 1;
	}
else
	{
	relative_file_loaded_flag = 1;
	}	

/*
signal that rates have not been read yet, simu button and rates button
will try to load rates if this flag is not set.
*/
share_rates_form_rates_read_flag = 0;
option_rates_form_rates_read_flag = 0;

return(1);
}/* end function load_file */


print_graph()
{
int a, b, i, j;
float fa, fb, fc;
double dfdaily_standard_deviation;
float  value, closeprice;
extern FD_xkra *fdui;
float fxmin, fxmax, fymin, fymax, fyperc, ftexty;
float madiv1, masum1, mamul1, maval1;
float madiv2, masum2, mamul2, maval2;
char temp[1024];
float utbp, ltbp;
float lastma1, lastma2, madiff, oldmadiff;
int oldmadiff_valid_flag;
int crosstype, oldcrosstype;
float volumemin, volumemax;
float pricerange, volumerange, scalefactor;
float relstrength, relmin, relmax, relrange;
int relxoffsett;
float momentum, mommax, mommin, momrange;
int lastcrosstype, maxmintype, oldmaxmintype;
int buy_sell;
int buy_sell_state;
float shares, sharesmax, sharesmin, sharesrange;
float vermogen, vermogenmax, vermogenmin, vermogenrange;

if(! file_loaded_flag) return(0);

if(! optimize_flag)
	{
	maday1 = atoi(fl_get_input(fdui -> ma1_input) );
	if(maday1 < 1) maday1 = 1;
	maday2 = atoi(fl_get_input(fdui -> ma2_input) );
	if(maday2 < 1) maday2 = 1;
	}/* end if ! optimize_flag */
	
/* START XFORMS BUG WORK AROUND
/* delete any previous overlays */
/* NOTE: whether they existed or not, to avoid segmentation error in xforms,
/* this happens if second x axis size is smaller then first xaxis size.
*/
if(! optimize_flag)
	{
	for(i = 1; i <= 10; i++)
		{
		fl_delete_xyplot_overlay(fdui -> ixyplot, i);
		}
	}/* end if ! optimize_flag */
/* END XFORMS BUG WORKAROUND */
	
/* for simulation only */
/* reset the simulation parameters */
present_shares = start_shares;
present_capital = start_capital;
transactions = 0;

/*
fprintf(stdout, "HAVE fymink=%f fymaxk=%f\n", fymink, fymaxk);
*/

/* calculate and plot close price as the main graph */
for(i = 0; i < valuesread; i++)
	{
	x[i] = xclose[i];
	y[i] = yclose[i];
	}

if(! optimize_flag)
	{
	fl_set_xyplot_xbounds(fdui -> ixyplot, start, end);
	fl_set_xyplot_ybounds(fdui -> ixyplot, fymink, fymaxk);

	fl_set_object_color(fdui -> ixyplot, FL_WHITE, FL_BLACK);

	sprintf(temp,\
	"trading days (last update %s %s %d price %.2f %.2f%c)",\
	last_year, last_month, last_day, yclose[valuesread - 1],\
	( (yclose[valuesread - 1] / yclose[valuesread - 2]) - 1) * 100,\
	'%');

	fl_set_xyplot_data(fdui -> ixyplot, x, y, valuesread,\
	fund_name, temp, "close     ");

	/* get the autoscale parameters */
	fl_get_xyplot_xbounds(fdui -> ixyplot, &fxmin, &fxmax);
	fl_get_xyplot_ybounds(fdui -> ixyplot, &fymin, &fymax);

	}/* end if ! optimize_flag */
/*
fprintf(stdout, "AUTOSCALE: fxmin=%f fxmax=%f fymin=%f fymax=%f\n",\
fxmin, fxmax, fymin, fymax);
*/

fyperc = (fymax - fymin ) / 100.0;

if(! optimize_flag)
	{
	/* over write main graph with same in white if no close price */
	if(close_flag)
		{
/*
NOTE:
EXEPTION OCCURS HERE IF FILETYPE CHANGE FROM .KRS TO .CSV FOR SAME FILE
.csv files are shorter
*/
		/*
		delete the white overlay over the black main graph,
		making it visible.
		*/
		fl_delete_xyplot_overlay(fdui -> ixyplot, 1);
		}
	else
		{
		/* no close price, overwrite close price main graph with white */
		fl_delete_xyplot_overlay(fdui -> ixyplot, 1);
		fl_add_xyplot_overlay(fdui -> ixyplot, 1, x, y, valuesread,\
		FL_WHITE);
		}
	}/* end if ! optimize_flag */

/* calculate and plot moving average 1 */
for(i = 0 ; i < valuesread; i++)
	{
	madiv1 = 0;/* divisor */
	masum1 = 0;/* reset av. */
	for(j = 0; j < maday1; j++)
		{
		a = i - j;/* ptr */
		if(a < 0) break;/* end */
		/* calculate weighting */
		mamul1 = 1 + mawei1 * (maday1 - 1 - j);
/*
fprintf(stdout,"i=%d j=%d mamul1=%f\n", i, j, mamul1);
*/
		/* prev. is 1 for weighting 0,
		/* if weighting = 1 then day 1 *1, day 2 *2 etc. */
		madiv1 += mamul1;
		masum1 += yclose[a] * mamul1;
		}/* end for j */
/*	if(j != maday1) maval1 = yclose[i];/* fymin; /*y[a];/* was -1, not valid */
	if(j != maday1) maval1 = -1;
	else
		{
		/* avoid divide by zero exeption */
		if(madiv1 != 0) maval1 = masum1 / madiv1;
		else maval1 = -1;
		}	
	b = i + ma1_lead_days;
	if(b < 0) continue;
	xo2[i] = i;
	yo2[b] = maval1;  
/*
fprintf(stdout, "moving_average 1 i=%d, maval1=%4.2f\n", i, maval1);
*/
	}/* end for i */

if(! optimize_flag)
	{
	if(ma1_flag)
		{
		/* print the moving average 1 */
		fl_add_xyplot_overlay(fdui -> ixyplot, 2, xo2, yo2, valuesread, FL_BLUE);
		}/* end if ma1_flag */
	else
		{
		fl_delete_xyplot_overlay(fdui -> ixyplot, 2);
		}	
	}/* end if ! optimize_flag */

/* calculate and plot moving average 2 */
for(i = 0 ; i < valuesread; i++)
	{
	madiv2 = 0;/* divisor */
	masum2 = 0;/* reset av. */
	for(j = 0; j < maday2; j++)
		{
		a = i - j;/* ptr */
		if(a < 0) break;/* end */
		/* calculate weighting */
		mamul2 = 1 + mawei2 * (maday2 - 1 - j);
		/* prev. is 1 for weighting 0,
		/* if weighting = 1 then day 1 *1, day 2 *2 etc. */
		madiv2 += mamul2;
		masum2 += yclose[a] * mamul2;
		}/* end for j */
/*	if(j != maday2) maval2 = yclose[i];/* fymin; /*y[a];/* was -1, not valid */
	if(j != maday2) maval2 = -1;
	else
		{
		/* avoid divide by zero exeption */
		if(madiv2 != 0) maval2 = masum2 / madiv2;
		else maval2 = -1;
		}
	b = i + ma2_lead_days;
	if(b < 0) continue;
	xo3[i] = i;
	yo3[b] = maval2;  
/*
fprintf(stdout, "moving_average 2 i=%d, maval1=%4.2f\n", i, maval2);
*/
	}/* end for i */

if(! optimize_flag)
	{
	if(ma2_flag)
		{
		/* print the moving average 2 */
		fl_add_xyplot_overlay(fdui -> ixyplot, 3, xo3, yo3, valuesread, FL_RED);
		}/* end if ma2_flag */
	else
		{
		fl_delete_xyplot_overlay(fdui -> ixyplot, 3);
		}	
	}/* end if ! optimize_flag */
	
/* calculate and plot the upper band */
if(upper_flag)
	{
	utbp = 3.0;
	for(i = 0; i < valuesread; i++)
		{
		xo4[i] = xclose[i];
		yo4[i] = yo2[i] + (utbp / 100.0) * yo2[i];
		}
	if(! optimize_flag)
		{
		fl_add_xyplot_overlay(fdui -> ixyplot, 4, xo4, yo4,\
		valuesread, FL_ORCHID);
		}
	}
else
	{
	if(! optimize_flag) fl_delete_xyplot_overlay(fdui -> ixyplot, 4);
	}
	
/* calculate and plot the lower band */
if(lower_flag)
	{
	ltbp = 2.0;
	for(i = 0; i < valuesread; i++)
		{
		xo5[i] = xclose[i];
		yo5[i] = yo2[i] - (ltbp / 100.0) * yo2[i];
		}
	if(! optimize_flag)
		{
		fl_add_xyplot_overlay(fdui -> ixyplot, 5, xo5, yo5,\
		valuesread, FL_DARKORANGE);
		}
	}
else
	{
	if(! optimize_flag) fl_delete_xyplot_overlay(fdui -> ixyplot, 5);
	}

if(! optimize_flag)
	{
	/* print year, month, ex divident occurences */
	/* NOTE XFORMS fails here, bug reported, so far no fix */
	if(text_flag)
		{
		/* print the month year and ex divident in the graph */
		for(i = 0; i < valuesread; i++)
			{
			if(tx[i] == -1)/* month */
				{
				/* calculate the vertical print position */
				ftexty = fymin + fyperc * 7.5; 
				strcpy(temp, ttext[i]);
				temp[1] = 0;/* string termination, only print first char in graph */
				fl_add_xyplot_text(fdui -> ixyplot, (float)i, ftexty,\
				temp, FL_ALIGN_LEFT_TOP, FL_DARKGOLD);
/*
fprintf(stdout, "printing MONTH %s at x=%d y=%f\n", temp, i, ftexty);
*/
				}
			if(tx[i] == -2)/* ex divident day of month */
				{
				/* calculate the vertical print position */
				ftexty = yclose[i];
				fl_add_xyplot_text(fdui -> ixyplot, (float)i, ftexty,\
				ttext[i], FL_ALIGN_LEFT_TOP, FL_DARKGOLD);
/*	
fprintf(stdout, "printing EXDIV %s at x=%d y=%f\n", ttext[i], i, ftexty);
*/	
				}
			if(tx[i] == -3)/* year */
				{
				/* calculate the vertical print position */
				ftexty = fymin + fyperc * 15; 
				fl_add_xyplot_text(fdui -> ixyplot, (float)i, ftexty,\
				ttext[i], FL_ALIGN_LEFT_TOP, FL_DARKGOLD);
/*
fprintf(stdout, "printing YEAR %s at x=%d y=%f\n", ttext[i], i, ftexty);
*/
				}		
			}/* end for all text fields */
	
		}/* end if text_flag */
	else
		{
		/* clear any text in graph */
		for(i = 0; i < DATA_SIZE; i++)
			{
			if(tx[i] == -1)/* month */
				{
				strcpy(temp, ttext[i]);
				temp[1] = 0;/* string termination first character month only */
				fl_delete_xyplot_text(fdui -> ixyplot, temp);
				}
			if(tx[i] == -2)/* ex divident day of month */
				{
				fl_delete_xyplot_text(fdui -> ixyplot, ttext[i]);
				}
			if(tx[i] == -3)/* year */
				{
				fl_delete_xyplot_text(fdui -> ixyplot, ttext[i]);
				}		
			}/* end for erase all kinds of text */
		}/* end else no text_flag */
	}/* end if ! optimize_flag */
	
if(! optimize_flag)
	{
	/* plot the volume */
	if(volume_flag)
		{
		volumemin = 1000000000;
		volumemax = -1;
		/* determine maximum and minimum */
		for(i = 0; i < valuesread; i++)
			{
			if(yvolume[i] < volumemin) volumemin = yvolume[i];
			if(yvolume[i] > volumemax) volumemax = yvolume[i];
			}
		pricerange = fymaxk - fymink;
		volumerange = volumemax - volumemin;
		/* avoid divide by zero (and floating point exeption) */
		if(volumerange == 0) volumerange = .001;
		scalefactor = pricerange / volumerange;
		/* rescale to closeprices */
		for(i = 0; i < valuesread; i++)
			{
			xo6[i] = i;
			yo6[i] = ( (yvolume[i] - volumemin) * scalefactor) + fymink;
			}
		fl_add_xyplot_overlay(fdui -> ixyplot, 6, xo6, yo6,\
		valuesread, FL_DARKTOMATO);
	
		sprintf(temp, "volume %.0f to %.0f", volumemin, volumemax);
		fl_set_object_label(fdui -> volumefield, temp);

		}/* end if volume_flag */
	else
		{
		fl_delete_xyplot_overlay(fdui -> ixyplot, 6);
		fl_set_object_label(fdui -> volumefield, "");
		}/* end if not volume_flag */
	}/* end if ! optimize_flag */

if(! optimize_flag)
	{
	/* calculate relative strength */
	if(relative_flag && file_loaded_flag && (yclose[valuesread - 1] != 0) )
		{
		/* calculate relative strenght relative to last value */
		scalefactor = yrelative[relative_valuesread - 1] / yclose[valuesread - 1];
		relmax = -1;
		relmin = 1000000000;
		a = relative_valuesread - 1;
		for(i = valuesread - 1; i >= 0; i--)
			{
			xo7[i] = i;
			if(a >= 0)
				{
				/* avoid divide by zero exeption */
				if(yrelative[a] == 0) fa = .001;
				else fa = (yclose[i] * scalefactor) / yrelative[a];
				if(fa < relmin) relmin = fa;
				if(fa > relmax) relmax = fa;
				yo7[i] = fa;
				}	
			else
				{
				yo7[i] = fymink;/*flag */
				}
			a--;
			}
		/* rescale relative range to full scale */	
		relrange = relmax - relmin;
		pricerange = fymaxk - fymink;
		if(relrange == 0) relrange = .001;
		scalefactor = pricerange / relrange;
		for(i = 0; i < valuesread; i++)
			{
			yo7[i] = ( (yo7[i] - relmin) * scalefactor ) + fymink;
			}

		fl_add_xyplot_overlay(fdui -> ixyplot, 7, xo7, yo7,\
		valuesread, FL_GREEN);
		}
	else
		{
		fl_delete_xyplot_overlay(fdui -> ixyplot, 7);
		}/* end relative_flag not set */
	}/* end if ! optimize_flag */

/* calculate and plot momentum */
mommax = -1;
mommin = 1000000000;
a = valuesread - 1 - momentum_days;
for(i = valuesread - 1; i >= 0; i--)
	{
	xo8[i] = i;
	if(a >= 0) 
		{
		fa = (yclose[i] / yclose[a]) * 100.0;
		ymomentum[i] = fa;
		if(fa > mommax) mommax = fa;
		if(fa < mommin) mommin = fa;
		}
	else
		{
		ymomentum[i] = -1;
		}
	a--;
	}/* end for momentum calculation */
if(! optimize_flag)
	{
	if(momentum_flag)
		{
		momrange = mommax - mommin;
		pricerange = fymaxk - fymink;
		if(momrange == 0) momrange = .001;
		scalefactor = pricerange / momrange;
		for(i = 0; i < valuesread; i++)
			{
			if(ymomentum[i] != -1)/* valid */
				{
				yo8[i] = ( (ymomentum[i] - mommin) * scalefactor ) + fymink;
				}
			else
				{
				yo8[i] = fymink;
				}
			}

		fl_add_xyplot_overlay(fdui -> ixyplot, 8, xo8, yo8,\
		valuesread, FL_DARKCYAN);
		}/* end if momentum_flag */
	else
		{
		fl_delete_xyplot_overlay(fdui -> ixyplot, 8);
		}/* end if not momentum_flag */
	}/* end if !optimize_flag */

/* calculate gold - dead cross, maximum - minimum, shares in simulation */
sharesmax = -1;
sharesmin = 1000000000;
buy_sell = 0;
buy_sell_state = 0;/* have not bought, 1 is bought */
lastcrosstype = 0;
maxmintype = 0;
oldmaxmintype = 0;
crosstype = 0;
oldcrosstype = 0;
oldmadiff = 0;
oldmadiff_valid_flag = 0;
for(i = 0; i < valuesread; i++)
	{
	xshares[i] = i;
	yshares[i] = present_shares;
	if(present_shares > sharesmax) sharesmax = present_shares;
	if(present_shares < sharesmin) sharesmin = present_shares;
	
	vermogen = present_shares * yclose[i] + present_capital;
	xvermogen[i] = i;
	yvermogen[i] = vermogen;
	if(vermogen > vermogenmax) vermogenmax = vermogen;
	if(vermogen < vermogenmin) vermogenmin = vermogen;
		
	/* test if both moving averages valid */
	if( (yo2[i] == -1) || (yo3[i] == -1) ) continue;
	
	if(maday2 > maday1) madiff = yo2[i] - yo3[i];
	else madiff = yo3[i] - yo2[i];	

/*
fprintf(stdout,\
"yo2[%d]=%.2f yo3[%d]=%.2f\n",\
i, yo2[i], i, yo3[i]); 
fprintf(stdout,\
"maday1=%d maday2=%d madiff=%.2f oldmadiff=%.2f oldmadiff_valid_flag=%d\n",
maday1, maday2, madiff, oldmadiff, oldmadiff_valid_flag);
*/

	if(oldmadiff_valid_flag && (i > maday1) && (i > maday2) )
		{
		/* gold cross / dead cross determination */	
		if( (madiff > 0) && (oldmadiff <= 0) ) 
			{
			crosstype = GOLD_CROSS;
			lastcrosstype = crosstype;
			}
		else if( (madiff < 0) && (oldmadiff >= 0) )
			{
			crosstype = DEAD_CROSS;
			lastcrosstype = crosstype;
			}
		if(crosstype == oldcrosstype)
			{
			crosstype = 0;
			oldcrosstype = 0;
			}
		oldcrosstype = crosstype;	

		/* maximum / minimum determination */
		if( (lastcrosstype == GOLD_CROSS) &&\
		(ymomentum[i] < 100) &&\
/*		(yo2[i] < yo2[i - momentum_days]) &&\*/
		(i > (momentum_days + maday1) ) )
			{
/*
fprintf(stdout, "MAX i=%d ymomentum[i]=%4.2f\n", i, ymomentum[i]);
*/
			maxmintype = MAXIMUM;/* going down in upper trading band */				lastcrosstype = 0;/* inhibit retrigger */
			}
		if( (lastcrosstype == DEAD_CROSS) &&\
		(ymomentum[i] > 100) &&\
/*		(yo2[i] > yo2[i - momentum_days]) &&\*/
		(i > (momentum_days + maday1) ) )
			{
/*
fprintf(stdout, "MIN i=%d ymomentum[i]=%4.2f\n", i, ymomentum[i]);
*/
			maxmintype = MINIMUM;/* going up in lower trading band */
			lastcrosstype = 0;
			}	
		if(maxmintype == oldmaxmintype)
			{
			maxmintype = 0;
			oldmaxmintype = 0;
			}
		oldmaxmintype = maxmintype;
		}/* end if oldmadiff_valid_flag */
/*
fprintf(stdout,\
"crosstype=%d oldcrosstype=%d\n\
maxmintype=%d oldmaxmintype=%d\n",\
crosstype, oldcrosstype, maxmintype, oldmaxmintype);
*/
	/* buy / sell determination */
	if(decision_methode == GOLD_DEAD)
		{
/*
fprintf(stdout, "DEC: i=%d crosstype=%d\n", i, crosstype);
*/
		if(crosstype == GOLD_CROSS) buy_sell = BUY;
		else if(crosstype == DEAD_CROSS) buy_sell = SELL;
		else buy_sell = 0;
		}
	if(decision_methode == MAX_MIN)
		{
/*fprintf(stdout, "setting buy_sell in decision_methode MAX_MIN\n");*/
		if(maxmintype == MINIMUM) buy_sell = BUY;
		else if(maxmintype == MAXIMUM) buy_sell = SELL;
		else buy_sell = 0;
		}
			
	oldmadiff = madiff;
	oldmadiff_valid_flag = 1;
/*
fprintf(stdout,
"setting OMAD decision_methode=%d buy_sell=%d\n\
simulation_flag=%d optimize_flag=%d\n",\
decision_methode, buy_sell,\
simulation_flag, optimize_flag);
*/
	if( simulation_flag || optimize_flag)
		{
/*
fprintf(stdout, "CALLING: i=%d crosstype=%d buy_sell=%d\n",\
i, crosstype, buy_sell);
*/
		if(! buy_sell_simulation(buy_sell, yclose[i], i) )
			{
			return(0);
			}

		}
	
	if(! optimize_flag)
		{
		if(i == valuesread - 1)/* only the present */
			{	
			if(decision_methode == GOLD_DEAD)
				{
				if(crosstype == GOLD_CROSS)	
					{
					fl_set_object_color(fdui -> info_light,\
					FL_DARKGOLD, FL_DARKGOLD);
					fl_set_object_label(fdui -> info_light, "GOLD CROSS");
					}
				else if(crosstype == DEAD_CROSS)
					{
					fl_set_object_color(fdui -> info_light,\
					FL_LEFT_BCOL, FL_COL1);
					fl_set_object_label(fdui -> info_light, "DEAD CROSS");
					}
				else
					{
					fl_set_object_color(fdui -> info_light, FL_COL1, FL_COL1);
					fl_set_object_label(fdui -> info_light, "NO ACTION");
					}
				}/* end decision_methode == GOLD_DEAD */
			if(decision_methode == MAX_MIN)
				{
				if(maxmintype == MINIMUM)
					{
					fl_set_object_color(fdui -> info_light,\
					FL_DARKGOLD, FL_DARKGOLD);
					fl_set_object_label(fdui -> info_light, "MINIMUM");
					}
				else if(maxmintype == MAXIMUM)
					{
					fl_set_object_color(fdui -> info_light,\
					FL_LEFT_BCOL, FL_COL1);
					fl_set_object_label(fdui -> info_light, "MAXIMUM");
					}
				else
					{
					fl_set_object_color(fdui -> info_light, FL_COL1, FL_COL1);
					fl_set_object_label(fdui -> info_light, "NO ACTION");
					}
				}/* end decision_methode == MAX_MIN */

			if(auto_trade_flag)
				{
				autotrade(crosstype);
				}
			}/* end if i = valuesread -1 && ! optimize_flag */
		}/* end if ! optimize _flag */
	}/* end for all values read */
	
if(optimize_flag)
	{
	last_close = yclose[valuesread - 1];
	total_capital = present_shares * last_close + present_capital;

	if(start_capital == 0) start_capital = .001;
	gain = total_capital / start_capital;
	return(1);
	}/* end if optimize_flag */

if(shares_flag && simulation_flag)
	{
	/* rescale and plot shares */
	sharesrange = sharesmax - sharesmin;
	pricerange = fymaxk - fymink;
	if(sharesrange == 0) sharesrange = .001;
	scalefactor = pricerange / sharesrange;
	for(i = 0; i < valuesread; i++)
		{
		xo9[i] = i;
		yo9[i] = ( (yshares[i] - sharesmin) * scalefactor ) + fymink;
		}
	fl_add_xyplot_overlay(fdui -> ixyplot, 9, xo9, yo9,\
	valuesread, FL_DEEPPINK);
	}/* end if shares_flag && simulation_flag */
else
	{
	fl_delete_xyplot_overlay(fdui -> ixyplot, 9);
	}/* end if not shares_flag && simulation_flag */

if(vermogen_flag && simulation_flag)
	{
	/* rescale and plot vermogen */
	vermogenrange = vermogenmax - vermogenmin;
	pricerange = fymaxk - fymink;
	if(vermogenrange == 0) vermogenrange = .001;
	scalefactor = pricerange / vermogenrange;
	for(i = 0; i < valuesread; i++)
		{
		xo10[i] = i;
		yo10[i] = ( (yvermogen[i] - vermogenmin) * scalefactor ) + fymink;
		}
	fl_add_xyplot_overlay(fdui -> ixyplot, 10, xo10, yo10,\
	valuesread, FL_DOGERBLUE);
	}/* end if vermogen_flag && simulation_flag */
else
	{
	fl_delete_xyplot_overlay(fdui -> ixyplot, 10);
	}/* end if not vermogen_flag && simulation_flag */

/*
fl_set_xyplot_data(fdui -> ixyplot, x, y, valuesread,\
fund_name, "trading days", "close     ");
*/

/* set the data box text */
fl_set_object_label(fdui -> outfield, "click on graph for data");

if(simulation_flag)
	{
	last_close = yclose[valuesread - 1];
	total_capital = present_shares * last_close + present_capital;

	sprintf(temp, "shares=%4.2f cash=%4.2f tr=%d",\
	present_shares, present_capital, transactions);
	fl_set_object_label(fdui -> outfield, temp);

	if(start_capital == 0) start_capital = .001;

	gain = total_capital / start_capital;
	sprintf(temp, "tot_cap=%4.2f gain=%4.1f",\
	total_capital, gain);
	fl_set_object_label(fdui -> volumefield, temp);
	}

return(1);
}/* end function print_graph */



/* callbacks for form main_form */

void file_type_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
file_type_flag = a;
if(a == 0) fl_set_object_label(ob, ".krs");
if(a == 1) fl_set_object_label(ob, ".csv");

/*
read the settings for the moving averages, this may fail,
if so use user defaults (as specified in xkra.c).
*/
sprintf(moving_averages_path_filename, "%s%s.mva", file_path, fund_name);
read_settings();

if(! load_file(0) ) return;

print_graph();
} 

void decision_switch_cb(FL_OBJECT *ob, long data)
{
int a;

a = fl_get_button(ob);

decision_methode++;
if(decision_methode == 2) decision_methode = 0;
if(decision_methode == GOLD_DEAD)
	{
	fl_set_object_label(ob, "cross");
	fl_set_object_lcol(ob, FL_BLACK);
	}
if(decision_methode == MAX_MIN)
	{
	fl_set_object_label(ob, "max");
	fl_set_object_lcol(ob, FL_MAGENTA);
	}
/* add your own schemes here, then increase the == 2 above */	

/*
read the settings for the moving averages, this may fail,
if so use user defaults (as specified in xkra.c).
*/
sprintf(moving_averages_path_filename, "%s%s.mva", file_path, fund_name);
read_settings();

if(! load_file(0) ) return;

print_graph();
}/* end function decision_switch_cb */ 


void exit_switch_cb(FL_OBJECT *ob, long data)
{
int some_form_open_flag;

/* test if any forms are open */
if(! test_for_open_forms(&some_form_open_flag) ) return;
if(some_form_open_flag) return;

if(! save_global_settings() )
	{
	fl_show_alert(\
	"Cannot save global setting",\
	"exiting in 4 seconds",\
	"settings not saved",\
	1);
	sleep(4);
	}
exit(1);
}


void close_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
close_flag = 1 - a;
print_graph();
}


void shares_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
shares_flag = a;
print_graph();
}


void vermogen_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
vermogen_flag = a;
print_graph();
}


void relative_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
relative_flag = a;
if(a)
	{
	sprintf(relative_path_filename, "%s%s.%s",\
	file_path, relative_filename, file_type);
	if(! load_file(1) ) return;
	}
print_graph();
}


void momentum_switch_cb(FL_OBJECT *ob, long data)
{
int a;

a = fl_get_button(ob);
momentum_flag = a;

print_graph();
}


void update_switch_cb(FL_OBJECT *ob, long data)
{
int a;
if(! file_loaded_flag)
	{
	fl_show_alert("No fund specified", "", "", 1);
	fl_set_button(ob, 0);
	return;
	}

a = fl_get_button(ob);
update_flag = a;
/* if not updating reset the auto_trade_switch */
if(! update_flag)
	{
	if(auto_trade_flag)
		{
		auto_trade_flag = 0;
		fl_set_button(fdui -> auto_trade_switch, 0);
		fl_set_object_lcol(fdui -> auto_trade_switch, FL_BLACK);
		fl_set_object_label(fdui -> auto_trade_switch, "MAN");
		}
	}

}/* end function update_switch_cb */


void auto_trade_switch_cb(FL_OBJECT *ob, long data)
{
int a;

/* if not updating inhibit auto trade */
if(! update_flag)
	{
	fl_set_button(ob, 0);
	return;
	}

a = fl_get_button(ob);
auto_trade_flag = a;
if(a == 0)
	{
	fl_set_object_lcol(ob, FL_BLACK);
	fl_set_object_label(ob, "MAN");
	}
if(a == 1)
	{
	fl_set_object_lcol(ob, FL_WHITE);
	fl_set_object_label(ob, "AUTO");
	}
}


void simulation_switch_cb(FL_OBJECT *ob, long data)
{
int a;

if(! file_loaded_flag)
	{
	fl_show_alert("No fund specified", "", "", 1);
	fl_set_button(ob, 0);
	return;
	}

a = fl_get_button(ob);
simulation_flag = a;
/*
if(a == 0)
	{
	fl_set_button(fdui -> vermogen_switch, 0);
	fl_set_button(fdui -> shares_switch, 0);
	}
*/
if(! read_share_rates() )
	{
	return;
	}

print_graph();
}/* end function simulation_switch_cb */


void volume_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
volume_flag = a;
print_graph();
}


void ma1_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
ma1_flag = a;
print_graph();
}


void ma2_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
ma2_flag = a;
print_graph();
}


void text_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
text_flag = a;
print_graph();
}


void upper_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
upper_flag = a;
print_graph();
}


void lower_switch_cb(FL_OBJECT *ob, long data)
{
int a;
a = fl_get_button(ob);
lower_flag = a;
print_graph();
}


void optimize_button_cb(FL_OBJECT *ob, long data)
{
int a;

if(! file_loaded_flag)
	{
	fl_show_alert("No fund specified", "", "", 1);
	fl_set_button(ob, 0);
	return;
	}

a = fl_get_button(ob);
if(a)
	{
	if(! read_share_rates() )
		{
		return;
		}

	fl_deactivate_object(fdui -> lower_switch);
	fl_deactivate_object(fdui -> ixyplot);
	fl_deactivate_object(fdui -> info_light);
	fl_deactivate_object(fdui -> fund_input);
	fl_deactivate_object(fdui -> directory_input);
	fl_deactivate_object(fdui -> outfield);
	fl_deactivate_object(fdui -> volumefield);
	fl_deactivate_object(fdui -> close_switch);
	fl_deactivate_object(fdui -> volume_switch);
	fl_deactivate_object(fdui -> start_input);
	fl_deactivate_object(fdui -> end_input);
	fl_deactivate_object(fdui -> moving_average_1_switch);
	fl_deactivate_object(fdui -> ma1_input);
	fl_deactivate_object(fdui -> moving_average_2_switch);
	fl_deactivate_object(fdui -> ma2_input);
	fl_deactivate_object(fdui -> mawei1_input);
	fl_deactivate_object(fdui -> mawei2_input);
	fl_deactivate_object(fdui -> text_switch);
	fl_deactivate_object(fdui -> upper_switch);
	fl_deactivate_object(fdui -> lower_switch);
	/*fl_deactivate_object(fdui -> exit_switch);*/
	/*fl_deactivate_object(fdui -> local_clock);*/
	/*fl_deactivate_object(fdui -> new_york_clock);*/
	/*fl_deactivate_object(fdui -> ny_clock_id);*/
	fl_deactivate_object(fdui -> file_type_switch);
	fl_deactivate_object(fdui -> relative_strength_switch);
	fl_deactivate_object(fdui -> update_switch);
	fl_deactivate_object(fdui -> relative_file_input);
	fl_deactivate_object(fdui -> auto_trade_switch);
	fl_deactivate_object(fdui -> momentum_switch);
	fl_deactivate_object(fdui -> momentum_days_input);
	fl_deactivate_object(fdui -> ma1_lead_input);
	fl_deactivate_object(fdui -> ma2_lead_input);
	fl_deactivate_object(fdui -> simulation_switch);
	fl_deactivate_object(fdui -> decision_switch);
	fl_deactivate_object(fdui -> vermogen_switch);
	fl_deactivate_object(fdui -> shares_switch);
	fl_deactivate_object(fdui -> start_capital_input);
	fl_deactivate_object(fdui -> main_form_window_menu);

	old_mawei1 = -1;
	old_mawei2 = -1;
	old_maday1 = -1;
	old_maday2 = -1;
	old_momentum_days = -1;
	old_gain = -1;/* to start an ill. value, gain is always postive or 0 */

	optimize_flag = 1;
	return;
	}
else
	{
	optimize_flag = 0;
	
	fl_activate_object(fdui -> lower_switch);
	fl_activate_object(fdui -> ixyplot);
	fl_activate_object(fdui -> info_light);
	fl_activate_object(fdui -> fund_input);
	fl_activate_object(fdui -> directory_input);
	fl_activate_object(fdui -> outfield);
	fl_activate_object(fdui -> volumefield);
	fl_activate_object(fdui -> close_switch);
	fl_activate_object(fdui -> volume_switch);
	fl_activate_object(fdui -> start_input);
	fl_activate_object(fdui -> end_input);
	fl_activate_object(fdui -> moving_average_1_switch);
	fl_activate_object(fdui -> ma1_input);
	fl_activate_object(fdui -> moving_average_2_switch);
	fl_activate_object(fdui -> ma2_input);
	fl_activate_object(fdui -> mawei1_input);
	fl_activate_object(fdui -> mawei2_input);
	fl_activate_object(fdui -> text_switch);
	fl_activate_object(fdui -> upper_switch);
	fl_activate_object(fdui -> lower_switch);
	/*fl_activate_object(fdui -> exit_switch);*/
	/*fl_activate_object(fdui -> local_clock);*/
	/*fl_activate_object(fdui -> new_york_clock);*/
	/*fl_activate_object(fdui -> ny_clock_id);*/
	fl_activate_object(fdui -> file_type_switch);
	fl_activate_object(fdui -> relative_strength_switch);
	fl_activate_object(fdui -> update_switch);
	fl_activate_object(fdui -> relative_file_input);
	fl_activate_object(fdui -> auto_trade_switch);
	fl_activate_object(fdui -> momentum_switch);
	fl_activate_object(fdui -> momentum_days_input);
	fl_activate_object(fdui -> ma1_lead_input);
	fl_activate_object(fdui -> ma2_lead_input);
	fl_activate_object(fdui -> simulation_switch);
	fl_activate_object(fdui -> decision_switch);
	fl_activate_object(fdui -> vermogen_switch);
	fl_activate_object(fdui -> shares_switch);
	fl_activate_object(fdui -> start_capital_input);
	fl_activate_object(fdui -> main_form_window_menu);
	
	/* write back the new moving average settings */
	write_settings();

	print_graph(0);
	return;
	}/* enf if ! a (button not pressed) */
}/* end function optimize_button_cb */


void start_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
int a;
char temp[80];

sprintf(user_data, "%79s", fl_get_input(ob) );
if(strcmp(user_data, "start") == 0) start = 0;
else
	{
	a = atoi(user_data);
	if(a > end) start = end;
	else if(a < 0) start = 0;
	else start = a;
	}
sprintf(temp, "%d", start);
fl_set_input(ob, temp);

print_graph();
}


void end_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
int a;
char temp[80];

sprintf(user_data, "%79s", fl_get_input(ob) );
if(strcmp(user_data, "end") == 0) end = valuesread;
else
	{
	a = atoi(user_data);
	if(a > valuesread) end = valuesread;
	else if(a < start) end = start;
	else end = a;
	}

sprintf(temp, "%d", end);
fl_set_input(ob, temp);

if(! print_graph() ) return;
}


void ma1_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
int a;
char temp[80];

sprintf(user_data, "%79s", fl_get_input(ob) );
a = atoi(user_data);
if(a >= 0) maday1 = a;

sprintf(temp, "%d", maday1);
fl_set_input(ob, temp);

/* save settings to disk */
if(! write_settings() ) return;

if(! print_graph() ) return;
}


void ma2_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
int a;
char temp[80];

sprintf(user_data, "%79s", fl_get_input(ob) );
a = atoi(user_data);
if(a >= 0) maday2 = a;

sprintf(temp, "%d", maday2);
fl_set_input(ob, temp);

/* save settings to disk */
if(! write_settings() ) return;

if(! print_graph() ) return;
}


void mawei1_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
float fa;
char temp[80];

sprintf(user_data, "%79s", fl_get_input(ob) );
fa = atof(user_data);
if(fa >= 0) mawei1 = fa;

sprintf(temp, "%.1f", mawei1);
fl_set_input(ob, temp);

/* save settings to disk */
if(! write_settings() ) return;

if(! print_graph() ) return;
}


void mawei2_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
float fa;
char temp[80];

sprintf(user_data, "%79s", fl_get_input(ob) );
fa = atof(user_data);
if(fa >= 0) mawei2 = fa;

sprintf(temp, "%.1f", mawei2);
fl_set_input(ob, temp);

/* save settings to disk */
if(! write_settings() ) return;

if(! print_graph() ) return;
}


void ma1_lead_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
char temp[80];
int a;

sprintf(user_data, "%79s", fl_get_input(ob) );
a = atoi(user_data);
if(a >= 0) ma1_lead_days = a;

sprintf(temp, "%d", ma1_lead_days);
fl_set_input(ob, temp);

/* save settings to disk */
if(! write_settings() ) return;

if(! print_graph() ) return;
}


void ma2_lead_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
char temp[80];
int a;

sprintf(user_data, "%79s", fl_get_input(ob) );
a = atoi(user_data);
if(a >= 0) ma2_lead_days = a;

sprintf(temp, "%d", ma2_lead_days);
fl_set_input(ob, temp);

/* save settings to disk */
if(! write_settings() ) return;

if(! print_graph() ) return;
}


void fund_input_cb(FL_OBJECT *ob, long data)
{
char temp[1204];

sprintf(fund_name, "%s", fl_get_input(ob) );

if(strcmp(fund_name, "exit") == 0) exit(1);
if(strcmp(fund_name, "quit") == 0) exit(1);

/* keep the focus on this input field */
fl_set_focus_object(fd_main_form -> main_form, ob);

if(file_type_flag == 0) strcpy(file_type, "krs");
else if(file_type_flag == 1) strcpy(file_type, "csv");

sprintf(path_filename, "%s%s.%s", file_path, fund_name, file_type);

/*
read the settings for the moving averages, this may fail,
if so use user defaults (as specified in xkra.c).
*/
sprintf(moving_averages_path_filename, "%s%s.mva", file_path, fund_name);
read_settings();

if(! load_file(0) ) return;

print_graph();
/* clear the input field */
fl_set_input(ob, "");
}/* end function fund_input_cb */


void directory_input_cb(FL_OBJECT *ob, long data)
{
char *ptr;
char temp[512];
int some_form_open_flag;

if(! test_for_open_forms(&some_form_open_flag) ) return;

if(some_form_open_flag) return;

ptr = (char *) fl_get_input(ob);
if(! ptr) return;

if(strlen(ptr) > 511)
	{
	sprintf(temp, "%d bytes", strlen(ptr) );
	fl_show_alert("Path to long", temp, "max. 511 bytes", 1);
	return;
	}
strcpy(file_path, ptr);

/* if file_path does not end in '/' append it */
if(file_path[strlen(file_path) - 1] != '/') strcat(file_path, "/");

if(! save_stock_directory_setting() )
	{
	return;
	}

directory_changed_flag = 1;

}/* end function directory_input_cb */


void relative_file_input_cb(FL_OBJECT *ob, long data)
{
sprintf(relative_filename, "%s", fl_get_input(ob) );

if(file_type_flag == 0) strcpy(file_type, "krs");
else if(file_type_flag == 1) strcpy(file_type, "csv");

sprintf(relative_path_filename, "%s%s.%s",\
file_path, relative_filename, file_type);
if(load_file(1) ) print_graph();

write_settings();
}


void momentum_days_input_cb(FL_OBJECT *ob, long data)
{
char user_data[512];
int a;
char temp[80];

sprintf(user_data, "%79s", fl_get_input(ob) );
a = atoi(user_data);
if(a >= 0) momentum_days = a;

sprintf(temp, "%d", momentum_days);
fl_set_input(ob, temp);

/* save settings to disk */
if(! write_settings() ) return;

print_graph();
}


void start_capital_input_cb(FL_OBJECT *ob, long data)
{
char user_data[256];
float fa;
char temp[80];

sprintf(user_data, "%79s", fl_get_input(ob) );
fa = atof(user_data);
if(fa >= 0) start_capital = fa;

sprintf(temp, "%4.2f", start_capital);
fl_set_input(ob, temp);

if(! print_graph() ) return;
}


void ixyplotcb(FL_OBJECT *ob, long data)
{
int a, i, j, n;
float fx, fy, fxmin, fxmax, fymin, fymax, ftextx, ftexty;
char temp[80];
char month[10];
char year[10];
extern FD_xkra *fdui;
FL_OBJECT *obj;
int clicked_x_pos;

fl_get_xyplot(ob, &fx, &fy, &n);

/*
Note: fx will hold points from start graph (if autoscale!), so need to
value obtained by fl_get_xyplot_bounds()
*/
/* get the autoscale parameters */
fl_get_xyplot_xbounds(ob, &fxmin, &fxmax);
fl_get_xyplot_ybounds(ob, &fymin, &fymax);

clicked_x_pos = (int)fx + (int) fxmin;

if(debug_flag)
	{
	fprintf(stdout, "ixyplot(): getxyplot= fx=%.2f fy=%.2f n=%d\n",\
	fx, fy, n);
	fprintf(stdout, "ixyplotcb(): clicked_x_pos=%d\n",\
	clicked_x_pos);
	fprintf(stdout, "ixyplotcb(): xyplot x bounds=%.2f %.2f\n",\
	fxmin, fxmax);
	}

/* determine the month */
month[0] = '?';/* undefined */
month[1] = 0;/* string termination for undefined */
/* scan backward, future not in array */
for(i = clicked_x_pos; i >= 0; i--)
	{
/*
fprintf(stdout, "i=%d tx[i]=%f ttext[i]=%s\n",\
i, tx[i], ttext[i]);
*/
	if(tx[i] == -1)
		{
		strcpy(month, ttext[i]);
		break;
		}
	}/* end for all values in array (backwards) */	

/* determine the year */
if(file_type_flag == 0) year[0] = '?';/* undefined */
if(file_type_flag == 1) year[0] = 0;
year[1] = 0;/* string termination for undefined */

/* scan backward, future not in array */
for(i = clicked_x_pos; i >= 0; i--)
	{
/*
fprintf(stdout, "i=%d tx[i]=%f ttext[i]=%s\n",\
i, tx[i], ttext[i]);
*/
	if(tx[i] == -3)
		{
		strcpy(year, ttext[i]);
		break;
		}
	}/* end for all values in array (backwards) */	

fy = y[clicked_x_pos];
sprintf(temp, "day -%d  %s  %s  price %4.2f",\
end - clicked_x_pos, month, year, fy);

fl_set_object_label(fdui -> outfield, temp);

/*
fl_set_object_lstyle(fdui -> outfield, FL_BOLD_STYLE);
fl_set_object_lcol(fdui -> outfield, FL_YELLOW);
*/

/* calculate position where to put text */
/*
ftextx = (fxmax - fxmin ) / 2;
ftexty = fymax - ( (fymax - fymin) / 10); 
*/

/* print the info text */
/*
fl_add_xyplot_text(ob, ftextx, ftexty, temp, FL_ALIGN_LEFT_TOP, FL_BLUE);
*/
}/* end function ixyplot_cb */


/* signal handler */
void print_new_york_time(int sig)
{
extern char *mydata;
char temp[512];
new_york_time();
fl_set_object_label(fdui -> new_york_clock, ny_time );
fl_add_signal_callback(SIGALRM, (FL_SIGNAL_HANDLER)print_new_york_time, &mydata);

if(update_flag)
	{
	/* add data aquisition here */
	if( (time(0) - last_update_time) >= update_minutes * 60)
		{
/*
fprintf(stdout, "UPDATING\n");	
*/
		/* read new data into the file */
		fl_set_object_label(fdui -> outfield, "UPDATING");
		if(acquire_data(fund_name) )
			{
			last_update_time = time(0);
			/* load and display the new data */
			if( load_file(0) )
				{
				print_graph();
				}
			}
		else
			{
			sprintf(temp, "Cannot update %s", fund_name);		
			fl_set_object_label(fdui -> outfield, temp);
			}
		}	
	}/* end if update_flag */

/* reenable the timer */
alarm(1);
}/* end function print_new_york_time (signal handler) */


new_york_time()
{
time_t now;
struct tm *broken_time;

now = time(0) + 3600 * new_york_offset;

broken_time = localtime (&now);

sprintf(ny_time, "%02d:%02d.%02d",\
broken_time -> tm_hour, broken_time -> tm_min, broken_time -> tm_sec);

return;
}/* end function new_york_time */


acquire_data(char *fundname)/* this may be done by an other program */
{
time_t now;
struct tm *broken_time;
int open_hour;
int open_minute;
int close_hour;
int close_minute;

open_hour = 9;
open_minute = 0;
close_hour = 16;
close_minute = 45;

now = time(0);
broken_time = localtime (&now);


if\
(
(broken_time -> tm_hour >= open_hour) &&\
(broken_time -> tm_min >= open_minute) &&\
(broken_time -> tm_hour <= close_hour) &&\
(broken_time -> tm_min <= close_minute)\
)
	{
	}
return(1);
}/* end function acquire _data */


int autotrade(int crosstype)
{
int a;

if(crosstype == GOLD_CROSS)
	{
	}	
else if(crosstype == DEAD_CROSS)
	{
	}
return(1);
}/* end function auto_trade */


int idle_cb(XEvent *ev, void *data)
{
int a;
char temp[512];

if(optimize_flag)
	{
	if(! print_graph() )
		{
		optimize_flag = 0;
		return;
		}
	
	sprintf(temp, "Testing madays1=%d madays2=%d",\
	maday1, maday2);
	fl_set_object_label(fdui -> outfield, temp);
	
/*
sprintf(temp, "%d", maday1);
fl_set_input(fdui -> ma1_input, temp);
sprintf(temp, "%d", maday2);
fl_set_input(fdui -> ma2_input, temp);
*/

	if(gain > old_gain)
		{
		old_mawei1 = mawei1;
		old_maday1 = maday1;
		old_maday2 = maday2;
		old_momentum_days = momentum_days;
		old_gain = gain;

		sprintf(temp, "best gain=%4.1f at ma1=%d ma2=%d",\
		old_gain, old_maday1, old_maday2);
		fl_set_object_label(fdui -> volumefield, temp);

		sprintf(temp, "%d", maday1);
		fl_set_input(fdui -> ma1_input, temp);

		sprintf(temp, "%d", maday2);
		fl_set_input(fdui -> ma2_input, temp);

		/* show the result */
/*		print_graph(0);*/
		}/* end if gain > old_gain */

	maday2++;
	if( (maday2 >= MAX_MADAY2) || (maday2 >= maday1) )
		{
		maday2 = 1;
		maday1++;
		if(maday1 >= MAX_MADAY1)
			{
			/* release optimize button */
			fl_set_button(fdui -> optimize_button, 0);
			optimize_flag = 0;
			return;
			}
		}
	return;
	}/* end if optimize_flag */

return;
}/* end function idle_cb */


int test_for_open_forms(int *some_form_open_flag)
{
*some_form_open_flag = 0;

if(portfolio_form_shown_flag) 
	{
	fl_show_alert("Close portfolio form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(share_rates_form_shown_flag) 
	{
	fl_show_alert("Close shares rates form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(option_rates_form_shown_flag) 
	{
	fl_show_alert("Close option rates form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(option_calculator_form_shown_flag) 
	{
	fl_show_alert("Close options form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(transaction_add_form_shown_flag)
	{
	fl_show_alert(\
		"Close transaction add form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(option_evaluation_form_shown_flag) 
	{
	fl_show_alert(\
		"Close option evaluation  form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

return 1;
}/* end function test_for_open_forms */


void main_form_window_menu_cb(FL_OBJECT *ob, long data)
{
int main_form_window_menu_selection;

main_form_window_menu_selection = fl_get_menu(ob);

/* if nothing selected, but mouse in menu, no action */
if(main_form_window_menu_selection <= 0) return;

if(main_form_window_menu_selection == PORTFOLIO_MANAGEMENT)
	{
	show_portfolio_form();

	portfolio_form_shown_flag = 1;
	return;
	}/* end if PORTFOLIO_MANAGEMENT */
else if(main_form_window_menu_selection == OPTION_CALCULATOR)
	{
	fl_show_form(\
		fdui -> option_calculator_form, FL_PLACE_CENTER, FL_UP_FRAME, "");

	option_calculator_form_shown_flag = 1;
	return;
	}/* end if OPTION_CALCULATOR */
else if(main_form_window_menu_selection == SHARE_RATES)
	{
	read_share_rates();

	fl_show_form(\
		fdui -> share_rates_form, FL_PLACE_CENTER, FL_NOBORDER, "");

	share_rates_form_shown_flag = 1;

	print_graph();
	}/* end if SHARE_RATES */
else if(main_form_window_menu_selection == OPTION_RATES)
	{
	read_option_rates();

	fl_show_form(\
		fdui -> option_rates_form, FL_PLACE_CENTER, FL_NOBORDER, "");

	option_rates_form_shown_flag = 1;

	print_graph();
	}/* end if OPTION_RATES */
else if(main_form_window_menu_selection == OPTION_EVALUATOR)
	{
	show_option_evaluation_form();
	
	option_evaluation_form_shown_flag = 1;
	}/* end if OPTION_EVALUATOR */

}/* end function main_form_window_menu_cb */

